@@ -2,7 +2,7 @@
  *
  * $Id: classdef.h,v 1.39 2001/03/19 19:27:39 root Exp $
  *
- * Copyright (C) 1997-2011 by Dimitri van Heesch.
+ * Copyright (C) 1997-2012 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -21,6 +21,7 @@
 #include "qtbc.h"
 #include <qlist.h>
 #include <qdict.h>
+#include <qptrdict.h>
 #include <qstrlist.h>
 
 #include "util.h"
@@ -47,7 +48,7 @@ class StringDict;
 struct IncludeInfo;
 class ClassDefImpl;
 
-/*! \brief This class contains all information about a compound.
+/** A model of a compound symbol.
  *
  *  A compound can be a class, struct, union, interface, or exception.
  *  \note This class should be renamed to CompoundDef
@@ -55,7 +56,7 @@ class ClassDefImpl;
 class ClassDef : public Definition
 {
   public:
-    /*! The various compound types */
+    /** The various compound types */
     enum CompoundType { Class, //=Entry::CLASS_SEC, 
                         Struct, //=Entry::STRUCT_SEC, 
                         Union, //=Entry::UNION_SEC,
@@ -65,7 +66,7 @@ class ClassDef : public Definition
                         Exception //=Entry::EXCEPTION_SEC
                       };
 
-    /*! Creates a new compound definition.
+    /** Creates a new compound definition.
      *  \param fileName  full path and file name in which this compound was
      *                   found.
      *  \param startLine line number where the definition of this compound
@@ -79,127 +80,129 @@ class ClassDef : public Definition
      *                   generates based on the compound type & name.
      *  \param isSymbol  If TRUE this class name is added as a publicly 
      *                   visible (and referencable) symbol.
+     *  \param isJavaEnum If TRUE this class is actually a Java enum.
+     *                    I didn't add this to CompoundType to avoid having
+     *                    to adapt all translators.
      */
     ClassDef(const char *fileName,int startLine,
              const char *name,CompoundType ct,
              const char *ref=0,const char *fName=0,
-             bool isSymbol=TRUE);
-    /*! Destroys a compound definition. */
+             bool isSymbol=TRUE,bool isJavaEnum=FALSE);
+    /** Destroys a compound definition. */
    ~ClassDef();
 
     //-----------------------------------------------------------------------------------
     // --- getters 
     //-----------------------------------------------------------------------------------
 
-    /*! Used for RTTI, this is a class */
+    /** Used for RTTI, this is a class */
     DefType definitionType() const { return TypeClass; }
 
-    /*! Returns the unique base name (without extension) of the class's file on disk */
-    QCString getOutputFileBase() const; 
-    QCString getXmlOutputFileBase() const;
+    /** Returns the unique base name (without extension) of the class's file on disk */
+    QCString getOutputFileBase() const;
     QCString getInstanceOutputFileBase() const; 
     QCString getFileBase() const;
 
-    /*! Returns the base name for the source code file */
+    /** Returns the base name for the source code file */
     QCString getSourceFileBase() const; 
 
-    /*! If this class originated from a tagfile, this will return the tag file reference */
+    /** If this class originated from a tagfile, this will return the tag file reference */
     QCString getReference() const;
 
-    /*! Returns TRUE if this class is imported via a tag file */
+    /** Returns TRUE if this class is imported via a tag file */
     bool isReference() const;
 
-    /*! Returns TRUE if this is a local class definition, see EXTRACT_LOCAL_CLASSES */
+    /** Returns TRUE if this is a local class definition, see EXTRACT_LOCAL_CLASSES */
     bool isLocal() const;
 
-    /*! returns the classes nested into this class */
-    ClassSDict *getInnerClasses();
+    /** returns the classes nested into this class */
+    ClassSDict *getClassSDict();
 
-    /*! returns TRUE if this class has documentation */
+    /** returns TRUE if this class has documentation */
     bool hasDocumentation() const;
 
-    /*! Returns the name as it is appears in the documentation */
-    QCString displayName() const;
+    /** Returns the name as it is appears in the documentation */
+    QCString displayName(bool includeScope=TRUE) const;
 
-    /*! Returns the type of compound this is, i.e. class/struct/union/.. */
+    /** Returns the type of compound this is, i.e. class/struct/union/.. */
     CompoundType compoundType() const;
 
-    /*! Returns the type of compound as a string */
+    /** Returns the type of compound as a string */
     QCString compoundTypeString() const;
 
-    /*! Returns the list of base classes from which this class directly
+    /** Returns the list of base classes from which this class directly
      *  inherits.
      */
     BaseClassList *baseClasses() const;
     
-    /*! Returns the list of sub classes that directly derive from this class
+    /** Returns the list of sub classes that directly derive from this class
      */
     BaseClassList *subClasses() const;
 
-    /*! Returns a dictionary of all members. This includes any inherited 
+    /** Returns a dictionary of all members. This includes any inherited 
      *  members. Members are sorted alphabetically.
      */ 
     MemberNameInfoSDict *memberNameInfoSDict() const;
 
-    /*! Return the protection level (Public,Protected,Private) in which 
+    /** Return the protection level (Public,Protected,Private) in which 
      *  this compound was found.
      */
     Protection protection() const;
 
-    /*! returns TRUE iff a link is possible to this item within this project.
+    /** returns TRUE iff a link is possible to this item within this project.
      */
     bool isLinkableInProject() const;
 
-    /*! return TRUE iff a link to this class is possible (either within 
+    /** return TRUE iff a link to this class is possible (either within 
      *  this project, or as a cross-reference to another project).
      */
     bool isLinkable() const;
 
-    /*! the class is visible in a class diagram, or class hierarchy */
+    /** the class is visible in a class diagram, or class hierarchy */
     bool isVisibleInHierarchy();
     
-    /*! Returns the template arguments of this class 
+    /** Returns the template arguments of this class 
      *  Will return 0 if not applicable.
      */
     ArgumentList *templateArguments() const;
 
-    /*! Returns the namespace this compound is in, or 0 if it has a global
+    /** Returns the namespace this compound is in, or 0 if it has a global
      *  scope.
      */
     NamespaceDef *getNamespaceDef() const;
 
-    /*! Returns the file in which this compound's definition can be found.
+    /** Returns the file in which this compound's definition can be found.
      *  Should not return 0 (but it might be a good idea to check anyway).
      */
     FileDef      *getFileDef() const;
 
-    /*! Returns the Java package this class is in or 0 if not applicable. 
+    /** Returns the Java package this class is in or 0 if not applicable. 
      */ 
 
     MemberDef    *getMemberByName(const QCString &) const;
     
-    /*! Returns TRUE iff \a bcd is a direct or indirect base class of this
+    /** Returns TRUE iff \a bcd is a direct or indirect base class of this
      *  class. This function will recusively traverse all branches of the
      *  inheritance tree.
      */
     bool isBaseClass(ClassDef *bcd,bool followInstances,int level=0);
 
-    /*! returns TRUE iff \a md is a member of this class or of the
+    /** returns TRUE iff \a md is a member of this class or of the
      *  the public/protected members of a base class 
      */
     bool isAccessibleMember(MemberDef *md);
 
-    /*! Returns a sorted dictionary with all template instances found for
+    /** Returns a sorted dictionary with all template instances found for
      *  this template class. Returns 0 if not a template or no instances.
      */
     QDict<ClassDef> *getTemplateInstances() const;
 
-    /*! Returns the template master of which this class is an instance.
+    /** Returns the template master of which this class is an instance.
      *  Returns 0 if not applicable.
      */
     ClassDef *templateMaster() const;
 
-    /*! Returns TRUE if this class is a template */
+    /** Returns TRUE if this class is a template */
     bool isTemplate() const;
 
     IncludeInfo *includeInfo() const;
@@ -212,13 +215,13 @@ class ClassDef : public Definition
 
     bool isTemplateArgument() const;
 
-    /*! Returns the definition of a nested compound if
+    /** Returns the definition of a nested compound if
      *  available, or 0 otherwise.
      *  @param name The name of the nested compound
      */
     virtual Definition *findInnerCompound(const char *name);
 
-    /*! Returns the template parameter lists that form the template
+    /** Returns the template parameter lists that form the template
      *  declaration of this class.
      *  
      *  Example: <code>template<class T> class TC {};</code>
@@ -230,32 +233,41 @@ class ClassDef : public Definition
     QCString qualifiedNameWithTemplateParameters(
         QList<ArgumentList> *actualParams=0) const;
 
-    /*! Returns TRUE if there is at least one pure virtual member in this
+    /** Returns TRUE if there is at least one pure virtual member in this
      *  class.
      */
     bool isAbstract() const;
 
-    /*! Returns TRUE if this class is implemented in Objective-C */
+    /** Returns TRUE if this class is implemented in Objective-C */
     bool isObjectiveC() const;
 
-    /*! Returns TRUE if this class is implemented in C# */
+    /** Returns TRUE if this class is implemented in C# */
     bool isCSharp() const;
 
-    /*! Returns the class of which this is a category (Objective-C only) */
+    /** Returns TRUE if this class is marked as final */
+    bool isFinal() const;
+
+    /** Returns TRUE if this class is marked as sealed */
+    bool isSealed() const;
+
+    /** Returns TRUE if this class represents an Objective-C 2.0 extension (nameless category) */
+    bool isExtension() const;
+
+    /** Returns the class of which this is a category (Objective-C only) */
     ClassDef *categoryOf() const;
 
-    /*! Returns the name of the class including outer classes, but not
+    /** Returns the name of the class including outer classes, but not
      *  including namespaces.
      */
     QCString className() const;
 
-    /*! Returns the members in the list identified by \a lt */
+    /** Returns the members in the list identified by \a lt */
     MemberList *getMemberList(MemberList::ListType lt);
 
-    /*! Returns the list containing the list of members sorted per type */
+    /** Returns the list containing the list of members sorted per type */
     const QList<MemberList> &getMemberLists() const;
 
-    /*! Returns the member groups defined for this class */
+    /** Returns the member groups defined for this class */
     MemberGroupSDict *getMemberGroupSDict() const;
 
     QDict<int> *getTemplateBaseClassNames() const;
@@ -272,6 +284,12 @@ class ClassDef : public Definition
     const ClassList *taggedInnerClasses() const;
     ClassDef *tagLessReference() const;
 
+    MemberDef *isSmartPointer() const;
+
+    bool isJavaEnum() const;
+
+    bool isGeneric() const;
+
     //-----------------------------------------------------------------------------------
     // --- setters ----
     //-----------------------------------------------------------------------------------
@@ -296,6 +314,7 @@ class ClassDef : public Definition
     void setIsStatic(bool b);
     void setCompoundType(CompoundType t);
     void setClassName(const char *name);
+    void setClassSpecifier(int spec);
 
     void setTemplateArguments(ArgumentList *al);
     void setTemplateBaseClassNames(QDict<int> *templateNames);
@@ -324,7 +343,8 @@ class ClassDef : public Definition
     void writeDocumentationForInnerClasses(OutputList &ol);
     void writeMemberPages(OutputList &ol);
     void writeMemberList(OutputList &ol);
-    void writeDeclaration(OutputList &ol,MemberDef *md,bool inGroup);
+    void writeDeclaration(OutputList &ol,MemberDef *md,bool inGroup,
+                          ClassDef *inheritedFrom,const char *inheritId);
     void writeQuickMemberLinks(OutputList &ol,MemberDef *md) const;
     void writeSummaryLinks(OutputList &ol);
     void reclassifyMember(MemberDef *md,MemberDef::MemberType t);
@@ -332,6 +352,9 @@ class ClassDef : public Definition
     void writeDeclarationLink(OutputList &ol,bool &found,
                               const char *header,bool localNames);
     void removeMemberFromLists(MemberDef *md);
+    void addGroupedInheritedMembers(OutputList &ol,MemberList::ListType lt,
+                              ClassDef *inheritedFrom,const QCString &inheritId);
+    int countMembersIncludingGrouped(MemberList::ListType lt,ClassDef *inheritedFrom,bool additional);
     
     bool visited;
 
@@ -342,22 +365,23 @@ class ClassDef : public Definition
     void showUsedFiles(OutputList &ol);
 
   private: 
-    void writeTagFileMarker(OutputList &ol);
+    void writeTagFileMarker();
     void writeDocumentationContents(OutputList &ol,const QCString &pageTitle);
     void internalInsertMember(MemberDef *md,Protection prot,bool addToAllList);
     QCString getMemberListFileName() const;
     void addMemberToList(MemberList::ListType lt,MemberDef *md,bool isBrief);
     MemberList *createMemberList(MemberList::ListType lt);
+    void writeInheritedMemberDeclarations(OutputList &ol,MemberList::ListType lt,const QCString &title,ClassDef *inheritedFrom,bool invert,QPtrDict<void> *visitedClasses);
     void writeMemberDeclarations(OutputList &ol,MemberList::ListType lt,const QCString &title,
-                                 const char *subTitle=0,bool showInline=FALSE);
+                                 const char *subTitle=0,bool showInline=FALSE,ClassDef *inheritedFrom=0,int lt2=-1,bool invert=FALSE,QPtrDict<void> *visitedClasses=0);
     void writeMemberDocumentation(OutputList &ol,MemberList::ListType lt,const QCString &title,bool showInline=FALSE);
     void writeSimpleMemberDocumentation(OutputList &ol,MemberList::ListType lt);
-    void writePlainMemberDeclaration(OutputList &ol,MemberList::ListType lt,bool inGroup);
+    void writePlainMemberDeclaration(OutputList &ol,MemberList::ListType lt,bool inGroup,ClassDef *inheritedFrom,const char *inheritId);
     void writeBriefDescription(OutputList &ol,bool exampleFlag);
     void writeDetailedDescription(OutputList &ol,const QCString &pageType,bool exampleFlag,
                                   const QCString &title,const QCString &anchor=QCString());
     void writeIncludeFiles(OutputList &ol);
-    void writeAllMembersLink(OutputList &ol);
+    //void writeAllMembersLink(OutputList &ol);
     void writeInheritanceGraph(OutputList &ol);
     void writeCollaborationGraph(OutputList &ol);
     void writeMemberGroups(OutputList &ol,bool showInline=FALSE);
@@ -371,11 +395,17 @@ class ClassDef : public Definition
     void writeMoreLink(OutputList &ol,const QCString &anchor);
     void writeDetailedDocumentationBody(OutputList &ol);
     
+    int countInheritedDecMembersRec(MemberList::ListType lt,ClassDef *inheritedFrom);
+    int countInheritedDecMembers(MemberList::ListType lt);
+    int countAdditionalInheritedMembers();
+    void writeAdditionalInheritedMembers(OutputList &ol);
+    void addClassAttributes(OutputList &ol);
+    
     ClassDefImpl *m_impl;
 
 };
 
-/*! \brief Class that contains information about a usage relation. 
+/** Class that contains information about a usage relation. 
  */
 struct UsesClassDef
 {
@@ -395,21 +425,21 @@ struct UsesClassDef
       accessors->insert(s,(void *)666);
     }
   }
-  /*! Class definition that this relation uses. */
+  /** Class definition that this relation uses. */
   ClassDef *classDef;
 
-  /*! Dictionary of member variable names that form the edge labels of the
+  /** Dictionary of member variable names that form the edge labels of the
    *  usage relation.
    */
   QDict<void> *accessors;
 
-  /*! Template arguments used for the base class */
+  /** Template arguments used for the base class */
   QCString templSpecifiers;
 
   bool containment;
 };
 
-/*! \brief Dictionary of usage relations. 
+/** Dictionary of usage relations. 
  */
 class UsesClassDict : public QDict<UsesClassDef>
 {
@@ -418,7 +448,7 @@ class UsesClassDict : public QDict<UsesClassDef>
    ~UsesClassDict() {}
 };
 
-/*! \brief Iterator class to iterate over a dictionary of usage relations. 
+/** Iterator class to iterate over a dictionary of usage relations. 
  */
 class UsesClassDictIterator : public QDictIterator<UsesClassDef>
 {
@@ -428,7 +458,7 @@ class UsesClassDictIterator : public QDictIterator<UsesClassDef>
    ~UsesClassDictIterator() {}
 };
 
-/*! \brief Class that contains information about an inheritance relation. 
+/** Class that contains information about an inheritance relation. 
  */
 struct BaseClassDef
 {
@@ -436,29 +466,29 @@ struct BaseClassDef
                Specifier v,const char *t) : 
         classDef(cd), usedName(n), prot(p), virt(v), templSpecifiers(t) {}
 
-  /*! Class definition that this relation inherits from. */
+  /** Class definition that this relation inherits from. */
   ClassDef *classDef;
 
-  /*! name used in the inheritance list 
+  /** name used in the inheritance list 
    * (may be a typedef name instead of the class name)
    */
   QCString   usedName; 
   
-  /*! Protection level of the inheritance relation: 
+  /** Protection level of the inheritance relation: 
    *  Public, Protected, or Private 
    */
   Protection prot;     
 
-  /*! Virtualness of the inheritance relation:
+  /** Virtualness of the inheritance relation:
    *  Normal, or Virtual
    */
   Specifier  virt;
 
-  /*! Template arguments used for the base class */
+  /** Template arguments used for the base class */
   QCString templSpecifiers;
 };
 
-/*! \brief list of base classes 
+/** List of base classes.
  *  
  *  The classes are alphabetically sorted on name if inSort() is used.
  */
@@ -477,7 +507,7 @@ class BaseClassList : public QList<BaseClassDef>
     }
 };
 
-/*! \brief Iterator for a list of base classes
+/** Iterator for a list of base classes.
  */
 class BaseClassListIterator : public QListIterator<BaseClassDef>
 {
