@@ -2,7 +2,7 @@
  *
  * $Id: code.h,v 1.9 2001/03/19 19:27:39 root Exp $
  *
- * Copyright (C) 1997-2007 by Dimitri van Heesch.
+ * Copyright (C) 1997-2012 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -26,9 +26,9 @@ class FileDef;
 class MemberDef;
 
 void parseCCode(CodeOutputInterface &,const char *,const QCString &, 
-            bool ,const char *,FileDef *fd=0,
-            int startLine=-1,int endLine=-1,bool inlineFragment=FALSE,
-            MemberDef *memberDef=0);
+            bool ,const char *,FileDef *fd,
+            int startLine,int endLine,bool inlineFragment,
+            MemberDef *memberDef,bool showLineNumbers,Definition *searchCtx);
 void resetCCodeParserState();
 void codeFreeScanner();
 
