@@ -3,7 +3,7 @@
  * $Id: diagram.cpp,v 1.30 2001/03/19 19:27:40 root Exp $
  *
  *
- * Copyright (C) 1997-2007 by Dimitri van Heesch.
+ * Copyright (C) 1997-2012 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -21,7 +21,7 @@
 #include <stdlib.h>
 #include <qlist.h>
 #include <qarray.h>
-#include <qtextstream.h>
+#include "ftextstream.h"
 #include <qfile.h>
 
 #include "diagram.h"
@@ -30,10 +30,111 @@
 #include "config.h"
 #include "message.h"
 #include "util.h"
-//#include "latexgen.h"
-//#include "htmlgen.h"
 #include "doxygen.h"
 #include "portable.h"
+#include "index.h"
+
+//-----------------------------------------------------------------------------
+
+class DiagramItemList;
+
+/** Class representing a single node in the built-in class diagram */
+class DiagramItem 
+{
+  public:
+    DiagramItem(DiagramItem *p,int number,ClassDef *cd,
+                Protection prot,Specifier virt,const char *ts);
+   ~DiagramItem();
+    QCString label() const;
+    QCString fileName() const;
+    DiagramItem *parentItem() { return parent; } 
+    DiagramItemList *getChildren() { return children; }
+    void move(int dx,int dy) { x+=dx; y+=dy; }
+    int xPos() const { return x; }
+    int yPos() const { return y; }
+    int avgChildPos() const;
+    int numChildren() const;
+    void addChild(DiagramItem *di);
+    int number() const { return num; }
+    Protection protection() const { return prot; }
+    Specifier virtualness() const { return virt; }
+    void putInList() { inList=TRUE; }
+    bool isInList() const { return inList; } 
+    ClassDef *getClassDef() const { return classDef; }
+  private:
+    DiagramItemList *children;
+    DiagramItem *parent;
+    int x,y;
+    int num;
+    Protection prot;
+    Specifier virt;
+    QCString templSpec;
+    bool inList;
+    ClassDef *classDef;
+};
+
+/** Class representing a list of DiagramItem object. */
+class DiagramItemList : public QList<DiagramItem>
+{
+  public:
+    DiagramItemList() : QList<DiagramItem>() {}
+   ~DiagramItemList() {}
+};
+
+/** Class representing a row in the built-in class diagram */
+class DiagramRow : public QList<DiagramItem> 
+{
+  public:
+    DiagramRow(TreeDiagram *d,int l) : QList<DiagramItem>() 
+    { 
+      diagram=d; 
+      level=l;
+      setAutoDelete(TRUE); 
+    }
+    void insertClass(DiagramItem *parent,ClassDef *cd,bool doBases,
+                     Protection prot,Specifier virt,const char *ts);
+    uint number() { return level; }
+  private:
+    TreeDiagram *diagram;
+    uint level;
+};
+
+/** Class representing iterator for the rows in the built-in class diagram. */
+class DiagramRowIterator : public QListIterator<DiagramRow>
+{
+  public:
+    DiagramRowIterator(const QList<DiagramRow> &d) 
+      : QListIterator<DiagramRow>(d) {}
+};
+
+/** Class represeting the tree layout for the built-in class diagram. */
+class TreeDiagram : public QList<DiagramRow>
+{
+  public:
+    TreeDiagram(ClassDef *root,bool doBases);
+   ~TreeDiagram();
+    void computeLayout();
+    uint computeRows();
+    //uint computeCols();
+    void moveChildren(DiagramItem *root,int dx);
+    void computeExtremes(uint *labelWidth,uint *xpos);
+    void drawBoxes(FTextStream &t,Image *image,
+                   bool doBase,bool bitmap,
+                   uint baseRows,uint superRows,
+                   uint cellWidth,uint cellHeight,
+                   QCString relPath="",
+                   bool generateMap=TRUE);
+    void drawConnectors(FTextStream &t,Image *image,
+                   bool doBase,bool bitmap,
+                   uint baseRows,uint superRows,
+                   uint cellWidth,uint cellheight);
+  private:
+    bool layoutTree(DiagramItem *root,int row);
+    TreeDiagram &operator=(const TreeDiagram &);
+    TreeDiagram(const TreeDiagram &);
+};
+
+
 
 //-----------------------------------------------------------------------------
 
@@ -143,7 +244,7 @@ static void writeBitmapBox(DiagramItem *di,Image *image,
   }
 }
 
-static void writeVectorBox(QTextStream &t,DiagramItem *di,
+static void writeVectorBox(FTextStream &t,DiagramItem *di,
                            float x,float y,bool children=FALSE)
 {
   if (di->virtualness()==Virtual) t << "dashed\n";
@@ -152,33 +253,33 @@ static void writeVectorBox(QTextStream &t,DiagramItem *di,
   if (di->virtualness()==Virtual) t << "solid\n";
 }
 
-static void writeMapArea(QTextStream &t,ClassDef *cd,QCString relPath,
+static void writeMapArea(FTextStream &t,ClassDef *cd,QCString relPath,
                          int x,int y,int w,int h)
 {
   if (cd->isLinkable())
   {
-    QCString *dest;
     QCString ref=cd->getReference();
     t << "<area ";
     if (!ref.isEmpty()) 
     {
-      t << "doxygen=\"" << ref << ":";
-      if ((dest=Doxygen::tagDestinationDict[ref])) t << *dest << "/";
-      t << "\" ";
+      t << externalLinkTarget() << externalRef(relPath,ref,FALSE);
     }
     t << "href=\"";
-    if (!ref.isEmpty())
+    t << externalRef(relPath,ref,TRUE);
+    t << cd->getOutputFileBase() << Doxygen::htmlFileExtension;
+    if (!cd->anchor().isEmpty())
     {
-      if ((dest=Doxygen::tagDestinationDict[ref])) t << *dest << "/";
+      t << "#" << cd->anchor();
     }
-    else
+    t << "\" ";
+    QCString tooltip = cd->briefDescriptionAsTooltip();
+    if (!tooltip.isEmpty())
     {
-      t << relPath;
+      t << "title=\"" << tooltip << "\" ";
     }
-    t << cd->getOutputFileBase() << Doxygen::htmlFileExtension << "\" ";
-    t << "alt=\"" << cd->displayName(); 
+    t << "alt=\"" << convertToXML(cd->displayName()); 
     t << "\" shape=\"rect\" coords=\"" << x << "," << y << ",";
-    t << (x+w) << "," << (y+h) << "\">" << endl;
+    t << (x+w) << "," << (y+h) << "\"/>" << endl;
   }
 }
 //-----------------------------------------------------------------------------
@@ -208,7 +309,14 @@ QCString DiagramItem::label() const
   QCString result;
   if (!templSpec.isEmpty())
   {
-    result=insertTemplateSpecifierInScope(classDef->displayName(),templSpec);
+    // we use classDef->name() here and not diplayName() in order
+    // to get the name used in the inheritance relation.
+    QCString n = classDef->name();
+    if (/*n.right(2)=="-g" ||*/ n.right(2)=="-p")
+    {
+      n = n.left(n.length()-2);
+    }
+    result=insertTemplateSpecifierInScope(n,templSpec);
   }
   else
   {
@@ -358,7 +466,7 @@ bool TreeDiagram::layoutTree(DiagramItem *root,int r)
     DiagramItem *di=dil->first();
     while (di && !moved && !di->isInList())
     {
-      moved = moved || layoutTree(di,r+1);
+      moved = layoutTree(di,r+1);
       di=dil->next();
     }
   }
@@ -391,7 +499,7 @@ void TreeDiagram::computeLayout()
 
   // re-organize the diagram items
   DiagramItem *root=getFirst()->getFirst();
-  while (layoutTree(root,0));
+  while (layoutTree(root,0)) { }
   
   // move first items of the lists
   if (row)
@@ -484,7 +592,7 @@ void TreeDiagram::computeExtremes(uint *maxLabelLen,uint *maxXPos)
   if (maxXPos)     *maxXPos=mx;
 }
 
-void TreeDiagram::drawBoxes(QTextStream &t,Image *image, 
+void TreeDiagram::drawBoxes(FTextStream &t,Image *image, 
                             bool doBase,bool bitmap,
                             uint baseRows,uint superRows,
                             uint cellWidth,uint cellHeight,
@@ -615,7 +723,7 @@ void TreeDiagram::drawBoxes(QTextStream &t,Image *image,
   }
 }
 
-void TreeDiagram::drawConnectors(QTextStream &t,Image *image,
+void TreeDiagram::drawConnectors(FTextStream &t,Image *image,
                                  bool doBase,bool bitmap,
                                  uint baseRows,uint superRows,
                                  uint cellWidth,uint cellHeight)
@@ -967,7 +1075,7 @@ ClassDiagram::~ClassDiagram()
   delete super;
 }
 
-void ClassDiagram::writeFigure(QTextStream &output,const char *path,
+void ClassDiagram::writeFigure(FTextStream &output,const char *path,
                                const char *fileName) const
 {
   uint baseRows=base->computeRows();
@@ -998,7 +1106,7 @@ void ClassDiagram::writeFigure(QTextStream &output,const char *path,
   }
 
   //output << "}\n";
-  output << ":\\begin{figure}[H]\n"
+  output << "\\begin{figure}[H]\n"
             "\\begin{center}\n"
             "\\leavevmode\n";
   output << "\\includegraphics[height=" << realHeight << "cm]{" 
@@ -1017,7 +1125,7 @@ void ClassDiagram::writeFigure(QTextStream &output,const char *path,
     err("Could not open file %s for writing\n",convertToQCString(f1.name()).data());
     exit(1);
   }
-  QTextStream t(&f1);
+  FTextStream t(&f1);
   
   //printf("writeEPS() rows=%d cols=%d\n",rows,cols);
   
@@ -1252,16 +1360,19 @@ void ClassDiagram::writeFigure(QTextStream &output,const char *path,
     epstopdfArgs.sprintf("\"%s.eps\" --outfile=\"%s.pdf\"",
                    epsBaseName.data(),epsBaseName.data());
     //printf("Converting eps using `%s'\n",epstopdfCmd.data());
+    portable_sysTimerStart();
     if (portable_system("epstopdf",epstopdfArgs)!=0)
     {
-       err("Error: Problems running epstopdf. Check your TeX installation!\n");
+       err("error: Problems running epstopdf. Check your TeX installation!\n");
+       portable_sysTimerStop();
        return;
     }
+    portable_sysTimerStop();
   }
 }
 
 
-void ClassDiagram::writeImage(QTextStream &t,const char *path,
+void ClassDiagram::writeImage(FTextStream &t,const char *path,
                               const char *relPath,const char *fileName, 
                               bool generateMap) const
 {
@@ -1288,7 +1399,9 @@ void ClassDiagram::writeImage(QTextStream &t,const char *path,
   base->drawConnectors(t,&image,TRUE,TRUE,baseRows,superRows,cellWidth,cellHeight);
   super->drawConnectors(t,&image,FALSE,TRUE,baseRows,superRows,cellWidth,cellHeight);
 
-  image.save((QCString)path+"/"+fileName+".png");
+#define IMAGE_EXT ".png"
+  image.save((QCString)path+"/"+fileName+IMAGE_EXT);
+  Doxygen::indexList.addImageFile(QCString(fileName)+IMAGE_EXT);
   
   if (generateMap) t << "</map>" << endl;
 }
