@@ -1,6 +1,6 @@
 /******************************************************************************
  *
- * Copyright (C) 1997-2011 by Dimitri van Heesch.
+ * Copyright (C) 1997-2012 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -13,12 +13,11 @@
  *
  */
 
+#include <qfile.h>
 #include "docsets.h"
 #include "config.h"
 #include "message.h"
 #include "doxygen.h"
-#include <qfile.h>
-
 
 DocSets::DocSets() : m_nodes(17), m_scopes(17)
 {
@@ -48,6 +47,8 @@ void DocSets::initialize()
   if (publisherId.isEmpty()) publisherId="PublisherId";
   QCString publisherName = Config_getString("DOCSET_PUBLISHER_NAME");
   if (publisherName.isEmpty()) publisherName="PublisherName";
+  QCString projectNumber = Config_getString("PROJECT_NUMBER");
+  if (projectNumber.isEmpty()) projectNumber="ProjectNumber";
 
   // -- write Makefile
   {
@@ -87,6 +88,9 @@ void DocSets::initialize()
         "\trm -f $(DOCSET_RESOURCES)/Nodes.xml\n"
         "\trm -f $(DOCSET_RESOURCES)/Tokens.xml\n"
         "\n"
+        "clean:\n"
+        "\trm -rf $(DOCSET_NAME)\n"
+        "\n"
         "install: docset\n"
         "\tmkdir -p $(DESTDIR)\n"
         "\tcp -R $(DOCSET_NAME) $(DESTDIR)\n"
@@ -116,7 +120,9 @@ void DocSets::initialize()
         "     <key>CFBundleName</key>\n" 
         "     <string>" << projectName << "</string>\n" 
         "     <key>CFBundleIdentifier</key>\n"
-        "     <string>" << bundleId << ".docset</string>\n" 
+        "     <string>" << bundleId << "</string>\n" 
+        "     <key>CFBundleVersion</key>\n"
+        "     <string>" << projectNumber << "</string>\n"
         "     <key>DocSetFeedName</key>\n" 
         "     <string>" << feedName << "</string>\n"
         "     <key>DocSetPublisherIdentifier</key>\n"
@@ -163,7 +169,10 @@ void DocSets::initialize()
 
 void DocSets::finalize()
 {
-  m_nts << indent() << " </Node>" << endl;
+  if (!m_firstNode.at(m_dc-1))
+  {
+    m_nts << indent() << " </Node>" << endl;
+  }
   m_dc--;
   m_nts << "      </Subnodes>" << endl;
   m_nts << "    </Node>" << endl;
@@ -188,6 +197,7 @@ QCString DocSets::indent()
 
 void DocSets::incContentsDepth()
 {
+  //printf("DocSets::incContentsDepth() m_dc=%d\n",m_dc);
   ++m_dc;
   m_nts << indent() << "<Subnodes>" << endl;
   m_firstNode.resize(m_dc);
@@ -205,16 +215,21 @@ void DocSets::decContentsDepth()
   }
   m_nts << indent() << "</Subnodes>" << endl;
   --m_dc;
+  //printf("DocSets::decContentsDepth() m_dc=%d\n",m_dc);
 }
 
 void DocSets::addContentsItem(bool isDir,
                               const char *name, 
                               const char *ref, 
                               const char *file,
-                              const char *anchor)
+                              const char *anchor,
+                              bool /* separateIndex */,
+                              bool /* addToNavIndex */,
+                              Definition * /*def*/)
 {
   (void)isDir;
-  if (file && ref==0)
+  //printf("DocSets::addContentsItem(%s) m_dc=%d\n",name,m_dc);
+  if (ref==0)
   {
     if (!m_firstNode.at(m_dc-1))
     {
@@ -223,10 +238,27 @@ void DocSets::addContentsItem(bool isDir,
     m_firstNode.at(m_dc-1)=FALSE;
     m_nts << indent() << " <Node>" << endl;
     m_nts << indent() << "  <Name>" << convertToXML(name) << "</Name>" << endl;
-    m_nts << indent() << "  <Path>" << file << Doxygen::htmlFileExtension << "</Path>" << endl;
-    if (anchor)
+    if (file && file[0]=='^') // URL marker
     {
-      m_nts << indent() << "  <Anchor>" << anchor << "</Anchor>" << endl;
+      m_nts << indent() << "  <URL>" << convertToXML(&file[1]) 
+            << "</URL>" << endl;
+    }
+    else // relative file
+    {
+      m_nts << indent() << "  <Path>";
+      if (file && file[0]=='!') // user specified file
+      {
+        m_nts << convertToXML(&file[1]);
+      }
+      else if (file) // doxygen generated file
+      {
+        m_nts << file << Doxygen::htmlFileExtension;
+      }
+      m_nts << "</Path>" << endl;
+      if (file && anchor)
+      {
+        m_nts << indent() << "  <Anchor>" << anchor << "</Anchor>" << endl;
+      }
     }
   }
 }
@@ -288,7 +320,8 @@ void DocSets::addIndexItem(Definition *context,MemberDef *md,const char *)
     case SrcLangExt_VHDL:    lang="vhdl"; break;       // VHDL
     case SrcLangExt_XML:     lang="xml"; break;        // DBUS XML
     case SrcLangExt_Tcl:     lang="tcl"; break;        // Tcl
-    case SrcLangExt_Unknown: lang="unknown"; break;   // should not happen!
+    case SrcLangExt_Markdown:lang="markdown"; break;   // Markdown
+    case SrcLangExt_Unknown: lang="unknown"; break;    // should not happen!
   }
 
   if (md)
@@ -362,7 +395,16 @@ void DocSets::addIndexItem(Definition *context,MemberDef *md,const char *)
     {
       scope = nd->name();
     }
-    writeToken(m_tts,md,type,lang,scope,md->anchor());
+    MemberDef *declMd = md->memberDeclaration();
+    if (declMd==0) declMd = md;
+    {
+      fd = md->getFileDef();
+      if (fd)
+      {
+        decl = fd->name();
+      }
+    }
+    writeToken(m_tts,md,type,lang,scope,md->anchor(),decl);
   }
   else if (context && context->isLinkable())
   {
@@ -439,7 +481,7 @@ void DocSets::writeToken(FTextStream &t,
 {
   t << "  <Token>" << endl;
   t << "    <TokenIdentifier>" << endl;
-  QString name = d->name();
+  QCString name = d->name();
   if (name.right(2)=="-p")  name=name.left(name.length()-2);
   t << "      <Name>" << convertToXML(name) << "</Name>" << endl;
   if (!lang.isEmpty())
