@@ -1,13 +1,12 @@
 "/* The standard CSS for doxygen */\n"
 "\n"
 "body, table, div, p, dl {\n"
-"	font-family: Lucida Grande, Verdana, Geneva, Arial, sans-serif;\n"
-"	font-size: 12px;\n"
+"	font: 400 14px/19px Roboto,sans-serif;\n"
 "}\n"
 "\n"
 "/* @group Heading Levels */\n"
 "\n"
-"h1 {\n"
+"h1.groupheader {\n"
 "	font-size: 150%;\n"
 "}\n"
 "\n"
@@ -17,14 +16,34 @@
 "	margin: 10px 2px;\n"
 "}\n"
 "\n"
-"h2 {\n"
-"	font-size: 120%;\n"
+"h2.groupheader {\n"
+"	border-bottom: 1px solid ##99;\n"
+"	color: ##44;\n"
+"	font-size: 150%;\n"
+"	font-weight: normal;\n"
+"	margin-top: 1.75em;\n"
+"	padding-top: 8px;\n"
+"	padding-bottom: 4px;\n"
+"	width: 100%;\n"
 "}\n"
 "\n"
-"h3 {\n"
+"h3.groupheader {\n"
 "	font-size: 100%;\n"
 "}\n"
 "\n"
+"h1, h2, h3, h4, h5, h6 {\n"
+"	-webkit-transition: text-shadow 0.5s linear;\n"
+"	-moz-transition: text-shadow 0.5s linear;\n"
+"	-ms-transition: text-shadow 0.5s linear;\n"
+"	-o-transition: text-shadow 0.5s linear;\n"
+"	transition: text-shadow 0.5s linear;\n"
+"	margin-right: 15px;\n"
+"}\n"
+"\n"
+"h1.glow, h2.glow, h3.glow, h4.glow, h5.glow, h6.glow {\n"
+"	text-shadow: 0 0 15px cyan;\n"
+"}\n"
+"\n"
 "dt {\n"
 "	font-weight: bold;\n"
 "}\n"
@@ -121,12 +140,12 @@
 "a.elRef {\n"
 "}\n"
 "\n"
-"a.code {\n"
-"	color: ##60;\n"
+"a.code, a.code:visited {\n"
+"	color: #4665A2; \n"
 "}\n"
 "\n"
-"a.codeRef {\n"
-"	color: ##60;\n"
+"a.codeRef, a.codeRef:visited {\n"
+"	color: #4665A2; \n"
 "}\n"
 "\n"
 "/* @end */\n"
@@ -135,20 +154,72 @@
 "	margin-left: -1cm;\n"
 "}\n"
 "\n"
-".fragment {\n"
-"	font-family: monospace, fixed;\n"
-"	font-size: 105%;\n"
+"pre.fragment {\n"
+"        border: 1px solid #C4CFE5;\n"
+"        background-color: #FBFCFD;\n"
+"        padding: 4px 6px;\n"
+"        margin: 4px 8px 4px 2px;\n"
+"        overflow: auto;\n"
+"        word-wrap: break-word;\n"
+"        font-size:  9pt;\n"
+"        line-height: 125%;\n"
+"        font-family: monospace, fixed;\n"
+"        font-size: 105%;\n"
 "}\n"
 "\n"
-"pre.fragment {\n"
-"	border: 1px solid ##CC;\n"
+"div.fragment {\n"
+"        padding: 4px;\n"
+"        margin: 4px;\n"
 "	background-color: ##FC;\n"
-"	padding: 4px 6px;\n"
-"	margin: 4px 8px 4px 2px;\n"
-"	overflow: auto;\n"
-"	word-wrap: break-word;\n"
-"	font-size:  9pt;\n"
-"	line-height: 125%;\n"
+"	border: 1px solid ##CC;\n"
+"}\n"
+"\n"
+"div.line {\n"
+"	font-family: monospace, fixed;\n"
+"        font-size: 13px;\n"
+"	min-height: 13px;\n"
+"	line-height: 1.0;\n"
+"	text-wrap: unrestricted;\n"
+"	white-space: -moz-pre-wrap; /* Moz */\n"
+"	white-space: -pre-wrap;     /* Opera 4-6 */\n"
+"	white-space: -o-pre-wrap;   /* Opera 7 */\n"
+"	white-space: pre-wrap;      /* CSS3  */\n"
+"	word-wrap: break-word;      /* IE 5.5+ */\n"
+"	text-indent: -53px;\n"
+"	padding-left: 53px;\n"
+"	padding-bottom: 0px;\n"
+"	margin: 0px;\n"
+"	-webkit-transition-property: background-color, box-shadow;\n"
+"	-webkit-transition-duration: 0.5s;\n"
+"	-moz-transition-property: background-color, box-shadow;\n"
+"	-moz-transition-duration: 0.5s;\n"
+"	-ms-transition-property: background-color, box-shadow;\n"
+"	-ms-transition-duration: 0.5s;\n"
+"	-o-transition-property: background-color, box-shadow;\n"
+"	-o-transition-duration: 0.5s;\n"
+"	transition-property: background-color, box-shadow;\n"
+"	transition-duration: 0.5s;\n"
+"}\n"
+"\n"
+"div.line.glow {\n"
+"	background-color: cyan;\n"
+"	box-shadow: 0 0 10px cyan;\n"
+"}\n"
+"\n"
+"\n"
+"span.lineno {\n"
+"	padding-right: 4px;\n"
+"	text-align: right;\n"
+"	border-right: 2px solid #0F0;\n"
+"	background-color: #E8E8E8;\n"
+"        white-space: pre;\n"
+"}\n"
+"span.lineno a {\n"
+"	background-color: #D8D8D8;\n"
+"}\n"
+"\n"
+"span.lineno a:hover {\n"
+"	background-color: #C8C8C8;\n"
 "}\n"
 "\n"
 "div.ah {\n"
@@ -188,7 +259,7 @@
 "\n"
 "div.contents {\n"
 "	margin-top: 10px;\n"
-"	margin-left: 8px;\n"
+"	margin-left: 12px;\n"
 "	margin-right: 8px;\n"
 "}\n"
 "\n"
@@ -198,6 +269,8 @@
 "	border: 1px solid ##cc;\n"
 "	margin: 2px 0px 2px 0;\n"
 "	padding: 2px 10px;\n"
+"        white-space: nowrap;\n"
+"        vertical-align: top;\n"
 "}\n"
 "\n"
 "td.indexvalue {\n"
@@ -290,6 +363,13 @@
 "	color: #ff0000 \n"
 "}\n"
 "\n"
+"blockquote {\n"
+"        background-color: ##F8;\n"
+"        border-left: 2px solid ##AA;\n"
+"        margin: 0 24px 0 4px;\n"
+"        padding: 0 12px 0 16px;\n"
+"}\n"
+"\n"
 "/* @end */\n"
 "\n"
 "/*\n"
@@ -343,6 +423,24 @@
 "	padding: 0px;\n"
 "}\n"
 "\n"
+".memberdecls td, .fieldtable tr {\n"
+"	-webkit-transition-property: background-color, box-shadow;\n"
+"	-webkit-transition-duration: 0.5s;\n"
+"	-moz-transition-property: background-color, box-shadow;\n"
+"	-moz-transition-duration: 0.5s;\n"
+"	-ms-transition-property: background-color, box-shadow;\n"
+"	-ms-transition-duration: 0.5s;\n"
+"	-o-transition-property: background-color, box-shadow;\n"
+"	-o-transition-duration: 0.5s;\n"
+"	transition-property: background-color, box-shadow;\n"
+"	transition-duration: 0.5s;\n"
+"}\n"
+"\n"
+".memberdecls td.glow, .fieldtable tr.glow {\n"
+"	background-color: cyan;\n"
+"	box-shadow: 0 0 15px cyan;\n"
+"}\n"
+"\n"
 ".mdescLeft, .mdescRight,\n"
 ".memItemLeft, .memItemRight,\n"
 ".memTemplItemLeft, .memTemplItemRight, .memTemplParams {\n"
@@ -357,8 +455,11 @@
 "	color: #555;\n"
 "}\n"
 "\n"
-".memItemLeft, .memItemRight, .memTemplParams {\n"
-"	border-top: 1px solid ##cc;\n"
+".memSeparator {\n"
+"        border-bottom: 1px solid #DEE4F0;\n"
+"        line-height: 1px;\n"
+"        margin: 0px;\n"
+"        padding: 0px;\n"
 "}\n"
 "\n"
 ".memItemLeft, .memTemplItemLeft {\n"
@@ -372,6 +473,7 @@
 ".memTemplParams {\n"
 "	color: ##60;\n"
 "        white-space: nowrap;\n"
+"	font-size: 80%;\n"
 "}\n"
 "\n"
 "/* @end */\n"
@@ -404,14 +506,28 @@
 "	padding: 0;\n"
 "	margin-bottom: 10px;\n"
 "	margin-right: 5px;\n"
+"        -webkit-transition: box-shadow 0.5s linear;\n"
+"        -moz-transition: box-shadow 0.5s linear;\n"
+"        -ms-transition: box-shadow 0.5s linear;\n"
+"        -o-transition: box-shadow 0.5s linear;\n"
+"        transition: box-shadow 0.5s linear;\n"
+"        display: table !important;\n"
+"        width: 100%;\n"
+"}\n"
+"\n"
+".memitem.glow {\n"
+"         box-shadow: 0 0 15px cyan;\n"
 "}\n"
 "\n"
 ".memname {\n"
-"        white-space: nowrap;\n"
 "        font-weight: bold;\n"
 "        margin-left: 6px;\n"
 "}\n"
 "\n"
+".memname td {\n"
+"	vertical-align: bottom;\n"
+"}\n"
+"\n"
 ".memproto, dl.reflist dt {\n"
 "        border-top: 1px solid ##B4;\n"
 "        border-left: 1px solid ##B4;\n"
@@ -420,21 +536,21 @@
 "        color: ##2b;\n"
 "        font-weight: bold;\n"
 "        text-shadow: 0px 1px 1px rgba(255, 255, 255, 0.9);\n"
-"        /* opera specific markup */\n"
-"        box-shadow: 5px 5px 5px rgba(0, 0, 0, 0.15);\n"
-"        border-top-right-radius: 8px;\n"
-"        border-top-left-radius: 8px;\n"
-"        /* firefox specific markup */\n"
-"        -moz-box-shadow: rgba(0, 0, 0, 0.15) 5px 5px 5px;\n"
-"        -moz-border-radius-topright: 8px;\n"
-"        -moz-border-radius-topleft: 8px;\n"
-"        /* webkit specific markup */\n"
-"        -webkit-box-shadow: 5px 5px 5px rgba(0, 0, 0, 0.15);\n"
-"        -webkit-border-top-right-radius: 8px;\n"
-"        -webkit-border-top-left-radius: 8px;\n"
 "        background-image:url('nav_f.png');\n"
 "        background-repeat:repeat-x;\n"
 "        background-color: ##E6;\n"
+"        /* opera specific markup */\n"
+"        box-shadow: 5px 5px 5px rgba(0, 0, 0, 0.15);\n"
+"        border-top-right-radius: 4px;\n"
+"        border-top-left-radius: 4px;\n"
+"        /* firefox specific markup */\n"
+"        -moz-box-shadow: rgba(0, 0, 0, 0.15) 5px 5px 5px;\n"
+"        -moz-border-radius-topright: 4px;\n"
+"        -moz-border-radius-topleft: 4px;\n"
+"        /* webkit specific markup */\n"
+"        -webkit-box-shadow: 5px 5px 5px rgba(0, 0, 0, 0.15);\n"
+"        -webkit-border-top-right-radius: 4px;\n"
+"        -webkit-border-top-left-radius: 4px;\n"
 "\n"
 "}\n"
 "\n"
@@ -442,23 +558,24 @@
 "        border-bottom: 1px solid ##B4;      \n"
 "        border-left: 1px solid ##B4;      \n"
 "        border-right: 1px solid ##B4; \n"
-"        padding: 2px 5px;\n"
+"        padding: 6px 10px 2px 10px;\n"
 "        background-color: ##FC;\n"
 "        border-top-width: 0;\n"
+"        background-image:url('nav_g.png');\n"
+"        background-repeat:repeat-x;\n"
+"        background-color: #FFFFFF;\n"
 "        /* opera specific markup */\n"
-"        border-bottom-left-radius: 8px;\n"
-"        border-bottom-right-radius: 8px;\n"
+"        border-bottom-left-radius: 4px;\n"
+"        border-bottom-right-radius: 4px;\n"
 "        box-shadow: 5px 5px 5px rgba(0, 0, 0, 0.15);\n"
 "        /* firefox specific markup */\n"
-"        -moz-border-radius-bottomleft: 8px;\n"
-"        -moz-border-radius-bottomright: 8px;\n"
+"        -moz-border-radius-bottomleft: 4px;\n"
+"        -moz-border-radius-bottomright: 4px;\n"
 "        -moz-box-shadow: rgba(0, 0, 0, 0.15) 5px 5px 5px;\n"
-"        background-image: -moz-linear-gradient(center top, ##FF 0%, ##FF 60%, ##F8 95%, ##F0);\n"
 "        /* webkit specific markup */\n"
-"        -webkit-border-bottom-left-radius: 8px;\n"
-"        -webkit-border-bottom-right-radius: 8px;\n"
+"        -webkit-border-bottom-left-radius: 4px;\n"
+"        -webkit-border-bottom-right-radius: 4px;\n"
 "        -webkit-box-shadow: 5px 5px 5px rgba(0, 0, 0, 0.15);\n"
-"        background-image: -webkit-gradient(linear,center top,center bottom,from(##FF), color-stop(0.6,##FF), color-stop(0.60,##FF), color-stop(0.95,##F8), to(##F0));\n"
 "}\n"
 "\n"
 "dl.reflist dt {\n"
@@ -485,9 +602,13 @@
 ".paramname em {\n"
 "	font-style: normal;\n"
 "}\n"
+".paramname code {\n"
+"        line-height: 14px;\n"
+"}\n"
 "\n"
 ".params, .retval, .exception, .tparams {\n"
-"        border-spacing: 6px 2px;\n"
+"        margin-left: 0px;\n"
+"        padding-left: 0px;\n"
 "}       \n"
 "\n"
 ".params .paramname, .retval .paramname {\n"
@@ -505,105 +626,116 @@
 "        vertical-align: top;\n"
 "}\n"
 "\n"
+"table.mlabels {\n"
+"	border-spacing: 0px;\n"
+"}\n"
+"\n"
+"td.mlabels-left {\n"
+"	width: 100%;\n"
+"	padding: 0px;\n"
+"}\n"
+"\n"
+"td.mlabels-right {\n"
+"	vertical-align: bottom;\n"
+"	padding: 0px;\n"
+"	white-space: nowrap;\n"
+"}\n"
+"\n"
+"span.mlabels {\n"
+"        margin-left: 8px;\n"
+"}\n"
+"\n"
+"span.mlabel {\n"
+"        background-color: ##88;\n"
+"        border-top:1px solid ##70;\n"
+"        border-left:1px solid ##70;\n"
+"        border-right:1px solid ##CC;\n"
+"        border-bottom:1px solid ##CC;\n"
+"	text-shadow: none;\n"
+"	color: white;\n"
+"	margin-right: 4px;\n"
+"	padding: 2px 3px;\n"
+"	border-radius: 3px;\n"
+"	font-size: 7pt;\n"
+"	white-space: nowrap;\n"
+"	vertical-align: middle;\n"
+"}\n"
 "\n"
 "\n"
 "\n"
 "/* @end */\n"
 "\n"
-"/* @group Directory (tree) */\n"
+"/* these are for tree view when not used as main index */\n"
 "\n"
-"/* for the tree view */\n"
-"\n"
-".ftvtree {\n"
-"	font-family: sans-serif;\n"
-"	margin: 0px;\n"
+"div.directory {\n"
+"        margin: 10px 0px;\n"
+"        border-top: 1px solid #A8B8D9;\n"
+"        border-bottom: 1px solid #A8B8D9;\n"
+"        width: 100%;\n"
 "}\n"
 "\n"
-"/* these are for tree view when used as main index */\n"
-"\n"
-".directory {\n"
-"	font-size: 9pt;\n"
-"	font-weight: bold;\n"
-"	margin: 5px;\n"
+".directory table {\n"
+"        border-collapse:collapse;\n"
 "}\n"
 "\n"
-".directory h3 {\n"
-"	margin: 0px;\n"
-"	margin-top: 1em;\n"
-"	font-size: 11pt;\n"
+".directory td {\n"
+"        margin: 0px;\n"
+"        padding: 0px;\n"
+"	vertical-align: top;\n"
 "}\n"
 "\n"
-"/*\n"
-"The following two styles can be used to replace the root node title\n"
-"with an image of your choice.  Simply uncomment the next two styles,\n"
-"specify the name of your image and be sure to set 'height' to the\n"
-"proper pixel height of your image.\n"
-"*/\n"
-"\n"
-"/*\n"
-".directory h3.swap {\n"
-"	height: 61px;\n"
-"	background-repeat: no-repeat;\n"
-"	background-image: url(\"yourimage.gif\");\n"
-"}\n"
-".directory h3.swap span {\n"
-"	display: none;\n"
-"}\n"
-"*/\n"
-"\n"
-".directory > h3 {\n"
-"	margin-top: 0;\n"
+".directory td.entry {\n"
+"        white-space: nowrap;\n"
+"        padding-right: 6px;\n"
 "}\n"
 "\n"
-".directory p {\n"
-"	margin: 0px;\n"
-"	white-space: nowrap;\n"
+".directory td.entry a {\n"
+"        outline:none;\n"
 "}\n"
 "\n"
-".directory div {\n"
-"	display: none;\n"
-"	margin: 0px;\n"
+".directory td.entry a img {\n"
+"        border: none;\n"
+"}\n"
+"\n"
+".directory td.desc {\n"
+"        width: 100%;\n"
+"        padding-left: 6px;\n"
+"	padding-right: 6px;\n"
+"	padding-top: 3px;\n"
+"	border-left: 1px solid rgba(0,0,0,0.05);\n"
+"}\n"
+"\n"
+".directory tr.even {\n"
+"	padding-left: 6px;\n"
+"	background-color: ##F8;\n"
 "}\n"
 "\n"
 ".directory img {\n"
 "	vertical-align: -30%;\n"
 "}\n"
 "\n"
-"/* these are for tree view when not used as main index */\n"
-"\n"
-".directory-alt {\n"
-"	font-size: 100%;\n"
-"	font-weight: bold;\n"
+".directory .levels {\n"
+"        white-space: nowrap;\n"
+"        width: 100%;\n"
+"        text-align: right;\n"
+"        font-size: 9pt;\n"
 "}\n"
 "\n"
-".directory-alt h3 {\n"
-"	margin: 0px;\n"
-"	margin-top: 1em;\n"
-"	font-size: 11pt;\n"
+".directory .levels span {\n"
+"        cursor: pointer;\n"
+"        padding-left: 2px;\n"
+"        padding-right: 2px;\n"
+"	color: ##50;\n"
 "}\n"
 "\n"
-".directory-alt > h3 {\n"
-"	margin-top: 0;\n"
-"}\n"
-"\n"
-".directory-alt p {\n"
-"	margin: 0px;\n"
-"	white-space: nowrap;\n"
-"}\n"
-"\n"
-".directory-alt div {\n"
-"	display: none;\n"
-"	margin: 0px;\n"
-"}\n"
-"\n"
-".directory-alt img {\n"
-"	vertical-align: -30%;\n"
-"}\n"
-"\n"
-"/* @end */\n"
-"\n"
 "div.dynheader {\n"
 "        margin-top: 8px;\n"
+"	-webkit-touch-callout: none;\n"
+"	-webkit-user-select: none;\n"
+"	-khtml-user-select: none;\n"
+"	-moz-user-select: none;\n"
+"	-ms-user-select: none;\n"
+"	user-select: none;\n"
 "}\n"
 "\n"
 "address {\n"
@@ -613,6 +745,8 @@
 "\n"
 "table.doxtable {\n"
 "	border-collapse:collapse;\n"
+"        margin-top: 4px;\n"
+"        margin-bottom: 4px;\n"
 "}\n"
 "\n"
 "table.doxtable td, table.doxtable th {\n"
@@ -626,7 +760,6 @@
 "	font-size: 110%;\n"
 "	padding-bottom: 4px;\n"
 "	padding-top: 5px;\n"
-"	text-align:left;\n"
 "}\n"
 "\n"
 "table.fieldtable {\n"
@@ -696,6 +829,7 @@
 "	font-size: 11px;\n"
 "	background-image:url('tab_b.png');\n"
 "	background-repeat:repeat-x;\n"
+"	background-position: 0 -5px;\n"
 "	height:30px;\n"
 "	line-height:30px;\n"
 "	color:##9b;\n"
@@ -723,6 +857,10 @@
 "	display:block;\n"
 "	text-decoration: none;\n"
 "	outline: none;\n"
+"	color: ##30;\n"
+"	font-family: 'Lucida Grande',Geneva,Helvetica,Arial,sans-serif;\n"
+"	text-shadow: 0px 1px 1px rgba(255, 255, 255, 0.9);\n"
+"	text-decoration: none;        \n"
 "}\n"
 "\n"
 ".navpath li.navelem a:hover\n"
@@ -760,9 +898,7 @@
 "\n"
 "div.ingroups\n"
 "{\n"
-"	margin-left: 5px;\n"
 "	font-size: 8pt;\n"
-"	padding-left: 5px;\n"
 "	width: 50%;\n"
 "	text-align: left;\n"
 "}\n"
@@ -783,7 +919,7 @@
 "\n"
 "div.headertitle\n"
 "{\n"
-"	padding: 5px 5px 5px 7px;\n"
+"	padding: 5px 5px 5px 10px;\n"
 "}\n"
 "\n"
 "dl\n"
@@ -791,47 +927,74 @@
 "        padding: 0 0 0 10px;\n"
 "}\n"
 "\n"
-"dl.note, dl.warning, dl.attention, dl.pre, dl.post, dl.invariant, dl.deprecated, dl.todo, dl.test, dl.bug\n"
+"/* dl.note, dl.warning, dl.attention, dl.pre, dl.post, dl.invariant, dl.deprecated, dl.todo, dl.test, dl.bug */\n"
+"dl.section\n"
 "{\n"
-"        border-left:4px solid;\n"
-"        padding: 0 0 0 6px;\n"
+"	margin-left: 0px;\n"
+"	padding-left: 0px;\n"
 "}\n"
 "\n"
 "dl.note\n"
 "{\n"
+"        margin-left:-7px;\n"
+"        padding-left: 3px;\n"
+"        border-left:4px solid;\n"
 "        border-color: #D0C000;\n"
 "}\n"
 "\n"
 "dl.warning, dl.attention\n"
 "{\n"
+"        margin-left:-7px;\n"
+"        padding-left: 3px;\n"
+"        border-left:4px solid;\n"
 "        border-color: #FF0000;\n"
 "}\n"
 "\n"
 "dl.pre, dl.post, dl.invariant\n"
 "{\n"
+"        margin-left:-7px;\n"
+"        padding-left: 3px;\n"
+"        border-left:4px solid;\n"
 "        border-color: #00D000;\n"
 "}\n"
 "\n"
 "dl.deprecated\n"
 "{\n"
+"        margin-left:-7px;\n"
+"        padding-left: 3px;\n"
+"        border-left:4px solid;\n"
 "        border-color: #505050;\n"
 "}\n"
 "\n"
 "dl.todo\n"
 "{\n"
+"        margin-left:-7px;\n"
+"        padding-left: 3px;\n"
+"        border-left:4px solid;\n"
 "        border-color: #00C0E0;\n"
 "}\n"
 "\n"
 "dl.test\n"
 "{\n"
+"        margin-left:-7px;\n"
+"        padding-left: 3px;\n"
+"        border-left:4px solid;\n"
 "        border-color: #3030E0;\n"
 "}\n"
 "\n"
 "dl.bug\n"
 "{\n"
+"        margin-left:-7px;\n"
+"        padding-left: 3px;\n"
+"        border-left:4px solid;\n"
 "        border-color: #C08050;\n"
 "}\n"
 "\n"
+"dl.section dd {\n"
+"	margin-bottom: 6px;\n"
+"}\n"
+"\n"
+"\n"
 "#projectlogo\n"
 "{\n"
 "	text-align: center;\n"
@@ -915,6 +1078,79 @@
 "        padding:5px 0;\n"
 "}\n"
 "\n"
+"div.toc {\n"
+"        padding: 14px 25px;\n"
+"        background-color: ##F6;\n"
+"        border: 1px solid ##DD;\n"
+"        border-radius: 7px 7px 7px 7px;\n"
+"        float: right;\n"
+"        height: auto;\n"
+"        margin: 0 20px 10px 10px;\n"
+"        width: 200px;\n"
+"}\n"
+"\n"
+"div.toc li {\n"
+"        background: url(\"bdwn.png\") no-repeat scroll 0 5px transparent;\n"
+"        font: 10px/1.2 Verdana,DejaVu Sans,Geneva,sans-serif;\n"
+"        margin-top: 5px;\n"
+"        padding-left: 10px;\n"
+"        padding-top: 2px;\n"
+"}\n"
+"\n"
+"div.toc h3 {\n"
+"        font: bold 12px/1.2 Arial,FreeSans,sans-serif;\n"
+"	color: ##60;\n"
+"        border-bottom: 0 none;\n"
+"        margin: 0;\n"
+"}\n"
+"\n"
+"div.toc ul {\n"
+"        list-style: none outside none;\n"
+"        border: medium none;\n"
+"        padding: 0px;\n"
+"}       \n"
+"\n"
+"div.toc li.level1 {\n"
+"        margin-left: 0px;\n"
+"}\n"
+"\n"
+"div.toc li.level2 {\n"
+"        margin-left: 15px;\n"
+"}\n"
+"\n"
+"div.toc li.level3 {\n"
+"        margin-left: 30px;\n"
+"}\n"
+"\n"
+"div.toc li.level4 {\n"
+"        margin-left: 45px;\n"
+"}\n"
+"\n"
+".inherit_header {\n"
+"        font-weight: bold;\n"
+"        color: gray;\n"
+"        cursor: pointer;\n"
+"	-webkit-touch-callout: none;\n"
+"	-webkit-user-select: none;\n"
+"	-khtml-user-select: none;\n"
+"	-moz-user-select: none;\n"
+"	-ms-user-select: none;\n"
+"	user-select: none;\n"
+"}\n"
+"\n"
+".inherit_header td {\n"
+"        padding: 6px 0px 2px 5px;\n"
+"}\n"
+"\n"
+".inherit {\n"
+"        display: none;\n"
+"}\n"
+"\n"
+"tr.heading h2 {\n"
+"        margin-top: 12px;\n"
+"        margin-bottom: 4px;\n"
+"}\n"
+"\n"
 "@media print\n"
 "{\n"
 "  #top { display: none; }\n"
@@ -932,15 +1168,5 @@
 "    overflow:inherit;\n"
 "    display:inline;\n"
 "  }\n"
-"  pre.fragment\n"
-"  {\n"
-"    overflow: visible;\n"
-"    text-wrap: unrestricted;\n"
-"    white-space: -moz-pre-wrap; /* Moz */\n"
-"    white-space: -pre-wrap; /* Opera 4-6 */\n"
-"    white-space: -o-pre-wrap; /* Opera 7 */\n"
-"    white-space: pre-wrap; /* CSS3  */\n"
-"    word-wrap: break-word; /* IE 5.5+ */\n"
-"  }\n"
 "}\n"
 "\n"
