@@ -2,7 +2,7 @@
  *
  * $Id: filedef.cpp,v 1.55 2001/03/19 19:27:40 root Exp $
  *
- * Copyright (C) 1997-2011 by Dimitri van Heesch.
+ * Copyright (C) 1997-2012 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -36,9 +36,11 @@
 #include "vhdldocgen.h"
 #include "debug.h"
 #include "layout.h"
+#include "entry.h"
 
 //---------------------------------------------------------------------------
 
+/** Class implementing CodeOutputInterface by throwing away everything. */
 class DevNullCodeDocInterface : public CodeOutputInterface
 {
   public:
@@ -48,7 +50,7 @@ class DevNullCodeDocInterface : public CodeOutputInterface
                                const char *) {}
     virtual void writeLineNumber(const char *,const char *,
                                  const char *,int) {}
-    virtual void startCodeLine() {}
+    virtual void startCodeLine(bool) {}
     virtual void endCodeLine() {}
     virtual void startCodeAnchor(const char *) {}
     virtual void endCodeAnchor() {}
@@ -165,13 +167,16 @@ void FileDef::findSectionsInDocumentation()
 void FileDef::writeDetailedDescription(OutputList &ol,const QCString &title)
 {
   if ((!briefDescription().isEmpty() && Config_getBool("REPEAT_BRIEF")) || 
-      !documentation().isEmpty() ||
+      !documentation().stripWhiteSpace().isEmpty() || // avail empty section
       (Config_getBool("SOURCE_BROWSER") && getStartBodyLine()!=-1 && getBodyDef())
      )
   {
-    ol.writeRuler();
     ol.pushGeneratorState();
-    ol.disableAllBut(OutputGenerator::Html);
+      ol.disable(OutputGenerator::Html);
+      ol.writeRuler();
+    ol.popGeneratorState();
+    ol.pushGeneratorState();
+      ol.disableAllBut(OutputGenerator::Html);
       ol.writeAnchor(0,"details"); 
     ol.popGeneratorState();
     ol.startGroupHeader();
@@ -261,71 +266,74 @@ void FileDef::writeIncludeFiles(OutputList &ol)
     IncludeInfo *ii;
     for (;(ii=ili.current());++ili)
     {
-      FileDef *fd=ii->fileDef;
-      bool isIDLorJava = FALSE;
-      if (fd)
+      if (!ii->indirect)
       {
-        SrcLangExt lang   = fd->getLanguage();
-        isIDLorJava = lang==SrcLangExt_IDL || lang==SrcLangExt_Java;
-      }
-      ol.startTypewriter();
-      if (isIDLorJava) // IDL/Java include
-      {
-        ol.docify("import ");
-      }
-      else if (ii->imported) // Objective-C include
-      {
-        ol.docify("#import ");
-      }
-      else // C/C++ include
-      {
-        ol.docify("#include ");
-      }
-      if (ii->local || isIDLorJava)
-        ol.docify("\"");
-      else
-        ol.docify("<");
-      ol.disable(OutputGenerator::Html);
-      ol.docify(ii->includeName);
-      ol.enableAll();
-      ol.disableAllBut(OutputGenerator::Html);
-      
-      // Here we use the include file name as it appears in the file.
-      // we could also we the name as it is used within doxygen,
-      // then we should have used fd->docName() instead of ii->includeName
-      if (fd && fd->isLinkable())
-      {
-        ol.writeObjectLink(fd->getReference(),
-            fd->generateSourceFile() ? fd->includeName() : fd->getOutputFileBase(),
-            0,ii->includeName);
-        if (!Config_getString("GENERATE_TAGFILE").isEmpty() && !fd->isReference()) 
+        FileDef *fd=ii->fileDef;
+        bool isIDLorJava = FALSE;
+        if (fd)
         {
-          const char *locStr = (ii->local    || isIDLorJava) ? "yes" : "no";
-          const char *impStr = (ii->imported || isIDLorJava) ? "yes" : "no";
-          Doxygen::tagFile << "    <includes id=\"" 
-                           << convertToXML(fd->getOutputFileBase()) << "\" "
-                           << "name=\"" << convertToXML(fd->name()) << "\" "
-                           << "local=\"" << locStr << "\" "
-                           << "imported=\"" << impStr << "\">"
-                           << convertToXML(ii->includeName)
-                           << "</includes>" 
-                           << endl;
+          SrcLangExt lang   = fd->getLanguage();
+          isIDLorJava = lang==SrcLangExt_IDL || lang==SrcLangExt_Java;
         }
-      }
-      else
-      {
+        ol.startTypewriter();
+        if (isIDLorJava) // IDL/Java include
+        {
+          ol.docify("import ");
+        }
+        else if (ii->imported) // Objective-C include
+        {
+          ol.docify("#import ");
+        }
+        else // C/C++ include
+        {
+          ol.docify("#include ");
+        }
+        if (ii->local || isIDLorJava)
+          ol.docify("\"");
+        else
+          ol.docify("<");
+        ol.disable(OutputGenerator::Html);
         ol.docify(ii->includeName);
+        ol.enableAll();
+        ol.disableAllBut(OutputGenerator::Html);
+        
+        // Here we use the include file name as it appears in the file.
+        // we could also we the name as it is used within doxygen,
+        // then we should have used fd->docName() instead of ii->includeName
+        if (fd && fd->isLinkable())
+        {
+          ol.writeObjectLink(fd->getReference(),
+              fd->generateSourceFile() ? fd->includeName() : fd->getOutputFileBase(),
+              0,ii->includeName);
+          if (!Config_getString("GENERATE_TAGFILE").isEmpty() && !fd->isReference()) 
+          {
+            const char *locStr = (ii->local    || isIDLorJava) ? "yes" : "no";
+            const char *impStr = (ii->imported || isIDLorJava) ? "yes" : "no";
+            Doxygen::tagFile << "    <includes id=\"" 
+                             << convertToXML(fd->getOutputFileBase()) << "\" "
+                             << "name=\"" << convertToXML(fd->name()) << "\" "
+                             << "local=\"" << locStr << "\" "
+                             << "imported=\"" << impStr << "\">"
+                             << convertToXML(ii->includeName)
+                             << "</includes>" 
+                             << endl;
+          }
+        }
+        else
+        {
+          ol.docify(ii->includeName);
+        }
+        
+        ol.enableAll();
+        if (ii->local || isIDLorJava)
+          ol.docify("\"");
+        else
+          ol.docify(">");
+        if (isIDLorJava) 
+          ol.docify(";");
+        ol.endTypewriter();
+        ol.lineBreak();
       }
-      
-      ol.enableAll();
-      if (ii->local || isIDLorJava)
-        ol.docify("\"");
-      else
-        ol.docify(">");
-      if (isIDLorJava) 
-        ol.docify(";");
-      ol.endTypewriter();
-      ol.lineBreak();
     }
     ol.endTextBlock();
   }
@@ -337,7 +345,11 @@ void FileDef::writeIncludeGraph(OutputList &ol)
   {
     //printf("Graph for file %s\n",name().data());
     DotInclDepGraph incDepGraph(this,FALSE);
-    if (!incDepGraph.isTrivial() && !incDepGraph.isTooBig())
+    if (incDepGraph.isTooBig())
+    {
+       err("warning: Include graph for '%s' not generated, too many nodes. Consider increasing DOT_GRAPH_MAX_NODES.\n",name().data());
+    }
+    else if (!incDepGraph.isTrivial())
     {
       ol.startTextBlock(); 
       ol.disable(OutputGenerator::Man);
@@ -357,7 +369,11 @@ void FileDef::writeIncludedByGraph(OutputList &ol)
   {
     //printf("Graph for file %s\n",name().data());
     DotInclDepGraph incDepGraph(this,TRUE);
-    if (!incDepGraph.isTrivial() && !incDepGraph.isTooBig())
+    if (incDepGraph.isTooBig())
+    {
+       err("warning: Included by graph for '%s' not generated, too many nodes. Consider increasing DOT_GRAPH_MAX_NODES.\n",name().data());
+    }
+    if (!incDepGraph.isTrivial())
     {
       ol.startTextBlock(); 
       ol.disable(OutputGenerator::Man);
@@ -471,6 +487,7 @@ void FileDef::writeSummaryLinks(OutputList &ol)
       LayoutDocManager::instance().docEntries(LayoutDocManager::File));
   LayoutDocEntry *lde;
   bool first=TRUE;
+  SrcLangExt lang=getLanguage();
   for (eli.toFirst();(lde=eli.current());++eli)
   {
     if ((lde->kind()==LayoutDocEntry::FileClasses && classSDict && classSDict->declVisible()) || 
@@ -479,7 +496,8 @@ void FileDef::writeSummaryLinks(OutputList &ol)
     {
       LayoutDocEntrySection *ls = (LayoutDocEntrySection*)lde;
       QCString label = lde->kind()==LayoutDocEntry::FileClasses ? "nested-classes" : "namespaces";
-      writeSummaryLink(ol,label,ls->title,first);
+      ol.writeSummaryLink(0,label,ls->title(lang),first);
+      first=FALSE;
     }
     else if (lde->kind()==LayoutDocEntry::MemberDecl)
     {
@@ -487,7 +505,8 @@ void FileDef::writeSummaryLinks(OutputList &ol)
       MemberList * ml = getMemberList(lmd->type);
       if (ml && ml->declVisible())
       {
-        writeSummaryLink(ol,ml->listTypeAsString(),lmd->title,first);
+        ol.writeSummaryLink(0,ml->listTypeAsString(),lmd->title(lang),first);
+        first=FALSE;
       }
     }
   }
@@ -522,7 +541,7 @@ void FileDef::writeDocumentation(OutputList &ol)
   QCString title = docname+versionTitle;
   QCString pageTitle=theTranslator->trFileReference(docname);
 
-  if (Config_getBool("SHOW_DIRECTORIES") && getDirDef())
+  if (getDirDef())
   {
     startFile(ol,getOutputFileBase(),name(),pageTitle,HLI_FileVisible,!generateTreeView);
     if (!generateTreeView)
@@ -568,7 +587,7 @@ void FileDef::writeDocumentation(OutputList &ol)
   
   if (Doxygen::searchIndex)
   {
-    Doxygen::searchIndex->setCurrentDoc(pageTitle,getOutputFileBase());
+    Doxygen::searchIndex->setCurrentDoc(this,anchor(),FALSE);
     Doxygen::searchIndex->addWord(localName(),TRUE);
   }
   
@@ -584,6 +603,7 @@ void FileDef::writeDocumentation(OutputList &ol)
 
   //---------------------------------------- start flexible part -------------------------------
   
+  SrcLangExt lang = getLanguage();
   QListIterator<LayoutDocEntry> eli(
       LayoutDocManager::instance().docEntries(LayoutDocManager::File));
   LayoutDocEntry *lde;
@@ -612,13 +632,13 @@ void FileDef::writeDocumentation(OutputList &ol)
       case LayoutDocEntry::FileClasses: 
         {
           LayoutDocEntrySection *ls = (LayoutDocEntrySection*)lde;
-          writeClassDeclarations(ol,ls->title);
+          writeClassDeclarations(ol,ls->title(lang));
         }
         break; 
       case LayoutDocEntry::FileNamespaces: 
         {
           LayoutDocEntrySection *ls = (LayoutDocEntrySection*)lde;
-          writeNamespaceDeclarations(ol,ls->title);
+          writeNamespaceDeclarations(ol,ls->title(lang));
         }
         break; 
       case LayoutDocEntry::MemberGroups: 
@@ -627,7 +647,7 @@ void FileDef::writeDocumentation(OutputList &ol)
       case LayoutDocEntry::MemberDecl: 
         {
           LayoutDocEntryMemberDecl *lmd = (LayoutDocEntryMemberDecl*)lde;
-          writeMemberDeclarations(ol,lmd->type,lmd->title);
+          writeMemberDeclarations(ol,lmd->type,lmd->title(lang));
         }
         break; 
       case LayoutDocEntry::MemberDeclEnd: 
@@ -636,7 +656,7 @@ void FileDef::writeDocumentation(OutputList &ol)
       case LayoutDocEntry::DetailedDesc: 
         {
           LayoutDocEntrySection *ls = (LayoutDocEntrySection*)lde;
-          writeDetailedDescription(ol,ls->title);
+          writeDetailedDescription(ol,ls->title(lang));
         }
         break;
       case LayoutDocEntry::MemberDefStart: 
@@ -648,7 +668,7 @@ void FileDef::writeDocumentation(OutputList &ol)
       case LayoutDocEntry::MemberDef: 
         {
           LayoutDocEntryMemberDef *lmd = (LayoutDocEntryMemberDef*)lde;
-          writeMemberDocumentation(ol,lmd->type,lmd->title);
+          writeMemberDocumentation(ol,lmd->type,lmd->title(lang));
         }
         break;
       case LayoutDocEntry::MemberDefEnd: 
@@ -694,12 +714,7 @@ void FileDef::writeDocumentation(OutputList &ol)
 
   ol.endContents();
 
-  if (generateTreeView)
-  {
-    writeNavigationPath(ol);
-  }
-
-  endFile(ol,TRUE);
+  endFileWithNavPath(this,ol);
 
   if (Config_getBool("SEPARATE_MEMBER_PAGES"))
   {
@@ -758,7 +773,7 @@ void FileDef::writeQuickMemberLinks(OutputList &ol,MemberDef *currentMd) const
           if (createSubDirs) ol.writeString("../../");
           ol.writeString(md->getOutputFileBase()+Doxygen::htmlFileExtension+"#"+md->anchor());
           ol.writeString("\">");
-          ol.writeString(md->localName());
+          ol.writeString(convertToHtml(md->localName()));
           ol.writeString("</a>");
         }
         ol.writeString("</td></tr>\n");
@@ -786,10 +801,13 @@ void FileDef::writeSource(OutputList &ol)
   ol.disable(OutputGenerator::RTF);
   if (!latexSourceCode) ol.disable(OutputGenerator::Latex);
 
-  if (Config_getBool("SHOW_DIRECTORIES") && getDirDef())
+  bool isDocFile = isDocumentationFile();
+  bool genSourceFile = !isDocFile && generateSourceFile();
+  if (getDirDef())
   {
     startFile(ol,getSourceFileBase(),0,pageTitle,HLI_FileVisible,
-        !generateTreeView,getOutputFileBase());
+        !generateTreeView,
+        !isDocFile && genSourceFile ? 0 : getOutputFileBase());
     if (!generateTreeView)
     {
       getDirDef()->writeNavigationPath(ol);
@@ -801,8 +819,8 @@ void FileDef::writeSource(OutputList &ol)
   }
   else
   {
-    startFile(ol,getSourceFileBase(),0,pageTitle,HLI_FileVisible,
-        !generateTreeView,getOutputFileBase());
+    startFile(ol,getSourceFileBase(),0,pageTitle,HLI_FileVisible,FALSE,
+        !isDocFile && genSourceFile ? 0 : getOutputFileBase());
     startTitle(ol,getSourceFileBase());
     ol.parseText(title);
     endTitle(ol,getSourceFileBase(),0);
@@ -828,15 +846,7 @@ void FileDef::writeSource(OutputList &ol)
            );
   ol.endCodeFragment();
   ol.endContents();
-  if (generateTreeView)
-  {
-    writeNavigationPath(ol);
-    endFile(ol,TRUE);
-  }
-  else
-  {
-    endFile(ol);
-  }
+  endFileWithNavPath(this,ol);
   ol.enableAll();
 }
 
@@ -977,7 +987,7 @@ void FileDef::addSourceRef(int line,Definition *d,MemberDef *md)
     srcDefDict->insert(line,d);
     if (md) srcMemberDict->insert(line,md);
     //printf("Adding member %s with anchor %s at line %d to file %s\n",
-    //    md->name().data(),md->anchor().data(),line,name().data());
+    //    md?md->name().data():"<none>",md?md->anchor().data():"<none>",line,name().data());
   }
 }
 
@@ -1034,7 +1044,7 @@ void FileDef::addUsingDeclaration(Definition *d)
 }
 
 void FileDef::addIncludeDependency(FileDef *fd,const char *incName,bool local,
-                                   bool imported)
+                                   bool imported,bool indirect)
 {
   //printf("FileDef::addIncludeDependency(%p,%s,%d)\n",fd,incName,local);
   QCString iName = fd ? fd->absFilePath().data() : incName;
@@ -1051,6 +1061,7 @@ void FileDef::addIncludeDependency(FileDef *fd,const char *incName,bool local,
     ii->includeName = incName;
     ii->local       = local;
     ii->imported    = imported;
+    ii->indirect    = indirect;
     includeList->append(ii);
     includeDict->insert(iName,ii);
   }
@@ -1148,6 +1159,7 @@ void FileDef::addIncludedByDependency(FileDef *fd,const char *incName,
     ii->includeName = incName;
     ii->local       = local;
     ii->imported    = imported;
+    ii->indirect    = FALSE;
     includedByList->append(ii);
     includedByDict->insert(iName,ii);
   }
@@ -1166,7 +1178,8 @@ bool FileDef::generateSourceFile() const
          (Config_getBool("SOURCE_BROWSER") || 
            (Config_getBool("VERBATIM_HEADERS") && guessSection(name())==Entry::HEADER_SEC) 
          ) &&
-         extension!=".doc" && extension!=".txt" && extension!=".dox"; 
+         extension!=".doc" && extension!=".txt" && extension!=".dox" &&
+         extension!=".md" && extension!=".markdown";  
 }
 
 
@@ -1492,13 +1505,15 @@ bool FileDef::isDocumentationFile() const
 {
   return name().right(4)==".doc" ||
          name().right(4)==".txt" ||
-         name().right(4)==".dox";
+         name().right(4)==".dox" ||
+         name().right(3)==".md"  ||
+         name().right(9)==".markdown";
 }
 
 void FileDef::acquireFileVersion()
 {
   QCString vercmd = Config_getString("FILE_VERSION_FILTER");
-  if (!vercmd.isEmpty() && filepath!="generated") 
+  if (!vercmd.isEmpty() && !filepath.isEmpty() && filepath!="generated") 
   {
     msg("Version of %s : ",filepath.data());
     QCString cmd = vercmd+" \""+filepath+"\"";
@@ -1513,9 +1528,8 @@ void FileDef::acquireFileVersion()
     char buf[bufSize];
     int numRead = fread(buf,1,bufSize,f);
     portable_pclose(f);
-    if (numRead > 0) 
+    if (numRead>0 && !(fileVersion=QCString(buf,numRead).stripWhiteSpace()).isEmpty())
     {
-      fileVersion = QCString(buf,numRead).stripWhiteSpace();
       msg("%s\n",fileVersion.data());
     }
     else 
@@ -1624,7 +1638,8 @@ void FileDef::writeMemberDeclarations(OutputList &ol,MemberList::ListType lt,con
   {
     if (optVhdl) // use specific declarations function
     {
-      VhdlDocGen::writeVhdlDeclarations(ml,ol,0,0,this);
+
+      VhdlDocGen::writeVhdlDeclarations(ml,ol,0,0,this,0);
     }
     else
     {
@@ -1645,10 +1660,11 @@ bool FileDef::isLinkableInProject() const
   return hasDocumentation() && !isReference() && showFiles;
 }
 
+#if 0
 bool FileDef::includes(FileDef *incFile,QDict<FileDef> *includedFiles) const
 {
-  if (incFile==this) return TRUE;
   //printf("%s::includes(%s)\n",name().data(),incFile->name().data());
+  if (incFile==this) return TRUE;
   includedFiles->insert(absFilePath(),this);
   if (includeList)
   {
@@ -1656,12 +1672,30 @@ bool FileDef::includes(FileDef *incFile,QDict<FileDef> *includedFiles) const
     IncludeInfo *ii;
     for (;(ii=ili.current());++ili)
     {
-      if (ii->fileDef && 
-          includedFiles->find(ii->fileDef->absFilePath())==0 &&
-          ii->fileDef->includes(incFile,includedFiles)) return TRUE;
+      //printf("ii=%s\n",ii->includeName.data());
+      if ((ii->fileDef && 
+           includedFiles->find(ii->fileDef->absFilePath())==0 &&
+           ii->fileDef->includes(incFile,includedFiles)
+          ) ||
+          incFile->absFilePath()==ii->includeName
+         ) return TRUE;
     }
   }
   return FALSE;
 }
 
-
+bool FileDef::includesByName(const QCString &fileName) const
+{
+  if (includeList)
+  {
+    QListIterator<IncludeInfo> ili(*includeList);
+    IncludeInfo *ii;
+    for (;(ii=ili.current());++ili)
+    {
+      //printf("ii=%s\n",ii->includeName.data());
+      if (fileName==ii->includeName) return TRUE;
+    }
+  }
+  return FALSE;
+}
+#endif
