@@ -2,7 +2,7 @@
  *
  * $Id: fortranscanner.h,v 1.1 2006/08/23 07:44:03 zdv058 Exp $
  *
- * Copyright (C) 1997-2008 by Dimitri van Heesch.
+ * Copyright (C) 1997-2012 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -41,7 +41,9 @@ class FortranLanguageScanner : public ParserInterface
                    int startLine=-1,
                    int endLine=-1,
                    bool inlineFragment=FALSE,
-                   MemberDef *memberDef=0
+                   MemberDef *memberDef=0,
+                   bool showLineNumbers=TRUE,
+                   Definition *searchCtx=0
                   );
     void resetCodeParserState();
     void parsePrototype(const char *text);
