@@ -2,7 +2,7 @@
  *
  * $Id: groupdef.cpp,v 1.29 2001/03/19 19:27:40 root Exp $
  *
- * Copyright (C) 1997-2011 by Dimitri van Heesch.
+ * Copyright (C) 1997-2012 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -36,6 +36,8 @@
 #include "dot.h"
 #include "vhdldocgen.h"
 #include "layout.h"
+#include "arguments.h"
+#include "entry.h"
 
 //---------------------------------------------------------------------------
 
@@ -133,6 +135,7 @@ void GroupDef::addFile(const FileDef *def)
 {
   static bool sortBriefDocs = Config_getBool("SORT_BRIEF_DOCS");
   if (def->isHidden()) return;
+  updateLanguage(def);
   if (sortBriefDocs)
     fileList->inSort(def);
   else
@@ -143,6 +146,7 @@ bool GroupDef::addClass(const ClassDef *cd)
 {
   static bool sortBriefDocs = Config_getBool("SORT_BRIEF_DOCS");
   if (cd->isHidden()) return FALSE;
+  updateLanguage(cd);
   if (classSDict->find(cd->qualifiedName())==0)
   {
     QCString qn = cd->qualifiedName();
@@ -190,6 +194,7 @@ bool GroupDef::addNamespace(const NamespaceDef *def)
 {
   static bool sortBriefDocs = Config_getBool("SORT_BRIEF_DOCS");
   if (def->isHidden()) return FALSE;
+  updateLanguage(def);
   if (namespaceSDict->find(def->name())==0)
   {
     if (sortBriefDocs)
@@ -250,6 +255,7 @@ void GroupDef::addMembersToMemberGroup()
 bool GroupDef::insertMember(MemberDef *md,bool docOnly)
 {
   if (md->isHidden()) return FALSE;
+  updateLanguage(md);
   //printf("GroupDef(%s)::insertMember(%s)\n", title.data(), md->name().data());
   MemberNameInfo *mni=0;
   if ((mni=(*allMemberNameInfoSDict)[md->name()]))
@@ -545,9 +551,12 @@ void GroupDef::writeDetailedDescription(OutputList &ol,const QCString &title)
   {
     if (pageDict->count()!=countMembers()) // not only pages -> classical layout
     {
-      ol.writeRuler();
       ol.pushGeneratorState();
-      ol.disableAllBut(OutputGenerator::Html);
+        ol.disable(OutputGenerator::Html);
+        ol.writeRuler();
+      ol.popGeneratorState();
+      ol.pushGeneratorState();
+        ol.disableAllBut(OutputGenerator::Html);
         ol.writeAnchor(0,"details");
       ol.popGeneratorState();
       ol.startGroupHeader();
@@ -646,7 +655,8 @@ void GroupDef::writeFiles(OutputList &ol,const QCString &title)
     FileDef *fd=fileList->first();
     while (fd)
     {
-      ol.startMemberItem(0);
+      ol.startMemberDeclaration();
+      ol.startMemberItem(fd->getOutputFileBase(),0);
       ol.docify(theTranslator->trFile(FALSE,TRUE)+" ");
       ol.insertMemberAlign();
       ol.writeObjectLink(fd->getReference(),fd->getOutputFileBase(),0,fd->name());
@@ -657,10 +667,11 @@ void GroupDef::writeFiles(OutputList &ol,const QCString &title)
       ol.endMemberItem();
       if (!fd->briefDescription().isEmpty() && Config_getBool("BRIEF_MEMBER_DESC"))
       {
-        ol.startMemberDescription();
-        ol.parseDoc(briefFile(),briefLine(),fd,0,fd->briefDescription(),FALSE,FALSE);
+        ol.startMemberDescription(fd->getOutputFileBase());
+        ol.parseDoc(briefFile(),briefLine(),fd,0,fd->briefDescription(),FALSE,FALSE,0,TRUE,FALSE);
         ol.endMemberDescription();
       }
+      ol.endMemberDeclaration(0,0);
       fd=fileList->next();
     }
     ol.endMemberList();
@@ -676,30 +687,49 @@ void GroupDef::writeNamespaces(OutputList &ol,const QCString &title)
 void GroupDef::writeNestedGroups(OutputList &ol,const QCString &title)
 {
   // write list of groups
+  int count=0;
   if (groupList->count()>0)
+  {
+    GroupDef *gd=groupList->first();
+    while (gd)
+    {
+      if (gd->isVisible()) count++;
+      gd=groupList->next();
+    }
+  }
+  if (count>0)
   {
     ol.startMemberHeader("groups");
     ol.parseText(title);
     ol.endMemberHeader();
     ol.startMemberList();
+    if (Config_getBool("SORT_GROUP_NAMES"))
+    {
+      groupList->sort();
+    }
     GroupDef *gd=groupList->first();
     while (gd)
     {
-      ol.startMemberItem(0);
-      //ol.docify(theTranslator->trGroup(FALSE,TRUE));
-      //ol.docify(" ");
-      ol.insertMemberAlign();
-      ol.writeObjectLink(gd->getReference(),gd->getOutputFileBase(),0,gd->groupTitle());
-      if (!Config_getString("GENERATE_TAGFILE").isEmpty()) 
+      if (gd->isVisible())
       {
-        Doxygen::tagFile << "    <subgroup>" << convertToXML(gd->name()) << "</subgroup>" << endl;
-      }
-      ol.endMemberItem();
-      if (!gd->briefDescription().isEmpty() && Config_getBool("BRIEF_MEMBER_DESC"))
-      {
-        ol.startMemberDescription();
-        ol.parseDoc(briefFile(),briefLine(),gd,0,gd->briefDescription(),FALSE,FALSE);
-        ol.endMemberDescription();
+        ol.startMemberDeclaration();
+        ol.startMemberItem(gd->getOutputFileBase(),0);
+        //ol.docify(theTranslator->trGroup(FALSE,TRUE));
+        //ol.docify(" ");
+        ol.insertMemberAlign();
+        ol.writeObjectLink(gd->getReference(),gd->getOutputFileBase(),0,gd->groupTitle());
+        if (!Config_getString("GENERATE_TAGFILE").isEmpty()) 
+        {
+          Doxygen::tagFile << "    <subgroup>" << convertToXML(gd->name()) << "</subgroup>" << endl;
+        }
+        ol.endMemberItem();
+        if (!gd->briefDescription().isEmpty() && Config_getBool("BRIEF_MEMBER_DESC"))
+        {
+          ol.startMemberDescription(gd->getOutputFileBase());
+          ol.parseDoc(briefFile(),briefLine(),gd,0,gd->briefDescription(),FALSE,FALSE,0,TRUE,FALSE);
+          ol.endMemberDescription();
+        }
+        ol.endMemberDeclaration(0,0);
       }
       gd=groupList->next();
     }
@@ -719,7 +749,8 @@ void GroupDef::writeDirs(OutputList &ol,const QCString &title)
     DirDef *dd=dirList->first();
     while (dd)
     {
-      ol.startMemberItem(0);
+      ol.startMemberDeclaration();
+      ol.startMemberItem(dd->getOutputFileBase(),0);
       ol.parseText(theTranslator->trDir(FALSE,TRUE));
       ol.insertMemberAlign();
       ol.writeObjectLink(dd->getReference(),dd->getOutputFileBase(),0,dd->shortName());
@@ -730,10 +761,11 @@ void GroupDef::writeDirs(OutputList &ol,const QCString &title)
       }
       if (!dd->briefDescription().isEmpty() && Config_getBool("BRIEF_MEMBER_DESC"))
       {
-        ol.startMemberDescription();
-        ol.parseDoc(briefFile(),briefLine(),dd,0,dd->briefDescription(),FALSE,FALSE);
+        ol.startMemberDescription(dd->getOutputFileBase());
+        ol.parseDoc(briefFile(),briefLine(),dd,0,dd->briefDescription(),FALSE,FALSE,0,TRUE,FALSE);
         ol.endMemberDescription();
       }
+      ol.endMemberDeclaration(0,0);
       dd=dirList->next();
     }
 
@@ -776,7 +808,7 @@ void GroupDef::writePageDocumentation(OutputList &ol)
         ol.endSection(si->label,SectionInfo::Subsection);
       }
       ol.startTextBlock();
-      ol.parseDoc(pd->docFile(),pd->docLine(),pd,0,pd->documentation()+pd->inbodyDocumentation(),TRUE,FALSE);
+      ol.parseDoc(pd->docFile(),pd->docLine(),pd,0,pd->documentation()+pd->inbodyDocumentation(),TRUE,FALSE,0,TRUE,FALSE);
       ol.endTextBlock();
     }
   }
@@ -849,6 +881,7 @@ void GroupDef::writeSummaryLinks(OutputList &ol)
       LayoutDocManager::instance().docEntries(LayoutDocManager::Group));
   LayoutDocEntry *lde;
   bool first=TRUE;
+  SrcLangExt lang = getLanguage();
   for (eli.toFirst();(lde=eli.current());++eli)
   {
     if ((lde->kind()==LayoutDocEntry::GroupClasses && classSDict->declVisible()) || 
@@ -864,7 +897,8 @@ void GroupDef::writeSummaryLinks(OutputList &ol)
                        lde->kind()==LayoutDocEntry::GroupFiles        ? "files"          :
                        lde->kind()==LayoutDocEntry::GroupNestedGroups ? "groups"         :
                        "dirs";
-      writeSummaryLink(ol,label,ls->title,first);
+      ol.writeSummaryLink(0,label,ls->title(lang),first);
+      first=FALSE;
     }
     else if (lde->kind()==LayoutDocEntry::MemberDecl)
     {
@@ -872,7 +906,8 @@ void GroupDef::writeSummaryLinks(OutputList &ol)
       MemberList * ml = getMemberList(lmd->type);
       if (ml && ml->declVisible())
       {
-        writeSummaryLink(ol,ml->listTypeAsString(),lmd->title,first);
+        ol.writeSummaryLink(0,ml->listTypeAsString(),lmd->title(lang),first);
+        first=FALSE;
       }
     }
   }
@@ -887,7 +922,7 @@ void GroupDef::writeDocumentation(OutputList &ol)
 {
   //static bool generateTreeView = Config_getBool("GENERATE_TREEVIEW");
   ol.pushGeneratorState();
-  startFile(ol,getOutputFileBase(),name(),title);
+  startFile(ol,getOutputFileBase(),name(),title,HLI_None);
 
   ol.startHeaderSection();
   writeSummaryLinks(ol);
@@ -903,7 +938,7 @@ void GroupDef::writeDocumentation(OutputList &ol)
 
   if (Doxygen::searchIndex)
   {
-    Doxygen::searchIndex->setCurrentDoc(title,getOutputFileBase());
+    Doxygen::searchIndex->setCurrentDoc(this,anchor(),FALSE);
     static QRegExp we("[a-zA-Z_][-a-zA-Z_0-9]*");
     int i=0,p=0,l=0;
     while ((i=we.match(title,p,&l))!=-1) // foreach word in the title
@@ -926,6 +961,7 @@ void GroupDef::writeDocumentation(OutputList &ol)
 
   //---------------------------------------- start flexible part -------------------------------
 
+  SrcLangExt lang=getLanguage();
   QListIterator<LayoutDocEntry> eli(
       LayoutDocManager::instance().docEntries(LayoutDocManager::Group));
   LayoutDocEntry *lde;
@@ -942,7 +978,7 @@ void GroupDef::writeDocumentation(OutputList &ol)
       case LayoutDocEntry::GroupClasses: 
         {
           LayoutDocEntrySection *ls = (LayoutDocEntrySection*)lde;
-          writeClasses(ol,ls->title);
+          writeClasses(ol,ls->title(lang));
         }
         break; 
       case LayoutDocEntry::GroupInlineClasses: 
@@ -953,7 +989,7 @@ void GroupDef::writeDocumentation(OutputList &ol)
       case LayoutDocEntry::GroupNamespaces: 
         {
           LayoutDocEntrySection *ls = (LayoutDocEntrySection*)lde;
-          writeNamespaces(ol,ls->title);
+          writeNamespaces(ol,ls->title(lang));
         }
         break; 
       case LayoutDocEntry::MemberGroups: 
@@ -962,7 +998,7 @@ void GroupDef::writeDocumentation(OutputList &ol)
       case LayoutDocEntry::MemberDecl: 
         {
           LayoutDocEntryMemberDecl *lmd = (LayoutDocEntryMemberDecl*)lde;
-          writeMemberDeclarations(ol,lmd->type,lmd->title);
+          writeMemberDeclarations(ol,lmd->type,lmd->title(lang));
         }
         break; 
       case LayoutDocEntry::MemberDeclEnd: 
@@ -971,7 +1007,7 @@ void GroupDef::writeDocumentation(OutputList &ol)
       case LayoutDocEntry::DetailedDesc: 
         {
           LayoutDocEntrySection *ls = (LayoutDocEntrySection*)lde;
-          writeDetailedDescription(ol,ls->title);
+          writeDetailedDescription(ol,ls->title(lang));
         }
         break;
       case LayoutDocEntry::MemberDefStart: 
@@ -980,7 +1016,7 @@ void GroupDef::writeDocumentation(OutputList &ol)
       case LayoutDocEntry::MemberDef: 
         {
           LayoutDocEntryMemberDef *lmd = (LayoutDocEntryMemberDef*)lde;
-          writeMemberDocumentation(ol,lmd->type,lmd->title);
+          writeMemberDocumentation(ol,lmd->type,lmd->title(lang));
         }
         break;
       case LayoutDocEntry::MemberDefEnd: 
@@ -989,7 +1025,7 @@ void GroupDef::writeDocumentation(OutputList &ol)
       case LayoutDocEntry::GroupNestedGroups: 
         {
           LayoutDocEntrySection *ls = (LayoutDocEntrySection*)lde;
-          writeNestedGroups(ol,ls->title);
+          writeNestedGroups(ol,ls->title(lang));
         }
         break;
       case LayoutDocEntry::GroupPageDocs: 
@@ -998,13 +1034,13 @@ void GroupDef::writeDocumentation(OutputList &ol)
       case LayoutDocEntry::GroupDirs: 
         {
           LayoutDocEntrySection *ls = (LayoutDocEntrySection*)lde;
-          writeDirs(ol,ls->title);
+          writeDirs(ol,ls->title(lang));
         }
         break;
       case LayoutDocEntry::GroupFiles: 
         {
           LayoutDocEntrySection *ls = (LayoutDocEntrySection*)lde;
-          writeFiles(ol,ls->title);
+          writeFiles(ol,ls->title(lang));
         }
         break;
       case LayoutDocEntry::GroupGraph: 
@@ -1105,7 +1141,7 @@ void GroupDef::writeQuickMemberLinks(OutputList &ol,MemberDef *currentMd) const
         if (createSubDirs) ol.writeString("../../");
         ol.writeString(md->getOutputFileBase()+Doxygen::htmlFileExtension+"#"+md->anchor());
         ol.writeString("\">");
-        ol.writeString(md->localName());
+        ol.writeString(convertToHtml(md->localName()));
         ol.writeString("</a>");
       }
       ol.writeString("</td></tr>\n");
@@ -1422,7 +1458,7 @@ void GroupDef::writeMemberDeclarations(OutputList &ol,MemberList::ListType lt,co
   MemberList * ml = getMemberList(lt);
   if (optimizeVhdl && ml) 
   {
-    VhdlDocGen::writeVhdlDeclarations(ml,ol,this,0,0);
+    VhdlDocGen::writeVhdlDeclarations(ml,ol,this,0,0,0);
     return;
   }
   if (ml) 
@@ -1443,3 +1479,28 @@ void GroupDef::removeMemberFromList(MemberList::ListType lt,MemberDef *md)
     if (ml) ml->remove(md); 
 }
 
+void GroupDef::sortSubGroups() 
+{ 
+    groupList->sort(); 
+}
+
+bool GroupDef::isLinkableInProject() const
+{
+  return !isReference() && isLinkable();
+}
+
+bool GroupDef::isLinkable() const
+{
+  return hasUserDocumentation();
+}
+
+// let the "programming language" for a group depend on what is inserted into it.
+// First item that has an associated languages determines the language for the whole group.
+void GroupDef::updateLanguage(const Definition *d)
+{
+  if (getLanguage()==SrcLangExt_Unknown && d->getLanguage()!=SrcLangExt_Unknown)
+  {
+    setLanguage(d->getLanguage());
+  }
+}
+
