@@ -2,7 +2,7 @@
  *
  * $Id: groupdef.h,v 1.18 2001/03/19 19:27:40 root Exp $
  *
- * Copyright (C) 1997-2005 by Dimitri van Heesch.
+ * Copyright (C) 1997-2012 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -39,20 +39,25 @@ class PageSDict;
 class PageDef;
 class DirDef;
 class DirList;
+class FTVHelp;
+class Entry;
 
+/** A model of a group of symbols. */
 class GroupDef : public Definition
 {
   public:
     GroupDef(const char *fileName,int line,const char *name,const char *title,const char *refFileName=0);
    ~GroupDef();
-    DefType definitionType() { return TypeGroup; }
+    DefType definitionType() const { return TypeGroup; }
     QCString getOutputFileBase() const;
+    QCString anchor() const { return QCString(); }
+    QCString displayName(bool=TRUE) const { return hasGroupTitle() ? title : Definition::name(); }
     const char *groupTitle() const { return title; }
     void setGroupTitle( const char *newtitle );
-    bool hasGroupTitle( ) { return titleSet; }
+    bool hasGroupTitle( ) const { return titleSet; }
     void addFile(const FileDef *def); 
-    void addClass(const ClassDef *def);
-    void addNamespace(const NamespaceDef *def);
+    bool addClass(const ClassDef *def);
+    bool addNamespace(const NamespaceDef *def);
     void addGroup(const GroupDef *def);
     void addParentGroup(const GroupDef *def);
     void addPage(PageDef *def);
@@ -61,17 +66,12 @@ class GroupDef : public Definition
     bool insertMember(MemberDef *def,bool docOnly=FALSE);
     void removeMember(MemberDef *md);
     bool containsGroup(const GroupDef *def);    // true if def is already a subgroup
-    void writeDetailedDocumentation(OutputList &ol);
     void writeDocumentation(OutputList &ol);
-    int countMembers() const;
-    bool isLinkableInProject() const
-    {
-      return !isReference();
-    }
-    bool isLinkable() const
-    {
-      return TRUE;
-    }
+    void writeMemberPages(OutputList &ol);
+    void writeQuickMemberLinks(OutputList &ol,MemberDef *currentMd) const;
+    int  countMembers() const;
+    bool isLinkableInProject() const;
+    bool isLinkable() const;
     bool isASubGroup() const;
     void computeAnchors();
 
@@ -80,31 +80,21 @@ class GroupDef : public Definition
     void findSectionsInDocumentation();
 
     void addListReferences();
+    void sortMemberLists();
 
     bool visited;    // number of times accessed for output - KPW
 
-    friend void writeGroupTreeNode(OutputList&, GroupDef*, int);      
+    //friend void writeGroupTreeNode(OutputList&, GroupDef*, int, FTVHelp*);      
                     // make accessible for writing tree view of group in index.cpp - KPW
 
-    // members in the declaration part of the documentation
-    MemberList decDefineMembers;
-    MemberList decProtoMembers;
-    MemberList decTypedefMembers;
-    MemberList decEnumMembers;
-    MemberList decEnumValMembers;
-    MemberList decFuncMembers;
-    MemberList decVarMembers;
+    void setGroupScope(Definition *d) { groupScope = d; }
+    Definition *getGroupScope() const { return groupScope; }
 
-    // members in the documentation part of the documentation
-    MemberList docDefineMembers;
-    MemberList docProtoMembers;
-    MemberList docTypedefMembers;
-    MemberList docEnumMembers;
-    MemberList docFuncMembers;
-    MemberList docVarMembers;
+    MemberList *getMemberList(MemberList::ListType lt) const;
+    const QList<MemberList> &getMemberLists() const { return m_memberLists; }
 
     /* user defined member groups */
-    MemberGroupSDict *memberGroupSDict;
+    MemberGroupSDict *getMemberGroupSDict() const { return memberGroupSDict; }
 
     FileList *      getFiles() const        { return fileList; }
     ClassSDict *    getClasses() const      { return classSDict; }
@@ -112,12 +102,38 @@ class GroupDef : public Definition
     GroupList *     getSubGroups() const    { return groupList; }
     PageSDict *     getPages() const        { return pageDict; }
     DirList *       getDirs() const         { return dirList; }
-    MemberList*     getMembers() const      { return allMemberList; }
+    PageSDict *     getExamples() const     { return exampleDict; }
+    //MemberList*     getMembers() const      { return allMemberList; }
+    void sortSubGroups();
     
   protected:
     void addMemberListToGroup(MemberList *,bool (MemberDef::*)() const);
 
   private: 
+    MemberList *createMemberList(MemberList::ListType lt);
+    void addMemberToList(MemberList::ListType lt,MemberDef *md);
+    void writeMemberDeclarations(OutputList &ol,MemberList::ListType lt,const QCString &title);
+    void writeMemberDocumentation(OutputList &ol,MemberList::ListType lt,const QCString &title);
+    void removeMemberFromList(MemberList::ListType lt,MemberDef *md);
+    void writeGroupGraph(OutputList &ol);
+    void writeFiles(OutputList &ol,const QCString &title);
+    void writeNamespaces(OutputList &ol,const QCString &title);
+    void writeNestedGroups(OutputList &ol,const QCString &title);
+    void writeDirs(OutputList &ol,const QCString &title);
+    void writeClasses(OutputList &ol,const QCString &title);
+    void writeInlineClasses(OutputList &ol);
+    void writePageDocumentation(OutputList &ol);
+    void writeDetailedDescription(OutputList &ol,const QCString &title);
+    void writeBriefDescription(OutputList &ol);
+    void writeMemberGroups(OutputList &ol);
+    void startMemberDeclarations(OutputList &ol);
+    void endMemberDeclarations(OutputList &ol);
+    void startMemberDocumentation(OutputList &ol);
+    void endMemberDocumentation(OutputList &ol);
+    void writeAuthorSection(OutputList &ol);
+    void writeSummaryLinks(OutputList &ol);
+    void updateLanguage(const Definition *);
+
     QCString title;                      // title of the group
     bool titleSet;                       // true if title is not the same as the name
     QCString fileName;                   // base name of the generated file
@@ -132,20 +148,36 @@ class GroupDef : public Definition
     MemberList *allMemberList;
     MemberNameInfoSDict *allMemberNameInfoSDict;
     
+    Definition *groupScope;
+
+    QList<MemberList> m_memberLists;
+    MemberGroupSDict *memberGroupSDict;
 
 };
 
+/** A sorted dictionary of GroupDef objects. */
 class GroupSDict : public SDict<GroupDef>
 {
   public:
     GroupSDict(uint size) : SDict<GroupDef>(size) {}
     virtual ~GroupSDict() {}
+    int compareItems(GCI item1,GCI item2)
+    {
+      return strcmp(((GroupDef*)item1)->groupTitle(),((GroupDef*)item2)->groupTitle());
+    }
 };
 
+/** A list of GroupDef objects. */
 class GroupList : public QList<GroupDef>
 {
+  public:
+    int compareItems(GCI item1,GCI item2)
+    {
+      return strcmp(((GroupDef*)item1)->groupTitle(),((GroupDef*)item2)->groupTitle());
+    }
 };
 
+/** An iterator for GroupDef objects in a GroupList. */
 class GroupListIterator : public QListIterator<GroupDef>
 {
   public:
