@@ -3,7 +3,7 @@
  * $Id: $
  *
  *
- * Copyright (C) 1997-2011 by Dimitri van Heesch.
+ * Copyright (C) 1997-2012 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -83,8 +83,8 @@ static bool mustBeOutsideParagraph(DocNode *n)
           /* \internal */
         case DocNode::Kind_Internal:
           /* <div> */
-        case DocNode::Kind_Verbatim:
         case DocNode::Kind_Include:
+        case DocNode::Kind_Verbatim:
         case DocNode::Kind_Image:
         case DocNode::Kind_SecRefList:
           /* <hr> */
@@ -94,6 +94,8 @@ static bool mustBeOutsideParagraph(DocNode *n)
            * preserve formatting.
            */
         case DocNode::Kind_Copy:
+          /* <blockquote> */
+        case DocNode::Kind_HtmlBlockQuote:
           return TRUE;
         case DocNode::Kind_StyleChange:
           return ((DocStyleChange*)n)->style()==DocStyleChange::Preformatted ||
@@ -107,8 +109,6 @@ static bool mustBeOutsideParagraph(DocNode *n)
   return FALSE;
 }
 
-
-
 static QString htmlAttribsToString(const HtmlAttribList &attribs)
 {
   QString result;
@@ -130,10 +130,11 @@ static QString htmlAttribsToString(const HtmlAttribList &attribs)
 //-------------------------------------------------------------------------
 
 HtmlDocVisitor::HtmlDocVisitor(FTextStream &t,CodeOutputInterface &ci,
-                               const char *langExt) 
+                               Definition *ctx,MemberDef *md) 
   : DocVisitor(DocVisitor_Html), m_t(t), m_ci(ci), m_insidePre(FALSE), 
-                                 m_hide(FALSE), m_langExt(langExt)
+                                 m_hide(FALSE), m_ctx(ctx), m_md(md)
 {
+  if (ctx) m_langExt=ctx->getDefFileExtension();
 }
 
   //--------------------------------------
@@ -173,51 +174,149 @@ void HtmlDocVisitor::visit(DocSymbol *s)
   if (m_hide) return;
   switch(s->symbol())
   {
-    case DocSymbol::BSlash:  m_t << "\\"; break;
-    case DocSymbol::At:      m_t << "@"; break;
-    case DocSymbol::Less:    m_t << "&lt;"; break;
-    case DocSymbol::Greater: m_t << "&gt;"; break;
-    case DocSymbol::Amp:     m_t << "&amp;"; break;
-    case DocSymbol::Dollar:  m_t << "$"; break;
-    case DocSymbol::Hash:    m_t << "#"; break;
-    case DocSymbol::DoubleColon: m_t << "::"; break;
-    case DocSymbol::Percent: m_t << "%"; break;
-    case DocSymbol::Copy:    m_t << "&copy;"; break;
-    case DocSymbol::Tm:      m_t << "&trade;"; break;
-    case DocSymbol::Reg:     m_t << "&reg;"; break;
-    case DocSymbol::Apos:    m_t << "'"; break;
-    case DocSymbol::Quot:    m_t << "\""; break;
-    case DocSymbol::Lsquo:   m_t << "&lsquo;"; break;
-    case DocSymbol::Rsquo:   m_t << "&rsquo;"; break;
-    case DocSymbol::Ldquo:   m_t << "&ldquo;"; break;
-    case DocSymbol::Rdquo:   m_t << "&rdquo;"; break;
-    case DocSymbol::Ndash:   m_t << "&ndash;"; break;
-    case DocSymbol::Mdash:   m_t << "&mdash;"; break;
-    case DocSymbol::Uml:     m_t << "&" << s->letter() << "uml;"; break;
-    case DocSymbol::Acute:   m_t << "&" << s->letter() << "acute;"; break;
-    case DocSymbol::Grave:   m_t << "&" << s->letter() << "grave;"; break;
-    case DocSymbol::Circ:    m_t << "&" << s->letter() << "circ;"; break;
-    case DocSymbol::Slash:   m_t << "&" << s->letter() << "slash;"; break;
-    case DocSymbol::Tilde:   m_t << "&" << s->letter() << "tilde;"; break;
-    case DocSymbol::Szlig:   m_t << "&szlig;"; break;
-    case DocSymbol::Cedil:   m_t << "&" << s->letter() << "cedil;"; break;
-    case DocSymbol::Ring:    m_t << "&" << s->letter() << "ring;"; break;
-    case DocSymbol::Nbsp:    m_t << "&#160;"; break;
-    case DocSymbol::AElig:   m_t << "&AElig;"; break;
-    case DocSymbol::Aelig:   m_t << "&aelig;"; break;
+    case DocSymbol::BSlash:        m_t << "\\"; break;
+    case DocSymbol::At:            m_t << "@"; break;
+    case DocSymbol::Less:          m_t << "&lt;"; break;
+    case DocSymbol::Greater:       m_t << "&gt;"; break;
+    case DocSymbol::Amp:           m_t << "&amp;"; break;
+    case DocSymbol::Dollar:        m_t << "$"; break;
+    case DocSymbol::Hash:          m_t << "#"; break;
+    case DocSymbol::DoubleColon:   m_t << "::"; break;
+    case DocSymbol::Percent:       m_t << "%"; break;
+    case DocSymbol::Pipe:          m_t << "|"; break;
+    case DocSymbol::Copy:          m_t << "&copy;"; break;
+    case DocSymbol::Tm:            m_t << "&trade;"; break;
+    case DocSymbol::Reg:           m_t << "&reg;"; break;
+    case DocSymbol::Apos:          m_t << "'"; break;
+    case DocSymbol::Quot:          m_t << "\""; break;
+    case DocSymbol::Lsquo:         m_t << "&lsquo;"; break;
+    case DocSymbol::Rsquo:         m_t << "&rsquo;"; break;
+    case DocSymbol::Ldquo:         m_t << "&ldquo;"; break;
+    case DocSymbol::Rdquo:         m_t << "&rdquo;"; break;
+    case DocSymbol::Ndash:         m_t << "&ndash;"; break;
+    case DocSymbol::Mdash:         m_t << "&mdash;"; break;
+    case DocSymbol::Uml:           m_t << "&" << s->letter() << "uml;"; break;
+    case DocSymbol::Acute:         m_t << "&" << s->letter() << "acute;"; break;
+    case DocSymbol::Grave:         m_t << "&" << s->letter() << "grave;"; break;
+    case DocSymbol::Circ:          m_t << "&" << s->letter() << "circ;"; break;
+    case DocSymbol::Slash:         m_t << "&" << s->letter() << "slash;"; break;
+    case DocSymbol::Tilde:         m_t << "&" << s->letter() << "tilde;"; break;
+    case DocSymbol::Szlig:         m_t << "&szlig;"; break;
+    case DocSymbol::Cedil:         m_t << "&" << s->letter() << "cedil;"; break;
+    case DocSymbol::Ring:          m_t << "&" << s->letter() << "ring;"; break;
+    case DocSymbol::Nbsp:          m_t << "&#160;"; break;
+    case DocSymbol::AElig:         m_t << "&AElig;"; break;
+    case DocSymbol::Aelig:         m_t << "&aelig;"; break;
+    case DocSymbol::GrkGamma:      m_t << "&Gamma;"; break;
+    case DocSymbol::GrkDelta:      m_t << "&Delta;"; break;
+    case DocSymbol::GrkTheta:      m_t << "&Theta;"; break;
+    case DocSymbol::GrkLambda:     m_t << "&Lambda;"; break;
+    case DocSymbol::GrkXi:         m_t << "&Xi;"; break;
+    case DocSymbol::GrkPi:         m_t << "&Pi;"; break;
+    case DocSymbol::GrkSigma:      m_t << "&Sigma;"; break;
+    case DocSymbol::GrkUpsilon:    m_t << "&Upsilon;"; break;
+    case DocSymbol::GrkPhi:        m_t << "&Phi;"; break;
+    case DocSymbol::GrkPsi:        m_t << "&Psi;"; break;
+    case DocSymbol::GrkOmega:      m_t << "&Omega;"; break;
+    case DocSymbol::Grkalpha:      m_t << "&alpha;"; break;
+    case DocSymbol::Grkbeta:       m_t << "&beta;"; break;
+    case DocSymbol::Grkgamma:      m_t << "&gamma;"; break;
+    case DocSymbol::Grkdelta:      m_t << "&delta;"; break;
+    case DocSymbol::Grkepsilon:    m_t << "&epsilon;"; break;
+    case DocSymbol::Grkzeta:       m_t << "&zeta;"; break;
+    case DocSymbol::Grketa:        m_t << "&eta;"; break;
+    case DocSymbol::Grktheta:      m_t << "&theta;"; break;
+    case DocSymbol::Grkiota:       m_t << "&iota;"; break;
+    case DocSymbol::Grkkappa:      m_t << "&kappa;"; break;
+    case DocSymbol::Grklambda:     m_t << "&lambda;"; break;
+    case DocSymbol::Grkmu:         m_t << "&mu;"; break;
+    case DocSymbol::Grknu:         m_t << "&nu;"; break;
+    case DocSymbol::Grkxi:         m_t << "&xi;"; break;
+    case DocSymbol::Grkpi:         m_t << "&pi;"; break;
+    case DocSymbol::Grkrho:        m_t << "&rho;"; break;
+    case DocSymbol::Grksigma:      m_t << "&sigma;"; break;
+    case DocSymbol::Grktau:        m_t << "&tau;"; break;
+    case DocSymbol::Grkupsilon:    m_t << "&upsilon;"; break;
+    case DocSymbol::Grkphi:        m_t << "&phi;"; break;
+    case DocSymbol::Grkchi:        m_t << "&chi;"; break;
+    case DocSymbol::Grkpsi:        m_t << "&psi;"; break;
+    case DocSymbol::Grkomega:      m_t << "&omega;"; break;
+    case DocSymbol::Grkvarsigma:   m_t << "&sigmaf;"; break;
+    case DocSymbol::Section:       m_t << "&sect;"; break;
+    case DocSymbol::Degree:        m_t << "&deg;"; break;
+    case DocSymbol::Prime:         m_t << "&prime;"; break;
+    case DocSymbol::DoublePrime:   m_t << "&Prime;"; break;
+    case DocSymbol::Infinity:      m_t << "&infin;"; break;
+    case DocSymbol::EmptySet:      m_t << "&empty;"; break;
+    case DocSymbol::PlusMinus:     m_t << "&plusmn;"; break;
+    case DocSymbol::Times:         m_t << "&times;"; break;
+    case DocSymbol::Minus:         m_t << "&minus;"; break;
+    case DocSymbol::CenterDot:     m_t << "&sdot;"; break;
+    case DocSymbol::Partial:       m_t << "&part;"; break;
+    case DocSymbol::Nabla:         m_t << "&nabla;"; break;
+    case DocSymbol::SquareRoot:    m_t << "&radic;"; break;
+    case DocSymbol::Perpendicular: m_t << "&perp;"; break;
+    case DocSymbol::Sum:           m_t << "&sum;"; break;
+    case DocSymbol::Integral:      m_t << "&int;"; break;
+    case DocSymbol::Product:       m_t << "&prod;"; break;
+    case DocSymbol::Similar:       m_t << "&sim;"; break;
+    case DocSymbol::Approx:        m_t << "&asymp;"; break;
+    case DocSymbol::NotEqual:      m_t << "&ne;"; break;
+    case DocSymbol::Equivalent:    m_t << "&equiv;"; break;
+    case DocSymbol::Proportional:  m_t << "&prop;"; break;
+    case DocSymbol::LessEqual:     m_t << "&le;"; break;
+    case DocSymbol::GreaterEqual:  m_t << "&ge;"; break;
+    case DocSymbol::LeftArrow:     m_t << "&larr;"; break;
+    case DocSymbol::RightArrow:    m_t << "&rarr;"; break;
+    case DocSymbol::SetIn:         m_t << "&isin;"; break;
+    case DocSymbol::SetNotIn:      m_t << "&notin;"; break;
+    case DocSymbol::LeftCeil:      m_t << "&lceil;"; break;
+    case DocSymbol::RightCeil:     m_t << "&rceil;"; break;
+    case DocSymbol::LeftFloor:     m_t << "&lfloor;"; break;
+    case DocSymbol::RightFloor:    m_t << "&rfloor;"; break;
     default:
-                             err("error: unknown symbol found\n");
+       err("error: unknown symbol found\n");
   }
 }
 
+void HtmlDocVisitor::writeObfuscatedMailAddress(const QCString &url)
+{
+  m_t << "<a href=\"#\" onclick=\"location.href='mai'+'lto:'";
+  uint i;
+  int size=3;
+  for (i=0;i<url.length();)
+  {
+    m_t << "+'" << url.mid(i,size) << "'";
+    i+=size;
+    if (size==3) size=2; else size=3;
+  }
+  m_t << "; return false;\">";
+}
+
 void HtmlDocVisitor::visit(DocURL *u)
 {
   if (m_hide) return;
-  m_t << "<a href=\"";
-  if (u->isEmail()) m_t << "mailto:";
-  m_t << u->url() << "\">";
-  filter(u->url());
-  m_t << "</a>";
+  if (u->isEmail()) // mail address
+  {
+    QCString url = u->url();
+    writeObfuscatedMailAddress(url);
+    uint size=5,i;
+    for (i=0;i<url.length();)
+    {
+      filter(url.mid(i,size));
+      if (i<url.length()-size) m_t << "<span style=\"display: none;\">.nosp@m.</span>";
+      i+=size;
+      if (size==5) size=4; else size=5;
+    }
+    m_t << "</a>";
+  }
+  else // web address
+  {
+    m_t << "<a href=\"";
+    m_t << u->url() << "\">";
+    filter(u->url());
+    m_t << "</a>";
+  }
 }
 
 void HtmlDocVisitor::visit(DocLineBreak *)
@@ -304,22 +403,38 @@ void HtmlDocVisitor::visit(DocStyleChange *s)
 void HtmlDocVisitor::visit(DocVerbatim *s)
 {
   if (m_hide) return;
+  QCString lang = m_langExt;
+  if (!s->language().isEmpty()) // explicit language setting
+  {
+    lang = s->language();
+  }
   switch(s->type())
   {
     case DocVerbatim::Code: 
       forceEndParagraph(s);
       m_t << PREFRAG_START;
-      Doxygen::parserManager->getParser(m_langExt)
-                            ->parseCode(m_ci,s->context(),s->text(),
-                                        s->isExample(),s->exampleFile());
+      Doxygen::parserManager->getParser(lang)
+                            ->parseCode(m_ci,
+                                        s->context(),
+                                        s->text(),
+                                        s->isExample(),
+                                        s->exampleFile(),
+                                        0,     // fileDef
+                                        -1,    // startLine
+                                        -1,    // endLine
+                                        FALSE, // inlineFragment
+                                        0,     // memberDef
+                                        TRUE,  // show line numbers
+                                        m_ctx  // search context
+                                       );
       m_t << PREFRAG_END;
       forceStartParagraph(s);
       break;
     case DocVerbatim::Verbatim: 
       forceEndParagraph(s);
-      m_t << PREFRAG_START;
+      m_t << /*PREFRAG_START <<*/ "<pre class=\"fragment\">";
       filter(s->text());
-      m_t << PREFRAG_END;
+      m_t << "</pre>" /*<< PREFRAG_END*/;
       forceStartParagraph(s);
       break;
     case DocVerbatim::HtmlOnly: 
@@ -328,6 +443,7 @@ void HtmlDocVisitor::visit(DocVerbatim *s)
     case DocVerbatim::ManOnly: 
     case DocVerbatim::LatexOnly: 
     case DocVerbatim::XmlOnly: 
+    case DocVerbatim::RtfOnly: 
       /* nothing */ 
       break;
 
@@ -410,10 +526,13 @@ void HtmlDocVisitor::visit(DocInclude *inc)
                                         inc->text(),
                                         inc->isExample(),
                                         inc->exampleFile(),
-                                        0,   // fd
-                                        -1,  // startLine
-                                        -1,  // endLine
-                                        TRUE // inlineFragment
+                                        0,     // fileDef
+                                        -1,    // startLine
+                                        -1,    // endLine
+                                        TRUE,  // inlineFragment
+                                        0,     // memberDef
+                                        TRUE,  // show line numbers
+                                        m_ctx  // search context 
                                        );
       m_t << PREFRAG_END;
       forceStartParagraph(inc);
@@ -423,13 +542,21 @@ void HtmlDocVisitor::visit(DocInclude *inc)
          forceEndParagraph(inc);
          m_t << PREFRAG_START;
          QFileInfo cfi( inc->file() );
-         FileDef fd( cfi.dirPath(), cfi.fileName() );
+         FileDef fd( cfi.dirPath().utf8(), cfi.fileName().utf8() );
          Doxygen::parserManager->getParser(inc->extension())
                                ->parseCode(m_ci,
                                            inc->context(),
                                            inc->text(),
                                            inc->isExample(),
-                                           inc->exampleFile(), &fd);
+                                           inc->exampleFile(), 
+                                           &fd,   // fileDef,
+                                           -1,    // start line
+                                           -1,    // end line
+                                           FALSE, // inline fragment
+                                           0,     // memberDef
+                                           TRUE,  // show line numbers
+                                           m_ctx  // search context
+                                           );
          m_t << PREFRAG_END;
          forceStartParagraph(inc);
       }
@@ -441,9 +568,9 @@ void HtmlDocVisitor::visit(DocInclude *inc)
       break;
     case DocInclude::VerbInclude: 
       forceEndParagraph(inc);
-      m_t << PREFRAG_START;
+      m_t << /*PREFRAG_START <<*/ "<pre class=\"fragment\">";
       filter(inc->text());
-      m_t << PREFRAG_END;
+      m_t << "</pre>" /*<< PREFRAG_END*/;
       forceStartParagraph(inc);
       break;
     case DocInclude::Snippet:
@@ -457,9 +584,12 @@ void HtmlDocVisitor::visit(DocInclude *inc)
                                            inc->isExample(),
                                            inc->exampleFile(), 
                                            0,
-                                           -1,  // startLine
-                                           -1,  // endLine
-                                           TRUE // inlineFragment
+                                           -1,    // startLine
+                                           -1,    // endLine
+                                           TRUE,  // inlineFragment
+                                           0,     // memberDef
+                                           TRUE,  // show line number
+                                           m_ctx  // search context
                                           );
          m_t << PREFRAG_END;
          forceStartParagraph(inc);
@@ -484,9 +614,20 @@ void HtmlDocVisitor::visit(DocIncOperator *op)
     if (!m_hide) 
     {
       Doxygen::parserManager->getParser(m_langExt)
-                            ->parseCode(m_ci,op->context(),
-                                op->text(),op->isExample(),
-                                op->exampleFile());
+                            ->parseCode(
+                                m_ci,
+                                op->context(),
+                                op->text(),
+                                op->isExample(),
+                                op->exampleFile(),
+                                0,     // fileDef
+                                -1,    // startLine
+                                -1,    // endLine
+                                FALSE, // inline fragment
+                                0,     // memberDef
+                                TRUE,  // show line numbers
+                                m_ctx  // search context
+                               );
     }
     pushEnabled();
     m_hide=TRUE;
@@ -536,8 +677,8 @@ void HtmlDocVisitor::visit(DocFormula *f)
     m_t << "\" alt=\"";
     filterQuotedCdataAttr(f->text());
     m_t << "\"";
-    /// @todo cache image dimensions on formula generation and give height/width
-    /// for faster preloading and better rendering of the page
+    // TODO: cache image dimensions on formula generation and give height/width
+    // for faster preloading and better rendering of the page
     m_t << " src=\"" << f->relPath() << f->name() << ".png\"/>";
 
   }
@@ -704,8 +845,10 @@ static int getParagraphContext(DocPara *p,bool &isFirst,bool &isLast)
     switch (p->parent()->kind()) 
     {
       case DocNode::Kind_AutoListItem:
-        isFirst=TRUE;
-        isLast =TRUE;
+        //isFirst=TRUE;
+        //isLast =TRUE;
+        isFirst=isFirstChildNode((DocAutoListItem*)p->parent(),p);
+        isLast =isLastChildNode ((DocAutoListItem*)p->parent(),p);
         t=1; // not used
         break;
       case DocNode::Kind_SimpleListItem:
@@ -792,6 +935,7 @@ void HtmlDocVisitor::visitPre(DocPara *p)
       case DocNode::Kind_SimpleSect:
       case DocNode::Kind_XRefItem:
       case DocNode::Kind_Copy:
+      case DocNode::Kind_HtmlBlockQuote:
         needsTag = TRUE;
         break;
       case DocNode::Kind_Root:
@@ -849,16 +993,6 @@ void HtmlDocVisitor::visitPre(DocPara *p)
 
 void HtmlDocVisitor::visitPost(DocPara *p)
 {
-//  if (m_hide) return;
-//  if (!p->isLast() &&            // omit <p> for last paragraph
-//      !(p->parent() &&           // and for parameter sections
-//        p->parent()->kind()==DocNode::Kind_ParamSect
-//       ) 
-//     ) 
-//  {
-//    m_t << "<p>\n";
-//  }
-
   bool needsTag = FALSE;
   if (p && p->parent()) 
   {
@@ -874,6 +1008,7 @@ void HtmlDocVisitor::visitPost(DocPara *p)
       case DocNode::Kind_SimpleSect:
       case DocNode::Kind_XRefItem:
       case DocNode::Kind_Copy:
+      case DocNode::Kind_HtmlBlockQuote:
         needsTag = TRUE;
         break;
       case DocNode::Kind_Root:
@@ -929,7 +1064,7 @@ void HtmlDocVisitor::visitPre(DocSimpleSect *s)
 {
   if (m_hide) return;
   forceEndParagraph(s);
-  m_t << "<dl class=\"" << s->typeString() << "\"><dt><b>";
+  m_t << "<dl class=\"section " << s->typeString() << "\"><dt>";
   switch(s->type())
   {
     case DocSimpleSect::See: 
@@ -970,7 +1105,7 @@ void HtmlDocVisitor::visitPre(DocSimpleSect *s)
   // special case 1: user defined title
   if (s->type()!=DocSimpleSect::User && s->type()!=DocSimpleSect::Rcs)
   {
-    m_t << ":</b></dt><dd>";
+    m_t << "</dt><dd>";
   }
 }
 
@@ -988,7 +1123,7 @@ void HtmlDocVisitor::visitPre(DocTitle *)
 void HtmlDocVisitor::visitPost(DocTitle *)
 {
   if (m_hide) return;
-  m_t << "</b></dt><dd>";
+  m_t << "</dt><dd>";
 }
 
 void HtmlDocVisitor::visitPre(DocSimpleList *sl)
@@ -1025,11 +1160,11 @@ void HtmlDocVisitor::visitPre(DocSection *s)
 {
   if (m_hide) return;
   forceEndParagraph(s);
-  m_t << "<h" << s->level()+1 << ">";
+  m_t << "<h" << s->level() << ">";
   m_t << "<a class=\"anchor\" id=\"" << s->anchor();
   m_t << "\"></a>" << endl;
   filter(convertCharEntitiesToUTF8(s->title().data()));
-  m_t << "</h" << s->level()+1 << ">\n";
+  m_t << "</h" << s->level() << ">\n";
 }
 
 void HtmlDocVisitor::visitPost(DocSection *s) 
@@ -1209,8 +1344,16 @@ void HtmlDocVisitor::visitPost(DocInternal *)
 void HtmlDocVisitor::visitPre(DocHRef *href)
 {
   if (m_hide) return;
-  m_t << "<a href=\"" << convertToXML(href->url())  << "\""
-      << htmlAttribsToString(href->attribs()) << ">";
+  if (href->url().left(7)=="mailto:")
+  {
+    writeObfuscatedMailAddress(href->url().mid(7));
+  }
+  else
+  {
+    QCString url = correctURL(href->url(),href->relPath());
+    m_t << "<a href=\"" << convertToXML(url)  << "\""
+        << htmlAttribsToString(href->attribs()) << ">";
+  }
 }
 
 void HtmlDocVisitor::visitPost(DocHRef *) 
@@ -1247,9 +1390,19 @@ void HtmlDocVisitor::visitPre(DocImage *img)
       baseName=baseName.right(baseName.length()-i-1);
     }
     m_t << "<div class=\"image\">" << endl;
-    m_t << "<img src=\"" << img->relPath() << img->name() << "\" alt=\"" 
-        << baseName << "\"" << htmlAttribsToString(img->attribs()) 
-        << "/>" << endl;
+    QCString url = img->url();
+    if (url.isEmpty())
+    {
+      m_t << "<img src=\"" << img->relPath() << img->name() << "\" alt=\"" 
+          << baseName << "\"" << htmlAttribsToString(img->attribs()) 
+          << "/>" << endl;
+    }
+    else
+    {
+      m_t << "<img src=\"" << correctURL(url,img->relPath()) << "\" " 
+          << htmlAttribsToString(img->attribs())
+          << "/>" << endl;
+    }
     if (img->hasCaption())
     {
       m_t << "<div class=\"caption\">" << endl;
@@ -1338,7 +1491,9 @@ void HtmlDocVisitor::visitPre(DocRef *ref)
   if (m_hide) return;
   if (!ref->file().isEmpty()) 
   {
-    startLink(ref->ref(),ref->file(),ref->relPath(),ref->anchor());
+    // when ref->isSubPage()==TRUE we use ref->file() for HTML and
+    // ref->anchor() for LaTeX/RTF
+    startLink(ref->ref(),ref->file(),ref->relPath(),ref->isSubPage() ? QCString() : ref->anchor());
   }
   if (!ref->hasLinkText()) filter(ref->targetTitle());
 }
@@ -1408,33 +1563,32 @@ void HtmlDocVisitor::visitPre(DocParamSect *s)
 {
   if (m_hide) return;
   forceEndParagraph(s);
-  m_t << "<dl><dt><b>";
   QCString className;
+  QCString heading;
   switch(s->type())
   {
     case DocParamSect::Param: 
-      m_t << theTranslator->trParameters(); 
+      heading=theTranslator->trParameters(); 
       className="params";
       break;
     case DocParamSect::RetVal: 
-      m_t << theTranslator->trReturnValues(); 
+      heading=theTranslator->trReturnValues(); 
       className="retval";
       break;
     case DocParamSect::Exception: 
-      m_t << theTranslator->trExceptions(); 
+      heading=theTranslator->trExceptions(); 
       className="exception";
       break;
     case DocParamSect::TemplateParam: 
-      /* TODO: add this 
-      m_t << theTranslator->trTemplateParam(); break;
-      */
-      m_t << "Template Parameters"; break;
+      heading=theTranslator->trTemplateParameters();
       className="tparams";
+      break;
     default:
       ASSERT(0);
   }
-  m_t << ":";
-  m_t << "</b></dt><dd>" << endl;
+  m_t << "<dl class=\"" << className << "\"><dt>";
+  m_t << heading;
+  m_t << "</dt><dd>" << endl;
   m_t << "  <table class=\"" << className << "\">" << endl;
 }
 
@@ -1584,6 +1738,29 @@ void HtmlDocVisitor::visitPost(DocText *)
 {
 }
 
+void HtmlDocVisitor::visitPre(DocHtmlBlockQuote *b)
+{
+  if (m_hide) return;
+  forceEndParagraph(b);
+
+  QString attrs = htmlAttribsToString(b->attribs());
+  if (attrs.isEmpty())
+  {
+    m_t << "<blockquote class=\"doxtable\">\n";
+  }
+  else
+  {
+    m_t << "<blockquote " << htmlAttribsToString(b->attribs()) << ">\n";
+  }
+}
+
+void HtmlDocVisitor::visitPost(DocHtmlBlockQuote *b)
+{
+  if (m_hide) return;
+  m_t << "</blockquote>" << endl;
+  forceStartParagraph(b);
+}
+
 void HtmlDocVisitor::filter(const char *str)
 { 
   if (str==0) return;
@@ -1657,7 +1834,7 @@ void HtmlDocVisitor::startLink(const QCString &ref,const QCString &file,
   if (!file.isEmpty()) m_t << file << Doxygen::htmlFileExtension;
   if (!anchor.isEmpty()) m_t << "#" << anchor;
   m_t << "\"";
-  if (!tooltip.isEmpty()) m_t << " title=\"" << tooltip << "\"";
+  if (!tooltip.isEmpty()) m_t << " title=\"" << substitute(tooltip,"\"","&quot;") << "\"";
   m_t << ">";
 }
 
@@ -1698,7 +1875,8 @@ void HtmlDocVisitor::writeDotFile(const QCString &fn,const QCString &relPath,
   writeDotImageMapFromFile(m_t,fn,outDir,relPath,baseName,context);
 }
 
-void HtmlDocVisitor::writeMscFile(const QCString &fileName,const QCString &relPath,
+void HtmlDocVisitor::writeMscFile(const QCString &fileName,
+                                  const QCString &relPath,
                                   const QCString &context)
 {
   QCString baseName=fileName;
