@@ -3,7 +3,7 @@
  * $Id: $
  *
  *
- * Copyright (C) 1997-2007 by Dimitri van Heesch.
+ * Copyright (C) 1997-2012 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -23,15 +23,17 @@
 #include <qstack.h>
 #include <qcstring.h>
 
-class QTextStream;
+class Definition;
+class MemberDef;
+class DocNode;
+class FTextStream;
 class CodeOutputInterface;
-class QString;
 
 /*! @brief Concrete visitor implementation for HTML output. */
 class HtmlDocVisitor : public DocVisitor
 {
   public:
-    HtmlDocVisitor(QTextStream &t,CodeOutputInterface &ci,const char *langExt);
+    HtmlDocVisitor(FTextStream &t,CodeOutputInterface &ci,Definition *ctx,MemberDef *md);
     
     //--------------------------------------
     // visitor functions for leaf nodes
@@ -51,6 +53,8 @@ class HtmlDocVisitor : public DocVisitor
     void visit(DocIncOperator *);
     void visit(DocFormula *);
     void visit(DocIndexEntry *);
+    void visit(DocSimpleSectSep *);
+    void visit(DocCite *);
 
     //--------------------------------------
     // visitor functions for compound nodes
@@ -102,6 +106,8 @@ class HtmlDocVisitor : public DocVisitor
     void visitPost(DocImage *);
     void visitPre(DocDotFile *);
     void visitPost(DocDotFile *);
+    void visitPre(DocMscFile *);
+    void visitPost(DocMscFile *);
     void visitPre(DocLink *);
     void visitPost(DocLink *);
     void visitPre(DocRef *);
@@ -122,6 +128,8 @@ class HtmlDocVisitor : public DocVisitor
     void visitPost(DocCopy *);
     void visitPre(DocText *);
     void visitPost(DocText *);
+    void visitPre(DocHtmlBlockQuote *);
+    void visitPost(DocHtmlBlockQuote *);
 
   private:
 
@@ -129,27 +137,33 @@ class HtmlDocVisitor : public DocVisitor
     // helper functions 
     //--------------------------------------
     
+    void writeObfuscatedMailAddress(const QCString &url);
     void filter(const char *str);
     void filterQuotedCdataAttr(const char* str);
-    void startLink(const QString &ref,const QString &file,
-                   const QString &relPath,const QString &anchor,
-                   const QString &tooltip = QString::null);
+    void startLink(const QCString &ref,const QCString &file,
+                   const QCString &relPath,const QCString &anchor,
+                   const QCString &tooltip = "");
     void endLink();
-    void writeDotFile(const QString &fileName,const QString &relPath,const QString &context);
-    void writeMscFile(const QString &fileName,const QString &relPath,const QString &context);
+    void writeDotFile(const QCString &fileName,const QCString &relPath,const QCString &context);
+    void writeMscFile(const QCString &fileName,const QCString &relPath,const QCString &context);
 
     void pushEnabled();
     void popEnabled();
 
+    void forceEndParagraph(DocNode *n);
+    void forceStartParagraph(DocNode *n);
+
     //--------------------------------------
     // state variables
     //--------------------------------------
 
-    QTextStream &m_t;
+    FTextStream &m_t;
     CodeOutputInterface &m_ci;
     bool m_insidePre;
     bool m_hide;
     QStack<bool> m_enabled;
+    Definition *m_ctx;
+    MemberDef *m_md;
     QCString m_langExt;
 };
 
