@@ -2,7 +2,7 @@
  *
  * $Id: htmlgen.h,v 1.51 2001/03/19 19:27:40 root Exp $
  *
- * Copyright (C) 1997-2011 by Dimitri van Heesch.
+ * Copyright (C) 1997-2012 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -20,13 +20,45 @@
 
 #include "qtbc.h"
 #include "outputgen.h"
+#include "ftextstream.h"
 
-#define PREFRAG_START "<div class=\"fragment\"><pre class=\"fragment\">"
-#define PREFRAG_END   "</pre></div>"
+//#define PREFRAG_START "<div class=\"fragment\"><pre class=\"fragment\">"
+//#define PREFRAG_END   "</pre></div>"
+#define PREFRAG_START "<div class=\"fragment\">"
+#define PREFRAG_END   "</div><!-- fragment -->"
 
 class QFile;
-class FTextStream;
 
+class HtmlCodeGenerator : public CodeOutputInterface
+{
+  public:
+    HtmlCodeGenerator(FTextStream &t,const QCString &relPath);
+    HtmlCodeGenerator();
+    void setTextStream(FTextStream &t);
+    void setRelativePath(const QCString &path);
+    void codify(const char *text);
+    void writeCodeLink(const char *ref,const char *file,
+                       const char *anchor,const char *name,
+                       const char *tooltip);
+    void writeLineNumber(const char *,const char *,const char *,int);
+    void startCodeLine(bool);
+    void endCodeLine();
+    void startCodeAnchor(const char *label);
+    void endCodeAnchor();
+    void startFontClass(const char *s);
+    void endFontClass();
+    void writeCodeAnchor(const char *anchor);
+    void linkableSymbol(int,const char *,Definition *,Definition *);
+
+  private:
+    void docify(const char *str);
+    bool m_streamSet;
+    FTextStream m_t;
+    int m_col;
+    QCString m_relPath;
+};
+
+/** Generator for HTML output */
 class HtmlGenerator : public OutputGenerator
 {
   public:
@@ -37,7 +69,7 @@ class HtmlGenerator : public OutputGenerator
     static void writeHeaderFile(QFile &f, const char *cssname);
     static void writeFooterFile(QFile &f);
     static void writeTabData();
-    static void writeSearchFooter(FTextStream &t,const QCString &relPath);
+    static void writeSearchInfo(FTextStream &t,const QCString &relPath);
     static void writeSearchData(const char *dir);
     static void writeSearchPage();
     static QCString writeLogoAsString(const char *path);
@@ -52,12 +84,41 @@ class HtmlGenerator : public OutputGenerator
     bool isEnabled(OutputType o) { return (o==Html && active); } 
     OutputGenerator *get(OutputType o) { return (o==Html) ? this : 0; }
 
-    void printDoc(DocNode *,const char *);
+    // ---- CodeOutputInterface
+    void codify(const char *text) 
+    { m_codeGen.codify(text); }
+    void writeCodeLink(const char *ref,const char *file,
+                       const char *anchor,const char *name,
+                       const char *tooltip)
+    { m_codeGen.writeCodeLink(ref,file,anchor,name,tooltip); }
+    void writeLineNumber(const char *ref,const char *file,const char *anchor,int lineNumber)
+    { m_codeGen.writeLineNumber(ref,file,anchor,lineNumber); }
+    void startCodeLine(bool hasLineNumbers)
+    { m_codeGen.startCodeLine(hasLineNumbers); }
+    void endCodeLine()
+    { m_codeGen.endCodeLine(); }
+    void startCodeAnchor(const char *label)
+    { m_codeGen.startCodeAnchor(label); }
+    void endCodeAnchor()
+    { m_codeGen.endCodeAnchor(); }
+    void startFontClass(const char *s) 
+    { m_codeGen.startFontClass(s); }
+    void endFontClass() 
+    { m_codeGen.endFontClass(); }
+    void writeCodeAnchor(const char *anchor) 
+    { m_codeGen.writeCodeAnchor(anchor); }
+    void linkableSymbol(int line,const char *symName,
+                        Definition *symDef,Definition *context) 
+    { m_codeGen.linkableSymbol(line,symName,symDef,context); }
+    // ---------------------------
+
+    void writeDoc(DocNode *,Definition *,MemberDef *);
 
     void startFile(const char *name,const char *manName,const char *title);
-    void writeFooter();
+    void writeFooter(const char *navPath);
     void endFile();
     void clearBuffer();
+    void writeSearchInfo();
 
     void startIndexSection(IndexSections) {}
     void endIndexSection(IndexSections) {}
@@ -86,12 +147,10 @@ class HtmlGenerator : public OutputGenerator
     void startIndexItem(const char *ref,const char *file);
     void endIndexItem(const char *ref,const char *file);
     void docify(const char *text);
-    void codify(const char *text);
+
     void writeObjectLink(const char *ref,const char *file,
                          const char *anchor,const char *name);
-    void writeCodeLink(const char *ref,const char *file,
-                       const char *anchor,const char *name,
-                       const char *tooltip);
+
     void startTextLink(const char *file,const char *anchor);
     void endTextLink();
     void startHtmlLink(const char *url);
@@ -119,10 +178,10 @@ class HtmlGenerator : public OutputGenerator
     void endInlineHeader();
     void startAnonTypeScope(int) {}
     void endAnonTypeScope(int) {}
-    void startMemberItem(int);
+    void startMemberItem(const char *anchor,int,const char *inheritId);
     void endMemberItem();
     void startMemberTemplateParams();
-    void endMemberTemplateParams();
+    void endMemberTemplateParams(const char *anchor,const char *inheritId);
 
     void startMemberGroupHeader(bool);
     void endMemberGroupHeader();
@@ -132,17 +191,19 @@ class HtmlGenerator : public OutputGenerator
     void endMemberGroup(bool);
 
     void insertMemberAlign(bool);
-    void startMemberDescription();
+    void startMemberDescription(const char *anchor,const char *inheritId);
     void endMemberDescription();
+    void startMemberDeclaration() {}
+    void endMemberDeclaration(const char *anchor,const char *inheritId);
+    void writeInheritedSectionTitle(const char *id,   const char *ref,
+                                    const char *file, const char *anchor,
+                                    const char *title,const char *name);
 
     void writeRuler()    { t << "<hr/>"; }
     void writeAnchor(const char *,const char *name) 
                          { t << "<a name=\"" << name <<"\" id=\"" << name << "\"></a>"; }
     void startCodeFragment() { t << PREFRAG_START; }
     void endCodeFragment()   { t << PREFRAG_END; } 
-    void writeLineNumber(const char *,const char *,const char *,int);
-    void startCodeLine() { col=0; }
-    void endCodeLine()   { codify("\n"); }
     void startEmphasis() { t << "<em>";  }
     void endEmphasis()   { t << "</em>"; }
     void startBold()     { t << "<b>"; }
@@ -161,8 +222,6 @@ class HtmlGenerator : public OutputGenerator
                          const char *anchor,const char *name,
                          const char *args);
     void endDoxyAnchor(const char *fName,const char *anchor);
-    void startCodeAnchor(const char *label) { t << "<a name=\"" << label << "\"></a>"; }
-    void endCodeAnchor() { }
     void writeLatexSpacing() {}
     void writeStartAnnoItem(const char *type,const char *file,
                             const char *path,const char *name);
@@ -195,8 +254,10 @@ class HtmlGenerator : public OutputGenerator
     void startQuickIndices() {}
     void endQuickIndices();
     void writeSplitBar(const char *name);
+    void writeNavigationPath(const char *s);
     void writeLogo();
-    void writeQuickLinks(bool compact,HighlightedItem hli);
+    void writeQuickLinks(bool compact,HighlightedItem hli,const char *file);
+    void writeSummaryLink(const char *file,const char *anchor,const char *title,bool first);
     void startContents();
     void endContents();
     void writeNonBreakableSpace(int);
@@ -261,18 +322,15 @@ class HtmlGenerator : public OutputGenerator
     void startInlineMemberDoc();
     void endInlineMemberDoc();
 
-    void startFontClass(const char *s) { t << "<span class=\"" << s << "\">"; }
-    void endFontClass() { t << "</span>"; }
+    void startLabels();
+    void writeLabel(const char *l,bool isLast);
+    void endLabels();
 
 
-    void writeCodeAnchor(const char *anchor) 
-    { t << "<a name=\"" << anchor << "\"></a>"; }
-    void linkableSymbol(int,const char *,Definition *,Definition *) {}
-
     //static void generateSectionImages();
 
   private:
-    static void writePageFooter(FTextStream &t,const QCString &,const QCString &);
+    static void writePageFooter(FTextStream &t,const QCString &,const QCString &,const QCString &);
     QCString lastTitle;
     QCString lastFile;
     QCString relPath;
@@ -281,9 +339,9 @@ class HtmlGenerator : public OutputGenerator
     HtmlGenerator &operator=(const HtmlGenerator &g);
     HtmlGenerator(const HtmlGenerator &g);
 
-    int col;
     int m_sectionCount;
     bool m_emptySection;
+    HtmlCodeGenerator m_codeGen;
 };
 
 #endif
