@@ -2,7 +2,7 @@
  *
  * $Id: latexgen.cpp,v 1.58 2001/03/19 19:27:40 root Exp $
  *
- * Copyright (C) 1997-2011 by Dimitri van Heesch.
+ * Copyright (C) 1997-2012 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -72,18 +72,10 @@ LatexGenerator::~LatexGenerator()
 {
 }
 
-void LatexGenerator::init()
+static void writeLatexMakefile()
 {
   bool generateBib = !Doxygen::citeDict->isEmpty();
-
   QCString dir=Config_getString("LATEX_OUTPUT");
-  QDir d(dir);
-  if (!d.exists() && !d.mkdir(dir))
-  {
-    err("Could not create output directory %s\n",dir.data());
-    exit(1);
-  }
-  
   QCString fileName=dir+"/Makefile";
   QFile file(fileName);
   if (!file.open(IO_WriteOnly))
@@ -112,13 +104,7 @@ void LatexGenerator::init()
       << "\tdvips -o refman.ps refman.dvi" << endl
       << endl;
     t << "refman.pdf: refman.ps" << endl;
-#if defined(_MSC_VER)
-    // ps2pdf.bat does not work properly from a makefile using GNU make!
-    t << "\tgswin32c -q -dNOPAUSE -dBATCH -sDEVICE=pdfwrite "
-      "-sOutputFile=refman.pdf -c save pop -f refman.ps" << endl << endl;
-#else
     t << "\tps2pdf refman.ps refman.pdf" << endl << endl;
-#endif
     t << "refman.dvi: clean refman.tex doxygen.sty" << endl
       << "\techo \"Running latex...\"" << endl
       << "\t" << latex_command << " refman.tex" << endl
@@ -129,7 +115,7 @@ void LatexGenerator::init()
       t << "\techo \"Running bibtex...\"" << endl;
       t << "\tbibtex refman" << endl;
       t << "\techo \"Rerunning latex....\"" << endl;
-      t << "\tpdflatex refman" << endl;
+      t << "\t" << latex_command << " refman.tex" << endl;
     }
     t << "\techo \"Rerunning latex....\"" << endl
       << "\t" << latex_command << " refman.tex" << endl
@@ -144,13 +130,7 @@ void LatexGenerator::init()
       << "\tpsnup -2 refman.ps >refman_2on1.ps" << endl
       << endl
       << "refman_2on1.pdf: refman_2on1.ps" << endl
-#if defined(_MSC_VER)
-      // ps2pdf.bat does not work properly from a makefile using GNU make!
-      << "\tgswin32c -q -dNOPAUSE -dBATCH -sDEVICE=pdfwrite "
-         "-sOutputFile=refman_2on1.pdf -c save pop -f refman_2on1.ps" << endl;
-#else
       << "\tps2pdf refman_2on1.ps refman_2on1.pdf" << endl;
-#endif
   }
   else // use pdflatex for higher quality output
   {
@@ -158,7 +138,7 @@ void LatexGenerator::init()
       << "pdf: refman.pdf" << endl << endl;
     t << "refman.pdf: clean refman.tex" << endl;
     t << "\tpdflatex refman" << endl;
-    t << "\t" << mkidx_command << " refman.tex" << endl;
+    t << "\t" << mkidx_command << " refman.idx" << endl;
     if (generateBib)
     {
       t << "\tbibtex refman" << endl;
@@ -176,12 +156,98 @@ void LatexGenerator::init()
 
   t << endl
     << "clean:" << endl
-#if defined(_MSC_VER)
-    << "\tdel /s/y " 
-#else
     << "\trm -f " 
-#endif
     << "*.ps *.dvi *.aux *.toc *.idx *.ind *.ilg *.log *.out *.brf *.blg *.bbl refman.pdf" << endl;
+}
+
+static void writeMakeBat()
+{
+#if defined(_MSC_VER)
+  QCString dir=Config_getString("LATEX_OUTPUT");
+  QCString fileName=dir+"/make.bat";
+  QCString latex_command = Config_getString("LATEX_CMD_NAME");
+  QCString mkidx_command = Config_getString("MAKEINDEX_CMD_NAME");
+  QFile file(fileName);
+  bool generateBib = !Doxygen::citeDict->isEmpty();
+  if (!file.open(IO_WriteOnly))
+  {
+    err("Could not open file %s for writing\n",fileName.data());
+    exit(1);
+  }
+  FTextStream t(&file);
+  t << "del /s /f *.ps *.dvi *.aux *.toc *.idx *.ind *.ilg *.log *.out *.brf *.blg *.bbl refman.pdf\n\n";
+  if (!Config_getBool("USE_PDFLATEX")) // use plain old latex
+  {
+    t << latex_command << " refman.tex\n";
+    t << "echo ----\n";
+    t << mkidx_command << " refman.idx\n";
+    if (generateBib)
+    {
+      t << "bibtex refman\n";
+      t << "echo ----\n";
+      t << latex_command << " refman.tex\n";
+    }
+    t << "setlocal enabledelayedexpansion\n";
+    t << "set count=5\n";
+    t << ":repeat\n";
+    t << "set content=X\n";
+    t << "for /F \"tokens=*\" %%T in ( 'findstr /C:\"Rerun LaTeX\" refman.log' ) do set content=\"%%~T\"\n";
+    t << "if !content! == X for /F \"tokens=*\" %%T in ( 'findstr /C:\"Rerun to get cross-references right\" refman.log' ) do set content=\"%%~T\"\n";
+    t << "if !content! == X goto :skip\n";
+    t << "set /a count-=1\n";
+    t << "if !count! EQU 0 goto :skip\n\n";
+    t << "echo ----\n";
+    t << latex_command << " refman.tex\n";
+    t << "goto :repeat\n";
+    t << ":skip\n";
+    t << "endlocal\n";
+    t << "dvips -o refman.ps refman.dvi\n";
+    t << "gswin32c -q -dNOPAUSE -dBATCH -sDEVICE=pdfwrite "
+         "-sOutputFile=refman.pdf -c save pop -f refman.ps\n";
+  }
+  else // use pdflatex
+  {
+    t << "pdflatex refman\n";
+    t << "echo ----\n";
+    t << mkidx_command << " refman.idx\n";
+    if (generateBib)
+    {
+      t << "bibtex refman" << endl;
+      t << "pdflatex refman" << endl;
+    }
+    t << "echo ----\n";
+    t << "pdflatex refman\n\n";
+    t << "setlocal enabledelayedexpansion\n";
+    t << "set count=5\n";
+    t << ":repeat\n";
+    t << "set content=X\n";
+    t << "for /F \"tokens=*\" %%T in ( 'findstr /C:\"Rerun LaTeX\" refman.log' ) do set content=\"%%~T\"\n";
+    t << "if !content! == X for /F \"tokens=*\" %%T in ( 'findstr /C:\"Rerun to get cross-references right\" refman.log' ) do set content=\"%%~T\"\n";
+    t << "if !content! == X goto :skip\n";
+    t << "set /a count-=1\n";
+    t << "if !count! EQU 0 goto :skip\n\n";
+    t << "echo ----\n";
+    t << "pdflatex refman\n";
+    t << "goto :repeat\n";
+    t << ":skip\n";
+    t << "endlocal\n";
+  }
+#endif
+}
+
+void LatexGenerator::init()
+{
+
+  QCString dir=Config_getString("LATEX_OUTPUT");
+  QDir d(dir);
+  if (!d.exists() && !d.mkdir(dir))
+  {
+    err("Could not create output directory %s\n",dir.data());
+    exit(1);
+  }
+
+  writeLatexMakefile();
+  writeMakeBat();
 
   createSubDirs(d);
 }
@@ -197,14 +263,16 @@ static void writeDefaultHeaderPart1(FTextStream &t)
     paperName="a4"; 
   else 
     paperName=paperType;
-  t << "\\documentclass[" << paperName << "paper";
-  //if (Config_getBool("PDF_HYPERLINKS")) t << ",ps2pdf";
-  t << "]{";
+  t << "\\documentclass";
+  //"[" << paperName << "paper";
+  //t << "]";
+  t << "{";
   if (Config_getBool("COMPACT_LATEX")) t << "article"; else t << "book";
   t << "}\n";
   // the next package is obsolete (see bug 563698)
   //if (paperType=="a4wide") t << "\\usepackage{a4wide}\n";
   t << 
+    "\\usepackage["<<paperName<<"paper,top=2.5cm,bottom=2.5cm,left=2.5cm,right=2.5cm]{geometry}\n"
     "\\usepackage{makeidx}\n"
     "\\usepackage{natbib}\n"
     "\\usepackage{graphicx}\n"
@@ -258,6 +326,7 @@ static void writeDefaultHeaderPart1(FTextStream &t)
   t << "\\usepackage[scaled=.90]{helvet}\n";
   t << "\\usepackage{courier}\n";
   t << "\\usepackage{sectsty}\n";
+  t << "\\usepackage{amssymb}\n";
   t << "\\usepackage[titles]{tocloft}\n";
   t << "\\usepackage{doxygen}\n";
 
@@ -345,6 +414,8 @@ static void writeDefaultStyleSheetPart1(FTextStream &t)
        "\\RequirePackage{longtable}\n"
        "\\RequirePackage{verbatim}\n"
        "\\RequirePackage{ifthen}\n"
+       "\\RequirePackage{xtab}\n"
+       "\\RequirePackage{multirow}\n"
        "\\RequirePackage[table]{xcolor}\n\n";
 
   t << "% Use helvetica font instead of times roman\n"
@@ -431,7 +502,7 @@ static void writeDefaultStyleSheetPart2(FTextStream &t)
 
 static void writeDefaultStyleSheetPart3(FTextStream &t)
 {
-  static bool latexSourceCode = Config_getBool("LATEX_SOURCE_CODE");
+  //static bool latexSourceCode = Config_getBool("LATEX_SOURCE_CODE");
   t << "}}\n";
   //t << "\\cfoot{}\n\n";
   t << "\\fancyfoot[CO]{\\fancyplain{}{}}\n";
@@ -487,25 +558,25 @@ static void writeDefaultStyleSheetPart3(FTextStream &t)
        "}\n\n";
   t << "% Used by @code ... @endcode\n"
        "\\newenvironment{DoxyCode}{%\n";
-  if (latexSourceCode)
-  {
+  //if (latexSourceCode)
+  //{
     t << "\n\n\\begin{scriptsize}\\begin{alltt}%" << endl;
-  }
-  else
-  {
-    t << "  \\footnotesize%\n"
-         "  \\verbatim%\n";
-  }
+  //}
+  //else
+  //{
+  //  t << "  \\footnotesize%\n"
+  //       "  \\verbatim%\n";
+  //}
   t << "}{%\n";
-  if (latexSourceCode)
-  {
+  //if (latexSourceCode)
+  //{
     t << "\\end{alltt}\\end{scriptsize}%" << endl; 
-  }
-  else
-  {
-    t << "  \\endverbatim%\n"
-         "  \\normalsize%\n";
-  }
+  //}
+  //else
+  //{
+  //  t << "  \\endverbatim%\n"
+  //       "  \\normalsize%\n";
+  //}
   t << "}\n\n";
   t << "% Used by @example, @include, @includelineno and @dontinclude\n"
        "\\newenvironment{DoxyCodeInclude}{%\n"
@@ -701,6 +772,7 @@ static void writeDefaultStyleSheetPart3(FTextStream &t)
        "    \\begin{longtable}{|>{\\raggedleft\\hspace{0pt}}p{0.15\\textwidth}|%\n"
        "                         p{0.15\\textwidth}|%\n"
        "                         p{0.635\\textwidth}|}%\n"
+       //"\\hline{\\sf\\textbf{Type}} & {\\sf\\textbf{Name}} & {\\sf\\textbf{Description}}\\endhead%\n"
        "    \\hline%\n"
        "}{%\n"
        "    \\end{longtable}%\n"
@@ -787,10 +859,10 @@ static void writeDefaultStyleSheetPart3(FTextStream &t)
   t << "{\n";
   t << "\\setlength{\\tmplength}\n";
   t << "     {\\linewidth/(#1)-\\tabcolsep*2-\\arrayrulewidth*(#1+1)/(#1)}\n";
-  t << "      \\par\\begin{tabular*}{\\linewidth}\n";
+  t << "      \\par\\begin{xtabular*}{\\linewidth}\n";
   t << "             {*{#1}{|>{\\PBS\\raggedright\\hspace{0pt}}p{\\the\\tmplength}}|}\n";
   t << "}\n";
-  t << "{\\end{tabular*}\\par}\n";
+  t << "{\\end{xtabular*}\\par}\n";
   t << "\\newcommand{\\entrylabel}[1]{\n";
   t << "   {\\parbox[b]{\\labelwidth-4pt}{\\makebox[0pt][l]{%\n";
   t << "   \\usefont{OT1}{phv}{bc}{n}\\color{darkgray}#1}\\vspace{1.5\\baselineskip}}}}\n";
@@ -874,6 +946,7 @@ static void writeDefaultStyleSheetPart3(FTextStream &t)
 static void writeDefaultFooter(FTextStream &t)
 {
   Doxygen::citeDict->writeLatexBibliography(t);
+  t << "\\addcontentsline{toc}{part}{" << theTranslator->trRTFGeneralIndex() << "}\n";
   t << "\\printindex\n";
   t << "\\end{document}\n";
 }
@@ -904,12 +977,12 @@ void LatexGenerator::writeStyleSheetFile(QFile &f)
   t << theTranslator->trGeneratedAt( dateToString(TRUE), projectName );
   t << " doxygen";
   //t << " " << theTranslator->trWrittenBy() << " ";
-  //t << "Dimitri van Heesch \\copyright~1997-2011";
+  //t << "Dimitri van Heesch \\copyright~1997-2012";
   writeDefaultStyleSheetPart2(t);
   t << theTranslator->trGeneratedAt( dateToString(TRUE), projectName );
   t << " doxygen";
   //t << " << theTranslator->trWrittenBy() << " ";
-  //t << "Dimitri van Heesch \\copyright~1997-2011";
+  //t << "Dimitri van Heesch \\copyright~1997-2012";
   writeDefaultStyleSheetPart3(t);
 }
 
@@ -956,7 +1029,10 @@ void LatexGenerator::startIndexSection(IndexSections is)
         else
         {
           QCString header = fileToString(latexHeader);
-          t << substituteKeywords(header,0);
+          t << substituteKeywords(header,0,
+              Config_getString("PROJECT_NAME"),
+              Config_getString("PROJECT_NUMBER"),
+              Config_getString("PROJECT_BRIEF"));
         }
       }
       break;
@@ -1351,7 +1427,10 @@ void LatexGenerator::endIndexSection(IndexSections is)
       else
       {
         QCString footer = fileToString(latexFooter);
-        t << substituteKeywords(footer,0);
+        t << substituteKeywords(footer,0,
+              Config_getString("PROJECT_NAME"),
+              Config_getString("PROJECT_NUMBER"),
+              Config_getString("PROJECT_BRIEF"));
       }
       break;
   }
@@ -1579,13 +1658,15 @@ void LatexGenerator::writeCodeLink(const char *ref,const char *f,
     t << "\n      ";
     col=0;
   }
-  if (m_prettyCode && !disableLinks && !ref && usePDFLatex && pdfHyperlinks)
+  if (/*m_prettyCode &&*/ !disableLinks && !ref && usePDFLatex && pdfHyperlinks)
   {
     t << "\\hyperlink{";
     if (f) t << stripPath(f);
     if (f && anchor) t << "_"; 
     if (anchor) t << anchor; 
-    t << "}{" << name << "}";
+    t << "}{";
+    codify(name);
+    t << "}";
   }
   else
   {
@@ -1600,7 +1681,7 @@ void LatexGenerator::startTitleHead(const char *fileName)
   static bool usePDFLatex   = Config_getBool("USE_PDFLATEX");
   if (usePDFLatex && pdfHyperlinks && fileName)
   {
-    t << "\\hypertarget{" << stripPath(fileName) << "}{" << endl;
+    t << "\\hypertarget{" << stripPath(fileName) << "}{";
   }
   if (Config_getBool("COMPACT_LATEX")) 
   {
@@ -1766,7 +1847,7 @@ void LatexGenerator::startDoxyAnchor(const char *fName,const char *,
     t << "\\hypertarget{";
     if (fName) t << stripPath(fName);
     if (anchor) t << "_" << anchor;
-    t << "}{" << endl;
+    t << "}{";
   }
 }
 
@@ -1776,7 +1857,7 @@ void LatexGenerator::endDoxyAnchor(const char *fName,const char *anchor)
   static bool usePDFLatex   = Config_getBool("USE_PDFLATEX");
   if (usePDFLatex && pdfHyperlinks)
   {
-    t << "}" << endl;
+    t << "}";
   }
   t << "\\label{";
   if (fName) t << fName;
@@ -1953,14 +2034,14 @@ void LatexGenerator::codify(const char *str)
                      }
                    }
                    result[i]=0; // add terminator
-                   if (m_prettyCode)
-                   {
+                   //if (m_prettyCode)
+                   //{
                      filterLatexString(t,result,insideTabbing,TRUE);
-                   }
-                   else
-                   {
-                     t << result;
-                   }
+                   //}
+                   //else
+                   //{
+                   //  t << result;
+                   //}
                    break;
       }
     }
@@ -2017,7 +2098,7 @@ void LatexGenerator::startMemberTemplateParams()
   }
 }
 
-void LatexGenerator::endMemberTemplateParams()
+void LatexGenerator::endMemberTemplateParams(const char *,const char *)
 {
   if (templateMemberItem)
   {
@@ -2025,7 +2106,7 @@ void LatexGenerator::endMemberTemplateParams()
   }
 }
 
-void LatexGenerator::startMemberItem(int annoType) 
+void LatexGenerator::startMemberItem(const char *,int annoType,const char *) 
 { 
   //printf("LatexGenerator::startMemberItem(%d)\n",annType);
   if (!insideTabbing)
@@ -2045,7 +2126,7 @@ void LatexGenerator::endMemberItem()
   t << endl; 
 }
 
-void LatexGenerator::startMemberDescription() 
+void LatexGenerator::startMemberDescription(const char *,const char *) 
 {
   if (!insideTabbing)
   { 
@@ -2106,19 +2187,23 @@ void LatexGenerator::endMemberList()
 void LatexGenerator::startMemberGroupHeader(bool hasHeader)
 {
   if (hasHeader) t << "\\begin{Indent}";
-  if (Config_getBool("COMPACT_LATEX")) 
-  {
-    t << "\\subparagraph*{";
-  }
-  else
-  {
-    t << "\\paragraph*{";
-  }
+  t << "{\\bf ";
+  // changed back to rev 756 due to bug 660501
+  //if (Config_getBool("COMPACT_LATEX")) 
+  //{
+  //  t << "\\subparagraph*{";
+  //}
+  //else
+  //{
+  //  t << "\\paragraph*{";
+  //}
 }
 
 void LatexGenerator::endMemberGroupHeader()
 {
-  t << "}" << endl;
+  // changed back to rev 756 due to bug 660501
+  t << "}\\par" << endl;
+  //t << "}" << endl;
 }
 
 void LatexGenerator::startMemberGroupDocs()
@@ -2260,15 +2345,10 @@ void LatexGenerator::endParameterList()
 {
 }
 
-
-void LatexGenerator::startParameterType(bool /*first*/,const char *key)
+void LatexGenerator::startParameterType(bool first,const char *key)
 {
   t << "\\item[{";
-  t << key;
-//  if (!first)
-//  {
-//    t << "\\/ " << key << " ";
-//  }
+  if (!first && key) t << key;
 }
 
 void LatexGenerator::endParameterType()
@@ -2293,9 +2373,10 @@ void LatexGenerator::endParameterName(bool last,bool /* emptyList */,bool closeB
 }
 
 
-void LatexGenerator::printDoc(DocNode *n,const char *langExt)
+void LatexGenerator::writeDoc(DocNode *n,Definition *ctx,MemberDef *)
 {
-  LatexDocVisitor *visitor = new LatexDocVisitor(t,*this,langExt,insideTabbing);
+  LatexDocVisitor *visitor = 
+    new LatexDocVisitor(t,*this,ctx?ctx->getDefFileExtension():QCString(""),insideTabbing);
   n->accept(visitor);
   delete visitor; 
 }
@@ -2440,7 +2521,7 @@ void LatexGenerator::writeLineNumber(const char *ref,const char *fileName,const
   }
 }
 
-void LatexGenerator::startCodeLine()
+void LatexGenerator::startCodeLine(bool)
 {
   col=0;
 }
@@ -2452,13 +2533,13 @@ void LatexGenerator::endCodeLine()
 
 void LatexGenerator::startFontClass(const char *name)
 {
-  if (!m_prettyCode) return;
+  //if (!m_prettyCode) return;
   t << "\\textcolor{" << name << "}{";
 }
 
 void LatexGenerator::endFontClass()
 {
-  if (!m_prettyCode) return;
+  //if (!m_prettyCode) return;
   t << "}";
 }
 
@@ -2466,7 +2547,7 @@ void LatexGenerator::startCodeAnchor(const char *name)
 {
   static bool usePDFLatex = Config_getBool("USE_PDFLATEX");
   static bool pdfHyperlinks = Config_getBool("PDF_HYPERLINKS");
-  if (!m_prettyCode) return;
+  //if (!m_prettyCode) return;
   if (usePDFLatex && pdfHyperlinks)
   {
     t << "\\hypertarget{" << stripPath(name) << "}{}";
@@ -2502,13 +2583,13 @@ void LatexGenerator::lineBreak(const char *)
   }
   else
   {
-    t << "\\*\n";
+    t << "\\\\*\n";
   }
 }
 
 void LatexGenerator::startMemberDocSimple()
 {
-  t << "\\begin{DoxyFields}{" << endl;
+  t << "\\begin{DoxyFields}{";
   docify(theTranslator->trCompoundMembers());
   t << "}" << endl;
 }
@@ -2545,3 +2626,18 @@ void LatexGenerator::endInlineMemberDoc()
   t << "\\\\\n\\hline\n" << endl;
 }
 
+void LatexGenerator::startLabels()
+{
+  t << "\\hspace{0.3cm}";
+}
+
+void LatexGenerator::writeLabel(const char *l,bool isLast)
+{
+  t << "{\\ttfamily [" << l << "]}";
+  if (!isLast) t << ", ";
+}
+
+void LatexGenerator::endLabels()
+{
+}
+
