@@ -3,7 +3,7 @@
  * $Id: layout.cpp,v 1.2 2001/03/19 19:27:41 root Exp $
  *
  *
- * Copyright (C) 1997-2011 by Dimitri van Heesch.
+ * Copyright (C) 1997-2012 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -21,7 +21,9 @@
 #include "language.h"
 #include "vhdldocgen.h"
 #include "util.h"
+#include "doxygen.h"
 
+#include <assert.h>
 #include <qxml.h>
 #include <qfile.h>
 #include <qstring.h>
@@ -32,6 +34,20 @@ static const char layout_default[] =
 #include "layout_default.h"
 ;
 
+#define ADD_OPTION(langId,text) "|"+QCString().setNum(langId)+"="+text
+
+#define COMPILE_FOR_1_OPTION(def,langId1,text1) \
+  def+ADD_OPTION(langId1,text1)
+
+#define COMPILE_FOR_2_OPTIONS(def,langId1,text1,langId2,text2) \
+  COMPILE_FOR_1_OPTION(def,langId1,text1)+ADD_OPTION(langId2,text2)
+
+#define COMPILE_FOR_3_OPTIONS(def,langId1,text1,langId2,text2,langId3,text3) \
+  COMPILE_FOR_2_OPTIONS(def,langId1,text1,langId2,text2)+ADD_OPTION(langId3,text3)
+
+#define COMPILE_FOR_4_OPTIONS(def,langId1,text1,langId2,text2,langId3,text3,langId4,text4) \
+  COMPILE_FOR_3_OPTIONS(def,langId1,text1,langId2,text2,langId3,text3)+ADD_OPTION(langId4,text4)
+
 static bool elemIsVisible(const QXmlAttributes &attrib,bool defVal=TRUE)
 {
   QCString visible = convertToQCString(attrib.value("visible"));
@@ -55,7 +71,8 @@ static bool elemIsVisible(const QXmlAttributes &attrib,bool defVal=TRUE)
 
 //---------------------------------------------------------------------------------
 
-LayoutNavEntry *LayoutNavEntry::find(LayoutNavEntry::Kind kind) const
+LayoutNavEntry *LayoutNavEntry::find(LayoutNavEntry::Kind kind,
+    const char *file) const
 {
   LayoutNavEntry *result=0;
   QListIterator<LayoutNavEntry> li(m_children);
@@ -64,9 +81,9 @@ LayoutNavEntry *LayoutNavEntry::find(LayoutNavEntry::Kind kind) const
   {
     // depth first search, needed to find the entry furthest from the 
     // root in case an entry is in the tree twice
-    result = entry->find(kind);
+    result = entry->find(kind,file);
     if (result) return result;
-    if (entry->kind()==kind)
+    if (entry->kind()==kind && (file==0 || entry->baseFile()==file))
     {
       return entry;
     }
@@ -74,6 +91,39 @@ LayoutNavEntry *LayoutNavEntry::find(LayoutNavEntry::Kind kind) const
   return result;
 }
 
+QCString LayoutNavEntry::url() const
+{
+  QCString url = baseFile().stripWhiteSpace();
+  if (kind()!=LayoutNavEntry::User)
+  {
+    url+=Doxygen::htmlFileExtension;
+  }
+  else if (url.left(5)=="@ref " || url.left(5)=="\\ref ")
+  {
+    Definition *d;
+    QCString anchor;
+    bool found=FALSE;
+    if (resolveLink(0,url.mid(5).stripWhiteSpace(),TRUE,&d,anchor))
+    {
+      if (d && d->isLinkable()) 
+      {
+        url=d->getOutputFileBase()+Doxygen::htmlFileExtension;
+        if (!anchor.isEmpty())
+        {
+          url+="#"+anchor;
+        }
+        found=TRUE;
+      }
+    }
+    if (!found)
+    {
+      msg("warning: explicit link request to '%s' in layout file '%s' could not be resolved\n",qPrint(url.mid(5)),qPrint(Config_getString("LAYOUT_FILE")));
+    }
+  }
+  //printf("LayoutNavEntry::url()=%s\n",url.data());
+  return url;
+}
+
 //---------------------------------------------------------------------------------
 
 class LayoutParser : public QXmlDefaultHandler
@@ -209,9 +259,9 @@ class LayoutParser : public QXmlDefaultHandler
       m_part = -1; // invalid
       m_rootNav = 0;
 
-      bool fortranOpt = Config_getBool("OPTIMIZE_FOR_FORTRAN");
-      bool vhdlOpt    = Config_getBool("OPTIMIZE_OUTPUT_VHDL");  
-      bool javaOpt    = Config_getBool("OPTIMIZE_OUTPUT_JAVA");
+      //bool fortranOpt = Config_getBool("OPTIMIZE_FOR_FORTRAN");
+      //bool vhdlOpt    = Config_getBool("OPTIMIZE_OUTPUT_VHDL");  
+      //bool javaOpt    = Config_getBool("OPTIMIZE_OUTPUT_JAVA");
 
       // start & end handlers
       m_sHandler.insert("doxygenlayout", 
@@ -255,10 +305,11 @@ class LayoutParser : public QXmlDefaultHandler
           new StartElementHandlerKind(this,LayoutDocEntry::MemberGroups,&LayoutParser::startSimpleEntry));
       m_sHandler.insert("class/memberdecl/nestedclasses", 
           new StartElementHandlerSection(this,LayoutDocEntry::ClassNestedClasses,&LayoutParser::startSectionEntry,
-                                         vhdlOpt ? VhdlDocGen::trVhdlType(VhdlDocGen::ARCHITECTURE,FALSE) :
-                                         fortranOpt ? theTranslator->trDataTypes() :
-                                         theTranslator->trCompounds() 
-                                         ));
+                                         COMPILE_FOR_2_OPTIONS(
+                                           theTranslator->trCompounds(),
+                                           SrcLangExt_VHDL,VhdlDocGen::trVhdlType(VhdlDocGen::ENTITY,FALSE),
+                                           SrcLangExt_Fortran,theTranslator->trDataTypes()
+                                         )));
       m_sHandler.insert("class/memberdecl/publictypes", 
           new StartElementHandlerMember(this,&LayoutParser::startMemberDeclEntry,
                                         MemberList::pubTypes,theTranslator->trPublicTypes()));
@@ -270,10 +321,18 @@ class LayoutParser : public QXmlDefaultHandler
                                         MemberList::signals,theTranslator->trSignals())); 
       m_sHandler.insert("class/memberdecl/publicmethods", 
           new StartElementHandlerMember(this,&LayoutParser::startMemberDeclEntry,
-                                        MemberList::pubMethods,theTranslator->trPublicMembers())); 
+                                        MemberList::pubMethods,
+                                        COMPILE_FOR_1_OPTION(
+                                          theTranslator->trPublicMembers(),
+                                          SrcLangExt_ObjC,theTranslator->trInstanceMethods()
+                                        ))); 
       m_sHandler.insert("class/memberdecl/publicstaticmethods", 
           new StartElementHandlerMember(this,&LayoutParser::startMemberDeclEntry,
-                                        MemberList::pubStaticMethods,theTranslator->trStaticPublicMembers())); 
+                                        MemberList::pubStaticMethods,
+                                        COMPILE_FOR_1_OPTION(
+                                          theTranslator->trStaticPublicMembers(),
+                                          SrcLangExt_ObjC,theTranslator->trClassMethods()
+                                        ))); 
       m_sHandler.insert("class/memberdecl/publicattributes", 
           new StartElementHandlerMember(this,&LayoutParser::startMemberDeclEntry,
                                         MemberList::pubAttribs,theTranslator->trPublicAttribs())); 
@@ -350,9 +409,10 @@ class LayoutParser : public QXmlDefaultHandler
           new StartElementHandler(this,&LayoutParser::startMemberDef));
       m_sHandler.insert("class/memberdef/inlineclasses", 
           new StartElementHandlerSection(this,LayoutDocEntry::ClassInlineClasses,&LayoutParser::startSectionEntry,
-                                         fortranOpt ? theTranslator->trTypeDocumentation() :
-                                         theTranslator->trClassDocumentation() 
-                                         ));
+                                         COMPILE_FOR_1_OPTION(
+                                           theTranslator->trClassDocumentation(),
+                                           SrcLangExt_Fortran,theTranslator->trTypeDocumentation()
+                                         )));
       m_sHandler.insert("class/memberdef/typedefs", 
           new StartElementHandlerMember(this,&LayoutParser::startMemberDefEntry,
                                         MemberList::typedefMembers,theTranslator->trMemberTypedefDocumentation()));
@@ -365,9 +425,11 @@ class LayoutParser : public QXmlDefaultHandler
       m_sHandler.insert("class/memberdef/functions", 
           new StartElementHandlerMember(this,&LayoutParser::startMemberDefEntry,
                                         MemberList::functionMembers,
-                                        fortranOpt ?
-                                        theTranslator->trMemberFunctionDocumentationFortran() :
-                                        theTranslator->trMemberFunctionDocumentation()));
+                                        COMPILE_FOR_2_OPTIONS(
+                                          theTranslator->trMemberFunctionDocumentation(),
+                                          SrcLangExt_ObjC,theTranslator->trMethodDocumentation(),
+                                          SrcLangExt_Fortran,theTranslator->trMemberFunctionDocumentationFortran()
+                                        )));
       m_sHandler.insert("class/memberdef/related", 
           new StartElementHandlerMember(this,&LayoutParser::startMemberDefEntry,
                                         MemberList::relatedMembers,theTranslator->trRelatedFunctionDocumentation()));
@@ -400,16 +462,19 @@ class LayoutParser : public QXmlDefaultHandler
           new StartElementHandler(this,&LayoutParser::startMemberDecl));
       m_sHandler.insert("namespace/memberdecl/nestednamespaces", 
           new StartElementHandlerSection(this,LayoutDocEntry::NamespaceNestedNamespaces,&LayoutParser::startSectionEntry,
-                                         javaOpt ? theTranslator->trPackages() :
-                                         fortranOpt ? theTranslator->trModules() :
-                                         theTranslator->trNamespaces()
-                                         ));
+                                         COMPILE_FOR_3_OPTIONS(
+                                           theTranslator->trNamespaces(),
+                                           SrcLangExt_Java,theTranslator->trPackages(),
+                                           SrcLangExt_VHDL,theTranslator->trPackages(),
+                                           SrcLangExt_Fortran,theTranslator->trModules()
+                                         )));
       m_sHandler.insert("namespace/memberdecl/classes", 
           new StartElementHandlerSection(this,LayoutDocEntry::NamespaceClasses,&LayoutParser::startSectionEntry,
-                                         vhdlOpt ? VhdlDocGen::trVhdlType(VhdlDocGen::ARCHITECTURE,FALSE) :
-                                         fortranOpt ? theTranslator->trDataTypes() :
-                                         theTranslator->trCompounds() 
-                                         ));
+                                         COMPILE_FOR_2_OPTIONS(
+                                           theTranslator->trCompounds(),
+                                           SrcLangExt_VHDL,VhdlDocGen::trVhdlType(VhdlDocGen::ENTITY,FALSE),
+                                           SrcLangExt_Fortran,theTranslator->trDataTypes()
+                                         )));
       m_sHandler.insert("namespace/memberdecl/membergroups", 
           new StartElementHandlerKind(this,LayoutDocEntry::MemberGroups,&LayoutParser::startSimpleEntry));
       m_sHandler.insert("namespace/memberdecl/typedefs", 
@@ -421,9 +486,11 @@ class LayoutParser : public QXmlDefaultHandler
       m_sHandler.insert("namespace/memberdecl/functions", 
           new StartElementHandlerMember(this,&LayoutParser::startMemberDeclEntry,
                                         MemberList::decFuncMembers,
-                                        fortranOpt ? theTranslator->trSubprograms()  : 
-                                        vhdlOpt    ? VhdlDocGen::trFunctionAndProc() :
-                                        theTranslator->trFunctions()));
+                                        COMPILE_FOR_2_OPTIONS(
+                                          theTranslator->trFunctions(),
+                                          SrcLangExt_Fortran,theTranslator->trSubprograms(),
+                                          SrcLangExt_VHDL,VhdlDocGen::trFunctionAndProc()
+                                        )));
       m_sHandler.insert("namespace/memberdecl/variables", 
           new StartElementHandlerMember(this,&LayoutParser::startMemberDeclEntry,
                                         MemberList::decVarMembers,theTranslator->trVariables()));
@@ -433,9 +500,10 @@ class LayoutParser : public QXmlDefaultHandler
           new StartElementHandler(this,&LayoutParser::startMemberDef));
       m_sHandler.insert("namespace/memberdef/inlineclasses", 
           new StartElementHandlerSection(this,LayoutDocEntry::NamespaceInlineClasses,&LayoutParser::startSectionEntry,
-                                         fortranOpt ? theTranslator->trTypeDocumentation() :
-                                         theTranslator->trClassDocumentation() 
-                                         ));
+                                         COMPILE_FOR_1_OPTION(
+                                           theTranslator->trClassDocumentation(),
+                                           SrcLangExt_Fortran,theTranslator->trTypeDocumentation()
+                                         )));
       m_sHandler.insert("namespace/memberdef/typedefs", 
           new StartElementHandlerMember(this,&LayoutParser::startMemberDefEntry,
                                         MemberList::docTypedefMembers,theTranslator->trTypedefDocumentation()));
@@ -445,9 +513,10 @@ class LayoutParser : public QXmlDefaultHandler
       m_sHandler.insert("namespace/memberdef/functions", 
           new StartElementHandlerMember(this,&LayoutParser::startMemberDefEntry,
                                         MemberList::docFuncMembers,
-                                        fortranOpt ? 
-                                        theTranslator->trSubprogramDocumentation() :
-                                        theTranslator->trFunctionDocumentation()));
+                                        COMPILE_FOR_1_OPTION(
+                                          theTranslator->trFunctionDocumentation(),
+                                          SrcLangExt_Fortran,theTranslator->trSubprogramDocumentation()
+                                        )));
       m_sHandler.insert("namespace/memberdef/variables", 
           new StartElementHandlerMember(this,&LayoutParser::startMemberDefEntry,
                                         MemberList::docVarMembers,theTranslator->trVariableDocumentation()));
@@ -480,16 +549,18 @@ class LayoutParser : public QXmlDefaultHandler
           new StartElementHandler(this,&LayoutParser::startMemberDecl));
       m_sHandler.insert("file/memberdecl/classes", 
           new StartElementHandlerSection(this,LayoutDocEntry::FileClasses,&LayoutParser::startSectionEntry,
-                                         vhdlOpt ? VhdlDocGen::trVhdlType(VhdlDocGen::ARCHITECTURE,FALSE) :
-                                         fortranOpt ? theTranslator->trDataTypes() :
-                                         theTranslator->trCompounds() 
-                                         ));
+                                         COMPILE_FOR_2_OPTIONS(
+                                           theTranslator->trCompounds(),
+                                           SrcLangExt_VHDL,VhdlDocGen::trVhdlType(VhdlDocGen::ENTITY,FALSE),
+                                           SrcLangExt_Fortran,theTranslator->trDataTypes()
+                                         )));
       m_sHandler.insert("file/memberdecl/namespaces", 
           new StartElementHandlerSection(this,LayoutDocEntry::FileNamespaces,&LayoutParser::startSectionEntry,
-                                         javaOpt ? theTranslator->trPackages() :
-                                         fortranOpt ? theTranslator->trModules() :
-                                         theTranslator->trNamespaces()
-                                         ));
+                                         COMPILE_FOR_2_OPTIONS(
+                                           theTranslator->trNamespaces(),
+                                           SrcLangExt_Java,theTranslator->trPackages(),
+                                           SrcLangExt_Fortran,theTranslator->trModules()
+                                         )));
       m_sHandler.insert("file/memberdecl/defines", 
           new StartElementHandlerMember(this,&LayoutParser::startMemberDeclEntry,
                                         MemberList::decDefineMembers,theTranslator->trDefines()));
@@ -502,9 +573,11 @@ class LayoutParser : public QXmlDefaultHandler
       m_sHandler.insert("file/memberdecl/functions", 
           new StartElementHandlerMember(this,&LayoutParser::startMemberDeclEntry,
                                         MemberList::decFuncMembers,
-                                        fortranOpt ? theTranslator->trSubprograms()  : 
-                                        vhdlOpt    ? VhdlDocGen::trFunctionAndProc() :
-                                        theTranslator->trFunctions())) ;
+                                        COMPILE_FOR_2_OPTIONS(
+                                          theTranslator->trFunctions(),
+                                          SrcLangExt_Fortran,theTranslator->trSubprograms(),
+                                          SrcLangExt_VHDL,VhdlDocGen::trFunctionAndProc()
+                                        )));
       m_sHandler.insert("file/memberdecl/variables", 
           new StartElementHandlerMember(this,&LayoutParser::startMemberDeclEntry,
                                         MemberList::decVarMembers,theTranslator->trVariables()));
@@ -515,9 +588,10 @@ class LayoutParser : public QXmlDefaultHandler
           new StartElementHandler(this,&LayoutParser::startMemberDef));
       m_sHandler.insert("file/memberdef/inlineclasses", 
           new StartElementHandlerSection(this,LayoutDocEntry::FileInlineClasses,&LayoutParser::startSectionEntry,
-                                         fortranOpt ? theTranslator->trTypeDocumentation() :
-                                         theTranslator->trClassDocumentation() 
-                                         ));
+                                         COMPILE_FOR_1_OPTION(
+                                           theTranslator->trClassDocumentation(),
+                                           SrcLangExt_Fortran,theTranslator->trTypeDocumentation()
+                                         )));
       m_sHandler.insert("file/memberdef/defines", 
           new StartElementHandlerMember(this,&LayoutParser::startMemberDefEntry,
                                         MemberList::docDefineMembers,theTranslator->trDefineDocumentation()));
@@ -531,7 +605,10 @@ class LayoutParser : public QXmlDefaultHandler
       m_sHandler.insert("file/memberdef/functions", 
           new StartElementHandlerMember(this,&LayoutParser::startMemberDefEntry,
                                         MemberList::docFuncMembers,
-                                        fortranOpt ? theTranslator->trSubprogramDocumentation() : theTranslator->trFunctionDocumentation()));
+                                        COMPILE_FOR_1_OPTION(
+                                          theTranslator->trFunctionDocumentation(),
+                                          SrcLangExt_Fortran,theTranslator->trSubprogramDocumentation()
+                                        )));
       m_sHandler.insert("file/memberdef/variables", 
           new StartElementHandlerMember(this,&LayoutParser::startMemberDefEntry,
                                         MemberList::docVarMembers,theTranslator->trVariableDocumentation()));
@@ -558,16 +635,18 @@ class LayoutParser : public QXmlDefaultHandler
           new StartElementHandler(this,&LayoutParser::startMemberDecl));
       m_sHandler.insert("group/memberdecl/classes", 
           new StartElementHandlerSection(this,LayoutDocEntry::GroupClasses,&LayoutParser::startSectionEntry,
-                                         vhdlOpt ? VhdlDocGen::trVhdlType(VhdlDocGen::ARCHITECTURE,FALSE) :
-                                         fortranOpt ? theTranslator->trDataTypes() :
-                                         theTranslator->trCompounds() 
-                                         ));
+                                         COMPILE_FOR_2_OPTIONS(
+                                           theTranslator->trCompounds(),
+                                           SrcLangExt_VHDL,VhdlDocGen::trVhdlType(VhdlDocGen::ENTITY,FALSE),
+                                           SrcLangExt_Fortran,theTranslator->trDataTypes()
+                                         )));
       m_sHandler.insert("group/memberdecl/namespaces", 
           new StartElementHandlerSection(this,LayoutDocEntry::GroupNamespaces,&LayoutParser::startSectionEntry,
-                                         javaOpt ? theTranslator->trPackages() :
-                                         fortranOpt ? theTranslator->trModules() :
-                                         theTranslator->trNamespaces()
-                                         ));
+                                         COMPILE_FOR_2_OPTIONS(
+                                           theTranslator->trNamespaces(),
+                                           SrcLangExt_Java,theTranslator->trPackages(),
+                                           SrcLangExt_Fortran,theTranslator->trModules()
+                                         )));
       m_sHandler.insert("group/memberdecl/dirs", 
           new StartElementHandlerSection(this,LayoutDocEntry::GroupDirs,&LayoutParser::startSectionEntry,
                                          theTranslator->trDirectories()
@@ -596,9 +675,11 @@ class LayoutParser : public QXmlDefaultHandler
       m_sHandler.insert("group/memberdecl/functions", 
           new StartElementHandlerMember(this,&LayoutParser::startMemberDeclEntry,
                                         MemberList::decFuncMembers,
-                                            fortranOpt ? theTranslator->trSubprograms()  : 
-                                            vhdlOpt    ? VhdlDocGen::trFunctionAndProc() :
-                                            theTranslator->trFunctions()));
+                                        COMPILE_FOR_2_OPTIONS(
+                                          theTranslator->trFunctions(),
+                                          SrcLangExt_Fortran,theTranslator->trSubprograms(),
+                                          SrcLangExt_VHDL,VhdlDocGen::trFunctionAndProc()
+                                        )));
       m_sHandler.insert("group/memberdecl/variables", 
           new StartElementHandlerMember(this,&LayoutParser::startMemberDeclEntry,
                                         MemberList::decVarMembers,theTranslator->trVariables()));
@@ -631,9 +712,10 @@ class LayoutParser : public QXmlDefaultHandler
           new StartElementHandlerKind(this,LayoutDocEntry::GroupPageDocs,&LayoutParser::startSimpleEntry));
       m_sHandler.insert("group/memberdef/inlineclasses", 
           new StartElementHandlerSection(this,LayoutDocEntry::GroupInlineClasses,&LayoutParser::startSectionEntry,
-                                         fortranOpt ? theTranslator->trTypeDocumentation() :
-                                         theTranslator->trClassDocumentation() 
-                                         ));
+                                         COMPILE_FOR_1_OPTION(
+                                           theTranslator->trClassDocumentation(),
+                                           SrcLangExt_Fortran,theTranslator->trTypeDocumentation()
+                                         )));
       m_sHandler.insert("group/memberdef/defines", 
           new StartElementHandlerMember(this,&LayoutParser::startMemberDefEntry,
                                         MemberList::docDefineMembers,theTranslator->trDefineDocumentation()));
@@ -649,9 +731,10 @@ class LayoutParser : public QXmlDefaultHandler
       m_sHandler.insert("group/memberdef/functions", 
           new StartElementHandlerMember(this,&LayoutParser::startMemberDefEntry,
                                         MemberList::docFuncMembers,
-                                        fortranOpt?
-                                        theTranslator->trSubprogramDocumentation():
-                                        theTranslator->trFunctionDocumentation()));
+                                        COMPILE_FOR_1_OPTION(
+                                          theTranslator->trFunctionDocumentation(),
+                                          SrcLangExt_Fortran,theTranslator->trSubprogramDocumentation()
+                                       )));
       m_sHandler.insert("group/memberdef/variables", 
           new StartElementHandlerMember(this,&LayoutParser::startMemberDefEntry,
                                         MemberList::docVarMembers,theTranslator->trVariableDocumentation()));
@@ -771,15 +854,15 @@ class LayoutParser : public QXmlDefaultHandler
     {
       m_scope="navindex/";
       m_rootNav = LayoutDocManager::instance().rootNavEntry();
-      m_rootNav->clear();
+      if (m_rootNav) m_rootNav->clear();
     }
 
     void endNavIndex()
     {
       m_scope="";
-      if (!m_rootNav->find(LayoutNavEntry::MainPage))
+      if (m_rootNav && !m_rootNav->find(LayoutNavEntry::MainPage))
       {
-        // no MainPage node... add one the first item of the root node...
+        // no MainPage node... add one as the first item of the root node...
         new LayoutNavEntry(m_rootNav,LayoutNavEntry::MainPage, TRUE, 
             /*Config_getBool("GENERATE_TREEVIEW") ? "main" :*/ "index",
             theTranslator->trMainPage(),TRUE);
@@ -827,32 +910,46 @@ class LayoutParser : public QXmlDefaultHandler
         },
         { "namespaces",
           LayoutNavEntry::Namespaces,
-          javaOpt    ? theTranslator->trPackages() : fortranOpt ? theTranslator->trModules() : theTranslator->trNamespaces(),
-          javaOpt    ? theTranslator->trPackages() : fortranOpt ? theTranslator->trModulesList() : theTranslator->trNamespaceList(),
-          javaOpt    ? theTranslator->trPackageListDescription() : fortranOpt ? theTranslator->trModulesListDescription(extractAll) : theTranslator->trNamespaceListDescription(extractAll),
+          javaOpt || vhdlOpt   ? theTranslator->trPackages() : fortranOpt ? theTranslator->trModules() : theTranslator->trNamespaces(),
+          javaOpt || vhdlOpt   ? theTranslator->trPackages() : fortranOpt ? theTranslator->trModulesList() : theTranslator->trNamespaceList(),
+          javaOpt || vhdlOpt   ? theTranslator->trPackageListDescription() : fortranOpt ? theTranslator->trModulesListDescription(extractAll) : theTranslator->trNamespaceListDescription(extractAll),
+          "namespaces"
+        },
+        { "namespacelist",
+          LayoutNavEntry::NamespaceList,
+          javaOpt || vhdlOpt   ? theTranslator->trPackages() : fortranOpt ? theTranslator->trModulesList() : theTranslator->trNamespaceList(),
+          QCString(),
+          javaOpt || vhdlOpt   ? theTranslator->trPackageListDescription() : fortranOpt ? theTranslator->trModulesListDescription(extractAll) : theTranslator->trNamespaceListDescription(extractAll),
           "namespaces"
         },
         { "namespacemembers",
           LayoutNavEntry::NamespaceMembers,
-          javaOpt    ? theTranslator->trPackageMembers() : fortranOpt ? theTranslator->trModulesMembers() : theTranslator->trNamespaceMembers(),
+          javaOpt || vhdlOpt   ? theTranslator->trPackageMembers() : fortranOpt ? theTranslator->trModulesMembers() : theTranslator->trNamespaceMembers(),
           QCString(),
           fortranOpt ? theTranslator->trModulesMemberDescription(extractAll) : theTranslator->trNamespaceMemberDescription(extractAll),
           "namespacemembers"
         },
         { "classindex",
-          LayoutNavEntry::Classes,
+          LayoutNavEntry::ClassIndex,
           fortranOpt ? theTranslator->trDataTypes() : vhdlOpt ? VhdlDocGen::trDesignUnits() : theTranslator->trCompoundIndex(),
           QCString(),
           QCString(),
           "classes"
         },
         { "classes",
-          LayoutNavEntry::ClassAnnotated,
+          LayoutNavEntry::Classes,
           fortranOpt ? theTranslator->trCompoundListFortran() : vhdlOpt ? VhdlDocGen::trDesignUnitList() : theTranslator->trClasses(),
           theTranslator->trCompoundList(),
           fortranOpt ? theTranslator->trCompoundListDescriptionFortran() : vhdlOpt ? VhdlDocGen::trDesignUnitListDescription() : theTranslator->trCompoundListDescription(),
           "annotated"
         },
+        { "classlist",
+          LayoutNavEntry::ClassList,
+          fortranOpt ? theTranslator->trCompoundListFortran() : vhdlOpt ? VhdlDocGen::trDesignUnitList() : theTranslator->trCompoundList(),
+          QCString(),
+          fortranOpt ? theTranslator->trCompoundListDescriptionFortran() : vhdlOpt ? VhdlDocGen::trDesignUnitListDescription() : theTranslator->trCompoundListDescription(),
+          "annotated"
+        },
         { "hierarchy",
           LayoutNavEntry::ClassHierarchy,
           vhdlOpt    ? VhdlDocGen::trDesignUnitHierarchy() : theTranslator->trClassHierarchy(),
@@ -874,6 +971,13 @@ class LayoutParser : public QXmlDefaultHandler
           theTranslator->trFileListDescription(extractAll),
           "files"
         },
+        { "filelist",
+          LayoutNavEntry::FileList,
+          theTranslator->trFileList(),
+          QCString(),
+          theTranslator->trFileListDescription(extractAll),
+          "files"
+        },
         { "globals",
           LayoutNavEntry::FileGlobals,
           theTranslator->trFileMembers(),
@@ -881,13 +985,13 @@ class LayoutParser : public QXmlDefaultHandler
           theTranslator->trFileMembersDescription(extractAll),
           "globals"
         },
-        { "dirs",
-          LayoutNavEntry::Dirs,
-          theTranslator->trDirectories(),
-          QCString(),
-          theTranslator->trDirDescription(),
-          "dirs"
-        },
+        //{ "dirs",
+        //  LayoutNavEntry::Dirs,
+        //  theTranslator->trDirectories(),
+        //  QCString(),
+        //  theTranslator->trDirDescription(),
+        //  "dirs"
+        //},
         { "examples",
           LayoutNavEntry::Examples,
           theTranslator->trExamples(),
@@ -895,6 +999,20 @@ class LayoutParser : public QXmlDefaultHandler
           theTranslator->trExamplesDescription(),
           "examples"
         },
+        { "user",
+          LayoutNavEntry::User,
+          QCString(),
+          QCString(),
+          QCString(),
+          "user"
+        },
+        { "usergroup",
+          LayoutNavEntry::UserGroup,
+          QCString(),
+          QCString(),
+          QCString(),
+          "usergroup"
+        },
         { 0, // end of list
           (LayoutNavEntry::Kind)0,
           QCString(),
@@ -924,8 +1042,9 @@ class LayoutParser : public QXmlDefaultHandler
         }
         else
         {
-          err("error: the type '%s' is not supported for the entry tag within a navindex! Check your layout file!\n");
+          err("error: the type '%s' is not supported for the entry tag within a navindex! Check your layout file!\n",type.data());
         }
+        m_invalidEntry=TRUE;
         return;
       }
       QCString baseFile = mapping[i].baseFile;
@@ -945,6 +1064,15 @@ class LayoutParser : public QXmlDefaultHandler
       {
         intro = mapping[i].intro;
       }
+      QCString url = convertToQCString(attrib.value("url"));
+      if (mapping[i].kind==LayoutNavEntry::User && !url.isEmpty())
+      {
+        baseFile=url;
+      }
+      else if (kind==LayoutNavEntry::UserGroup)
+      {
+        baseFile+=QCString().sprintf("%d",m_userGroupCount++);
+      }
       // create new item and make it the new root
       m_rootNav = new LayoutNavEntry(m_rootNav,kind,kind==LayoutNavEntry::MainPage?TRUE:isVisible,baseFile,title,intro);
     }
@@ -952,7 +1080,8 @@ class LayoutParser : public QXmlDefaultHandler
     void endNavEntry()
     {
       // set the root back to the parent
-      m_rootNav = m_rootNav->parent();
+      if (m_rootNav && !m_invalidEntry) m_rootNav = m_rootNav->parent();
+      m_invalidEntry=FALSE;
     }
 
     void startClass(const QXmlAttributes &)
@@ -1073,7 +1202,7 @@ class LayoutParser : public QXmlDefaultHandler
                        const QString& name, const QXmlAttributes& attrib )
     {
       //printf("startElement [%s]::[%s]\n",m_scope.data(),name.data());
-      StartElementHandler *handler = m_sHandler[m_scope+name];
+      StartElementHandler *handler = m_sHandler[m_scope+name.utf8()];
       if (handler)
       {
         (*handler)(attrib);
@@ -1089,13 +1218,13 @@ class LayoutParser : public QXmlDefaultHandler
     {
       //printf("endElement [%s]::[%s]\n",m_scope.data(),name.data());
       EndElementHandler *handler;
-      if (!m_scope.isEmpty() && m_scope.right(name.length()+1)==name+"/")
+      if (!m_scope.isEmpty() && m_scope.right(name.length()+1)==name.utf8()+"/")
       { // element ends current scope
         handler = m_eHandler[m_scope.left(m_scope.length()-1)];
       }
       else // continue with current scope
       {
-        handler = m_eHandler[m_scope+name];
+        handler = m_eHandler[m_scope+name.utf8()];
       }
       if (handler)
       {
@@ -1109,15 +1238,19 @@ class LayoutParser : public QXmlDefaultHandler
     }
 
   private:
-    LayoutParser() : m_sHandler(163), m_eHandler(17) { }
+    LayoutParser() : m_sHandler(163), m_eHandler(17), m_invalidEntry(FALSE) { }
 
     QDict<StartElementHandler> m_sHandler;
     QDict<EndElementHandler>   m_eHandler;
-    QString m_scope;
+    QCString m_scope;
     int m_part;
     LayoutNavEntry *m_rootNav;
+    bool m_invalidEntry;
+    static int m_userGroupCount;
 };
 
+int LayoutParser::m_userGroupCount=0;
+
 //---------------------------------------------------------------------------------
 
 class LayoutErrorHandler : public QXmlErrorHandler
@@ -1241,3 +1374,61 @@ void writeDefaultLayoutFile(const char *fileName)
   QTextStream t(&f);
   t << layout_default;
 }
+
+//----------------------------------------------------------------------------------
+
+// Convert input to a title.
+// The format of input can be a simple title "A title" or in case there are different 
+// titles for some programming languages they can take the following form:
+// "A title|16=Another title|8=Yet Another title"
+// where the number is a value of SrcLangExt in decimal notation (i.e. 16=Java, 8=IDL).
+QCString extractLanguageSpecificTitle(const QCString &input,SrcLangExt lang)
+{
+  int i,s=0,e=input.find('|');
+  if (e==-1) return input; // simple title case
+  int e1=e;
+  while (e!=-1) // look for 'number=title' pattern separated by '|'
+  {
+    s=e+1;
+    e=input.find('|',s);
+    i=input.find('=',s);
+    assert(i>s);
+    int key=input.mid(s,i-s).toInt();
+    if (key==(int)lang) // found matching key
+    {
+      if (e==-1) e=input.length();
+      return input.mid(i+1,e-i-1);
+    }
+  }
+  return input.left(e1); // fallback, no explicit language key found
+}
+
+//----------------------------------------------------------------------------------
+
+QCString LayoutDocEntrySection::title(SrcLangExt lang) const
+{
+  return extractLanguageSpecificTitle(m_title,lang);
+}
+
+//----------------------------------------------------------------------------------
+
+QCString LayoutDocEntryMemberDecl::title(SrcLangExt lang) const
+{
+  return extractLanguageSpecificTitle(m_title,lang);
+}
+
+QCString LayoutDocEntryMemberDecl::subtitle(SrcLangExt lang) const
+{
+  return extractLanguageSpecificTitle(m_subscript,lang);
+}
+
+//----------------------------------------------------------------------------------
+
+QCString LayoutDocEntryMemberDef::title(SrcLangExt lang) const
+{
+  return extractLanguageSpecificTitle(m_title,lang);
+}
+
+
+
+
