@@ -2,7 +2,7 @@
  *
  * $Id: htmlgen.cpp,v 1.17 1998/11/28 11:33:19 root Exp $
  *
- * Copyright (C) 1997-2006 by Dimitri van Heesch.
+ * Copyright (C) 1997-2012 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -30,6 +30,7 @@
 #include <string.h>
 #include "docparser.h"
 #include "mandocvisitor.h"
+#include "language.h"
 
 static QCString getExtension()
 {
@@ -100,6 +101,7 @@ void ManGenerator::init()
 static QCString buildFileName(const char *name)
 {
   QCString fileName;
+  if (name==0) return "noname";
 
   const char *p=name;
   char c;
@@ -179,13 +181,31 @@ void ManGenerator::newParagraph()
   paragraph=TRUE;
 }
 
+void ManGenerator::startParagraph()
+{
+  if (!paragraph)
+  {
+    if (!firstCol) t << endl;
+    t << ".PP" << endl;
+    firstCol=TRUE;
+  }
+  paragraph=TRUE;
+}
+
+void ManGenerator::endParagraph()
+{
+}
+
 void ManGenerator::writeString(const char *text)
 {
   docify(text);
 }
 
-void ManGenerator::writeIndexItem(const char *,const char *,
-                                   const char *)
+void ManGenerator::startIndexItem(const char *,const char *)
+{
+}
+
+void ManGenerator::endIndexItem(const char *,const char *)
 {
 }
 
@@ -201,7 +221,8 @@ void ManGenerator::writeObjectLink(const char *,const char *,
 }
 
 void ManGenerator::writeCodeLink(const char *,const char *,
-                                    const char *, const char *name)
+                                 const char *, const char *name,
+                                 const char *)
 {
   docify(name);
 }
@@ -219,7 +240,7 @@ void ManGenerator::endHtmlLink()
 //  docify(url);
 //}
 
-void ManGenerator::startGroupHeader()
+void ManGenerator::startGroupHeader(int)
 {
   if (!firstCol) t << endl;
   t << ".SH \"";
@@ -227,7 +248,7 @@ void ManGenerator::startGroupHeader()
   firstCol=FALSE;
 }
 
-void ManGenerator::endGroupHeader()
+void ManGenerator::endGroupHeader(int)
 {
   t << "\"\n.PP " << endl;
   firstCol=TRUE;
@@ -235,7 +256,7 @@ void ManGenerator::endGroupHeader()
   upperCase=FALSE;
 }
 
-void ManGenerator::startMemberHeader()
+void ManGenerator::startMemberHeader(const char *)
 {
   if (!firstCol) t << endl;
   t << ".SS \"";
@@ -258,6 +279,7 @@ void ManGenerator::docify(const char *str)
     {
       switch(c)
       {
+        case '.':  t << "\\&."; break; // see  bug652277
         case '\\': t << "\\\\"; col++; break;
         case '\n': t << "\n"; col=0; break;
         case '\"':  c = '\''; // no break!
@@ -283,9 +305,10 @@ void ManGenerator::codify(const char *str)
       c=*p++;
       switch(c)
       {
+        case '.':   t << "\\&."; break; // see  bug652277
         case '\t':  spacesToNextTabStop =
                           Config_getInt("TAB_SIZE") - (col%Config_getInt("TAB_SIZE"));
-                    t << spaces.left(spacesToNextTabStop); 
+                    t << Doxygen::spaces.left(spacesToNextTabStop); 
                     col+=spacesToNextTabStop; 
                     break;
         case '\n':  t << "\n"; firstCol=TRUE; col=0; break;
@@ -337,7 +360,7 @@ void ManGenerator::endTitle()
     t << "\"";
 }
 
-void ManGenerator::writeListItem() 
+void ManGenerator::startItemListItem() 
 { 
   if (!firstCol) t << endl; 
   t << ".TP" << endl; 
@@ -346,6 +369,10 @@ void ManGenerator::writeListItem()
   col=0;
 } 
 
+void ManGenerator::endItemListItem()
+{
+}
+
 void ManGenerator::startCodeFragment() 
 { 
   newParagraph();
@@ -363,7 +390,7 @@ void ManGenerator::endCodeFragment()
   col=0;
 }
 
-void ManGenerator::startMemberDoc(const char *,const char *,const char *,const char *) 
+void ManGenerator::startMemberDoc(const char *,const char *,const char *,const char *,bool) 
 { 
   if (!firstCol) t << endl;
   t << ".SS \""; 
@@ -384,8 +411,11 @@ void ManGenerator::startDoxyAnchor(const char *,const char *manName,
     // the name of the link file is derived from the name of the anchor:
     // - truncate after an (optional) ::
     QCString baseName = name;
-    int i=baseName.findRev(':');
-    if (i!=-1) baseName=baseName.right(baseName.length()-i-1);
+    int i=baseName.findRev("::");
+    if (i!=-1) baseName=baseName.right(baseName.length()-i-2);
+
+    //printf("Converting man link '%s'->'%s'->'%s'\n",
+    //       name,baseName.data(),buildFileName(baseName).data());
     
     // - remove dangerous characters and append suffix, then add dir prefix
     QCString fileName=dir+"/"+buildFileName( baseName );
@@ -395,11 +425,9 @@ void ManGenerator::startDoxyAnchor(const char *,const char *manName,
     {
 	if ( linkfile.open( IO_WriteOnly ) ) 
         {
-	      QTextStream linkstream;
+	      FTextStream linkstream;
 	      linkstream.setDevice(&linkfile);
-#if QT_VERSION >= 200
-	      linkstream.setEncoding(QTextStream::Latin1);
-#endif
+	      //linkstream.setEncoding(QTextStream::UnicodeUTF8);
 	      linkstream << ".so man" << getExtension() << "/" << buildFileName( manName ) << endl;
 	}
     }
@@ -408,7 +436,7 @@ void ManGenerator::startDoxyAnchor(const char *,const char *manName,
 
 void ManGenerator::endMemberDoc(bool)
 {
-    t << "\"";
+    t << "\"\n";
 }
 
 void ManGenerator::startSubsection()    
@@ -459,7 +487,7 @@ void ManGenerator::startDescItem()
 //  paragraph=TRUE;
 //}
 
-void ManGenerator::writeDescItem()
+void ManGenerator::startDescForItem()
 {
   if (!firstCol) t << endl;
   if (!paragraph) t << ".in -1c" << endl;
@@ -469,6 +497,10 @@ void ManGenerator::writeDescItem()
   col=0;
 }
 
+void ManGenerator::endDescForItem()
+{
+}
+
 void ManGenerator::endDescItem()
 {
   t << "\" 1c" << endl;
@@ -492,7 +524,7 @@ void ManGenerator::endAnonTypeScope(int indentLevel)
 }
 
 
-void ManGenerator::startMemberItem(int) 
+void ManGenerator::startMemberItem(const char *,int,const char *) 
 { 
   if (firstCol && !insideTabbing) t << ".in +1c\n";
   t << "\n.ti -1c\n.RI \""; 
@@ -557,11 +589,11 @@ void ManGenerator::startSection(const char *,const char *,SectionInfo::SectionTy
   {
     switch(type)
     {
-      case SectionInfo::Page:          startGroupHeader(); break;
-      case SectionInfo::Section:       startGroupHeader(); break;
-      case SectionInfo::Subsection:    startMemberHeader(); break;
-      case SectionInfo::Subsubsection: startMemberHeader(); break;
-      case SectionInfo::Paragraph:     startMemberHeader(); break;
+      case SectionInfo::Page:          startGroupHeader(FALSE); break;
+      case SectionInfo::Section:       startGroupHeader(FALSE); break;
+      case SectionInfo::Subsection:    startMemberHeader(0); break;
+      case SectionInfo::Subsubsection: startMemberHeader(0); break;
+      case SectionInfo::Paragraph:     startMemberHeader(0); break;
       default: ASSERT(0); break;
     }
   }
@@ -573,8 +605,8 @@ void ManGenerator::endSection(const char *,SectionInfo::SectionType type)
   {
     switch(type)
     {
-      case SectionInfo::Page:          endGroupHeader(); break;
-      case SectionInfo::Section:       endGroupHeader(); break;
+      case SectionInfo::Page:          endGroupHeader(0); break;
+      case SectionInfo::Section:       endGroupHeader(0); break;
       case SectionInfo::Subsection:    endMemberHeader(); break;
       case SectionInfo::Subsubsection: endMemberHeader(); break;
       case SectionInfo::Paragraph:     endMemberHeader(); break;
@@ -627,12 +659,144 @@ void ManGenerator::endParamList()
 {
 }
 
-void ManGenerator::printDoc(DocNode *n,const char *langExt)
+void ManGenerator::writeDoc(DocNode *n,Definition *ctx,MemberDef *)
 {
-  ManDocVisitor *visitor = new ManDocVisitor(t,*this,langExt);
+  ManDocVisitor *visitor = new ManDocVisitor(t,*this,ctx?ctx->getDefFileExtension():QCString(""));
   n->accept(visitor);
   delete visitor; 
   firstCol=FALSE;
   paragraph = FALSE;
 }
 
+void ManGenerator::startConstraintList(const char *header)
+{
+  if (!firstCol) 
+  { t << endl << ".PP" << endl; 
+    firstCol=TRUE; paragraph=TRUE; 
+    col=0;
+  }
+  paragraph=FALSE;
+  startBold();
+  docify(header);
+  endBold();
+  paragraph=TRUE;
+}
+
+void ManGenerator::startConstraintParam()
+{
+  startItemListItem();
+  startEmphasis();
+}
+
+void ManGenerator::endConstraintParam()
+{
+  endEmphasis();
+  endItemListItem();
+  t << " : ";
+}
+
+void ManGenerator::startConstraintType()
+{
+  startEmphasis();
+}
+
+void ManGenerator::endConstraintType()
+{
+  endEmphasis();
+}
+
+void ManGenerator::startConstraintDocs()
+{
+}
+
+void ManGenerator::endConstraintDocs()
+{
+  t << endl; firstCol=TRUE;
+}
+
+void ManGenerator::endConstraintList()
+{
+}
+
+
+void ManGenerator::startInlineHeader() 
+{
+  if (!firstCol) 
+  {
+    t << endl << ".PP" << endl << ".in -1c" << endl;
+  }
+  t << ".RI \"\\fB"; 
+}
+
+void ManGenerator::endInlineHeader() 
+{
+  t << "\\fP\"" << endl << ".in +1c" << endl;
+  firstCol = FALSE;
+}
+
+void ManGenerator::startMemberDocSimple()
+{
+  if (!firstCol) 
+  {
+    t << endl << ".PP" << endl;
+  }
+  t << "\\fB";
+  docify(theTranslator->trCompoundMembers());
+  t << ":\\fP" << endl;
+  t << ".RS 4" << endl;
+}
+
+void ManGenerator::endMemberDocSimple()
+{
+  if (!firstCol) t << endl;
+  t << ".RE" << endl;
+  t << ".PP" << endl;
+  firstCol=TRUE;
+}
+
+void ManGenerator::startInlineMemberType()
+{
+}
+
+void ManGenerator::endInlineMemberType()
+{
+  t << " ";
+}
+
+void ManGenerator::startInlineMemberName()
+{
+  t << "\\fI";
+}
+
+void ManGenerator::endInlineMemberName()
+{
+  t << "\\fP ";
+}
+
+void ManGenerator::startInlineMemberDoc()
+{
+}
+
+void ManGenerator::endInlineMemberDoc()
+{
+  if (!firstCol) t << endl;
+  t << ".br" << endl;
+  t << ".PP" << endl;
+  firstCol=TRUE;
+}
+
+void ManGenerator::startLabels()
+{
+}
+
+void ManGenerator::writeLabel(const char *l,bool isLast)
+{
+  t << "\\fC [" << l << "]\\fP";
+  if (!isLast) t << ", ";
+}
+
+void ManGenerator::endLabels()
+{
+}
+
+
