@@ -2,7 +2,7 @@
  *
  * 
  *
- * Copyright (C) 1997-2007 by Dimitri van Heesch.
+ * Copyright (C) 1997-2012 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -23,7 +23,7 @@
 #include <qdict.h>
 #include <qstack.h>
 
-#include "entry.h"
+#include "types.h"
 #include "definition.h"
 #include "sortdict.h"
 
@@ -40,12 +40,7 @@ class QTextStream;
 class ArgumentList;
 class MemberDefImpl;
 
-struct SourceReference
-{
-  FileDef *fd;
-  QCString anchor;
-};
-
+/** A model of a class/file/namespace member symbol. */
 class MemberDef : public Definition
 {
   public:
@@ -57,7 +52,6 @@ class MemberDef : public Definition
       Typedef, 
       Enumeration, 
       EnumValue,
-      Prototype,
       Signal,
       Slot,
       Friend,
@@ -69,10 +63,13 @@ class MemberDef : public Definition
     MemberDef(const char *defFileName,int defLine,
               const char *type,const char *name,const char *args,
               const char *excp,Protection prot,Specifier virt,bool stat,
-              bool related,MemberType t,const ArgumentList *tal,
+              Relationship related,MemberType t,const ArgumentList *tal,
               const ArgumentList *al);
    ~MemberDef(); 
     DefType definitionType() const        { return TypeMember; }
+    // move this member into a different scope
+    MemberDef *deepCopy() const;
+    void moveTo(Definition *);
     
     //-----------------------------------------------------------------------------------
     // ----  getters -----
@@ -110,7 +107,7 @@ class MemberDef : public Definition
     const char *getGroupFileName() const;
     int getGroupStartLine() const;
     bool getGroupHasDocs() const;
-    QCString qualifiedName();
+    QCString qualifiedName() const;
     QCString objCMethodName(bool localLink,bool showStatic) const; 
 
     // direct kind info 
@@ -127,12 +124,14 @@ class MemberDef : public Definition
     bool isEnumValue() const;
     bool isTypedef() const;
     bool isFunction() const;
+    bool isFunctionPtr() const;
     bool isDefine() const;
     bool isFriend() const;
     bool isDCOP() const;
     bool isProperty() const;
     bool isEvent() const;
     bool isRelated() const;
+    bool isForeign() const;
     bool isStatic() const;
     bool isInline() const;
     bool isExplicit() const;
@@ -148,14 +147,28 @@ class MemberDef : public Definition
     bool isAbstract() const;
     bool isOverride() const;
     bool isInitonly() const;
+    bool isOptional() const;
+    bool isRequired() const;
+    bool isNonAtomic() const;
+    bool isCopy() const;
+    bool isAssign() const;
+    bool isRetain() const;
+    bool isWeak() const;
+    bool isStrong() const;
+    bool isUnretained() const;
     bool isNew() const;
     bool isSealed() const;
     bool isImplementation() const;
     bool isExternal() const;
+    bool isAlias() const;
+    bool isDefault() const;
+    bool isDelete() const;
+    bool isNoExcept() const;
     bool isTemplateSpecialization() const;
     bool hasDocumentedParams() const;
     bool hasDocumentedReturnType() const;
     bool isObjCMethod() const;
+    bool isObjCProperty() const;
     bool isConstructor() const;
     bool isDestructor() const;
     bool hasOneLineInitializer() const;
@@ -166,6 +179,7 @@ class MemberDef : public Definition
     bool isLinkableInProject() const;
     bool isLinkable() const;
     bool hasDocumentation() const;  // overrides hasDocumentation in definition.h
+    //bool hasUserDocumentation() const; // overrides hasUserDocumentation
     bool isBriefSectionVisible() const;
     bool isDetailedSectionVisible(bool inGroup,bool inFile) const;
     bool isDetailedSectionLinkable() const;
@@ -173,11 +187,12 @@ class MemberDef : public Definition
     bool isDocumentedFriendClass() const;
 
     MemberDef *reimplements() const;
-    LockingPtr<MemberList> reimplementedBy() const;
+    LockingPtr< MemberList > reimplementedBy() const;
+    bool isReimplementedBy(ClassDef *cd) const;
 
-    int inbodyLine() const;
-    QCString inbodyFile() const;
-    const QCString &inbodyDocumentation() const;
+    //int inbodyLine() const;
+    //QCString inbodyFile() const;
+    //const QCString &inbodyDocumentation() const;
 
     ClassDef *relatedAlso() const;
 
@@ -185,7 +200,9 @@ class MemberDef : public Definition
     MemberDef *getAnonymousEnumType() const;
     bool isDocsForDefinition() const;
     MemberDef *getEnumScope() const;
-    LockingPtr<MemberList> enumFieldList() const;
+    LockingPtr< MemberList > enumFieldList() const;
+    void setEnumBaseType(const QCString &type);
+    QCString enumBaseType() const;
 
     bool hasExamples();
     LockingPtr<ExampleSDict> getExamples() const;
@@ -224,6 +241,11 @@ class MemberDef : public Definition
     MemberDef *inheritsDocsFrom() const;
     MemberDef *getGroupAlias() const;
 
+    ClassDef *category() const;
+    MemberDef *categoryRelation() const;
+
+    QCString displayName(bool=TRUE) const;
+
     //-----------------------------------------------------------------------------------
     // ----  setters -----
     //-----------------------------------------------------------------------------------
@@ -250,6 +272,7 @@ class MemberDef : public Definition
     void setTemplateSpecialization(bool b);
     
     void makeRelated();
+    void makeForeign();
     void setHasDocumentedParams(bool b);
     void setHasDocumentedReturnType(bool b);
     void setInheritsDocsFrom(MemberDef *md);
@@ -261,7 +284,7 @@ class MemberDef : public Definition
     void insertReimplementedBy(MemberDef *md);
 
     // in-body documentation
-    void setInbodyDocumentation(const char *docs,const char *file,int line);
+    //void setInbodyDocumentation(const char *docs,const char *file,int line);
 
     void setRelatedAlso(ClassDef *cd);
 
@@ -283,6 +306,8 @@ class MemberDef : public Definition
     void setDeclArgumentList(ArgumentList *al);
     void setDefinitionTemplateParameterLists(QList<ArgumentList> *lists);
     void setTypeConstraints(ArgumentList *al);
+    void setType(const char *t);
+    void setAccessorType(ClassDef *cd,const char *t);
 
     // namespace related members
     void setNamespace(NamespaceDef *nd);
@@ -306,6 +331,8 @@ class MemberDef : public Definition
 
     void cacheTypedefVal(ClassDef *val,const QCString &templSpec,const QCString &resolvedType);
     void invalidateTypedefValCache();
+
+    void invalidateCachedArgumentTypes();
     
     // declaration <-> definition relation
     void setMemberDefinition(MemberDef *md);
@@ -313,7 +340,16 @@ class MemberDef : public Definition
         
     void setAnonymousUsed();
     void copyArgumentNames(MemberDef *bmd);
-    
+
+    void setCategory(ClassDef *);
+    void setCategoryRelation(MemberDef *);
+
+    void setDocumentation(const char *d,const char *docFile,int docLine,bool stripWhiteSpace=TRUE);
+    void setBriefDescription(const char *b,const char *briefFile,int briefLine);
+    void setInbodyDocumentation(const char *d,const char *inbodyFile,int inbodyLine);
+
+    void setHidden(bool b);
+
     //-----------------------------------------------------------------------------------
     // --- actions ----
     //-----------------------------------------------------------------------------------
@@ -321,10 +357,12 @@ class MemberDef : public Definition
     // output generation
     void writeDeclaration(OutputList &ol,
                    ClassDef *cd,NamespaceDef *nd,FileDef *fd,GroupDef *gd,
-                   bool inGroup); 
+                   bool inGroup,ClassDef *inheritFrom=0,const char *inheritId=0); 
     void writeDocumentation(MemberList *ml,OutputList &ol,
                             const char *scopeName,Definition *container,
-                            bool inGroup,bool showEnumValues=FALSE);
+                            bool inGroup,bool showEnumValues=FALSE,bool
+                            showInline=FALSE);
+    void writeMemberDocSimple(OutputList &ol,Definition *container);
     void warnIfUndocumented();
     
     MemberDef *createTemplateInstanceMember(ArgumentList *formalArgs,
@@ -345,6 +383,20 @@ class MemberDef : public Definition
     void unlock() const;
     void saveToDisk() const;
     void makeResident() const;
+    void _computeLinkableInProject();
+    void _computeIsConstructor();
+    void _computeIsDestructor();
+    void _getLabels(QStrList &sl,Definition *container) const;
+    void _writeCallGraph(OutputList &ol);
+    void _writeCallerGraph(OutputList &ol);
+    void _writeReimplements(OutputList &ol);
+    void _writeReimplementedBy(OutputList &ol);
+    void _writeExamples(OutputList &ol);
+    void _writeTypeConstraints(OutputList &ol);
+    void _writeEnumValues(OutputList &ol,Definition *container,
+                          const QCString &cfname,const QCString &ciname,
+                          const QCString &cname);
+    void _writeCategoryRelation(OutputList &ol);
 
     static int s_indentLevel;
     // disable copying of member defs
@@ -359,6 +411,11 @@ class MemberDef : public Definition
     int m_cacheHandle;
     off_t m_storagePos;     // location where the item is stored in file (if impl==0)
     bool m_flushPending;
+    uchar m_isLinkableCached;    // 0 = not cached, 1=FALSE, 2=TRUE
+    uchar m_isConstructorCached; // 0 = not cached, 1=FALSE, 2=TRUE
+    uchar m_isDestructorCached;  // 0 = not cached, 1=FALSE, 2=TRUE
 };
 
+void combineDeclarationAndDefinition(MemberDef *mdec,MemberDef *mdef);
+
 #endif
