@@ -2,7 +2,7 @@
  *
  * $Id: memberlist.cpp,v 1.35 2001/03/19 19:27:41 root Exp $
  *
- * Copyright (C) 1997-2011 by Dimitri van Heesch.
+ * Copyright (C) 1997-2012 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -30,6 +30,9 @@
 
 MemberList::MemberList()
 {
+  memberGroupList=0;
+  m_numDecMembers=-1; // special value indicating that value needs to be computed
+  m_numDocMembers=-1; // special value indicating that value needs to be computed
 }
 
 MemberList::MemberList(ListType lt) : m_listType(lt)
@@ -64,6 +67,47 @@ int MemberList::compareItems(GCI item1, GCI item2)
   return cmp!=0 ? cmp : c1->getDefLine()-c2->getDefLine();
 }
 
+int MemberList::countInheritableMembers(ClassDef *inheritedFrom) const
+{
+  int count=0;
+  QListIterator<MemberDef> mli(*this);
+  MemberDef *md;
+  for (mli.toFirst();(md=mli.current());++mli)
+  {
+    if (md->isBriefSectionVisible())
+    {
+      if (md->memberType()!=MemberDef::Friend &&
+          md->memberType()!=MemberDef::EnumValue)
+      {
+        //printf("member %s: isReimplementedBy(%s)=%d\n",md->name().data(),
+        //    inheritedFrom->name().data(),
+        //    md->isReimplementedBy(inheritedFrom));
+        if (md->memberType()==MemberDef::Function)
+        {
+          if (!md->isReimplementedBy(inheritedFrom)) count++;
+        }
+        else
+        {
+          count++;
+        }
+      }
+    }
+  }
+  if (memberGroupList)
+  {
+    MemberGroupListIterator mgli(*memberGroupList);
+    MemberGroup *mg;
+    for (;(mg=mgli.current());++mgli)
+    {
+      count+=mg->countInheritableMembers(inheritedFrom);
+    }
+  }
+  //printf("%s::countInheritableMembers(%s)=%d\n",
+  //    listTypeAsString().data(),
+  //    inheritedFrom->name().data(),count);
+  return count;
+}
+
 /*! Count the number of members in this list that are visible in
  *  the declaration part of a compound's documentation page.
  */
@@ -248,7 +292,8 @@ bool MemberList::declVisible() const
 }
 
 void MemberList::writePlainDeclarations(OutputList &ol,
-                       ClassDef *cd,NamespaceDef *nd,FileDef *fd,GroupDef *gd
+                       ClassDef *cd,NamespaceDef *nd,FileDef *fd,GroupDef *gd,
+                       ClassDef *inheritedFrom,const char *inheritId
                       )
 {
   //printf("----- writePlainDeclaration() ----\n");
@@ -270,7 +315,8 @@ void MemberList::writePlainDeclarations(OutputList &ol,
   {
     //printf(">>> Member `%s' type=%d visible=%d\n",
     //    md->name().data(),md->memberType(),md->isBriefSectionVisible());
-    if (md->isBriefSectionVisible())
+    if ((inheritedFrom==0 || !md->isReimplementedBy(inheritedFrom)) &&
+        md->isBriefSectionVisible())
     {
       switch(md->memberType())
       {
@@ -286,7 +332,7 @@ void MemberList::writePlainDeclarations(OutputList &ol,
         case MemberDef::Event:  
           {
             if (first) ol.startMemberList(),first=FALSE;
-            md->writeDeclaration(ol,cd,nd,fd,gd,m_inGroup);
+            md->writeDeclaration(ol,cd,nd,fd,gd,m_inGroup,inheritedFrom,inheritId);
             break;
           }
         case MemberDef::Enumeration: 
@@ -319,14 +365,15 @@ void MemberList::writePlainDeclarations(OutputList &ol,
                 ol.startMemberList();
                 first=FALSE;
               }
-              ol.startMemberItem(0);
+              ol.startMemberDeclaration();
+              ol.startMemberItem(md->anchor(),0,inheritId);
               ol.writeString("enum ");
               ol.insertMemberAlign();
               md->writeEnumDeclaration(ol,cd,nd,fd,gd);
               ol.endMemberItem();
               if (!md->briefDescription().isEmpty() && Config_getBool("BRIEF_MEMBER_DESC"))
               {
-                ol.startMemberDescription();
+                ol.startMemberDescription(md->anchor());
                 ol.parseDoc(
                     md->briefFile(),md->briefLine(),
                     cd,md,
@@ -345,18 +392,20 @@ void MemberList::writePlainDeclarations(OutputList &ol,
                 }
                 ol.endMemberDescription();
               }
+              ol.endMemberDeclaration(md->anchor(),inheritId);
             }
             md->warnIfUndocumented();
             break;
           }
         case MemberDef::Friend:
+          if (inheritedFrom==0)
           {
             if (first) 
             {
               ol.startMemberList();
               first=FALSE;
             }
-            md->writeDeclaration(ol,cd,nd,fd,gd,m_inGroup);
+            md->writeDeclaration(ol,cd,nd,fd,gd,m_inGroup,inheritedFrom,inheritId);
             break;
           }
         case MemberDef::EnumValue: 
@@ -365,7 +414,7 @@ void MemberList::writePlainDeclarations(OutputList &ol,
             {
               //printf("EnumValue!\n");
               if (first) ol.startMemberList(),first=FALSE;
-              md->writeDeclaration(ol,cd,nd,fd,gd,m_inGroup);
+              md->writeDeclaration(ol,cd,nd,fd,gd,m_inGroup,inheritedFrom,inheritId);
             }
           }
           break;
@@ -407,101 +456,154 @@ void MemberList::writePlainDeclarations(OutputList &ol,
   //printf("----- end writePlainDeclaration() ----\n");
 }
 
+/** Writes the list of members to the output.
+ *  @param ol Output list to write to
+ *  @param cd non-null if this list is part of class documentation.
+ *  @param nd non-null if this list is part of namespace documentation.
+ *  @param fd non-null if this list is part of file documentation.
+ *  @param gd non-null if this list is part of group documentation.
+ *  @param title Title to use for the member list.
+ *  @param subtitle Sub title to use for the member list.
+ *  @param showEnumValues Obsolete, always set to FALSE.
+ *  @param showInline if set to TRUE if title is rendered differently
+ *  @param inheritedFrom if not 0, the list is shown inside the
+ *         given class as inherited members, parameter cd points to the
+ *         class containing the members.
+ */
 void MemberList::writeDeclarations(OutputList &ol,
              ClassDef *cd,NamespaceDef *nd,FileDef *fd,GroupDef *gd,
              const char *title,const char *subtitle, bool showEnumValues,
-             bool showInline)
+             bool showInline,ClassDef *inheritedFrom)
 {
-  //printf("----- writeDeclaration() this=%p ----\n",this);
-  static bool optimizeVhdl = Config_getBool("OPTIMIZE_OUTPUT_VHDL");
+  (void)showEnumValues; // unused
 
-  countDecMembers(showEnumValues,gd); // count members shown in this section
+  //printf("----- writeDeclaration() this=%p ---- inheritedFrom=%p\n",this,inheritedFrom);
+  static bool optimizeVhdl = Config_getBool("OPTIMIZE_OUTPUT_VHDL");
+  QCString inheritId;
+
+  countDecMembers(/*showEnumValues*/FALSE,gd); // count members shown in this section
   Definition *ctx = cd;
   if (ctx==0 && nd) ctx = nd;
   if (ctx==0 && gd) ctx = gd;
   if (ctx==0 && fd) ctx = fd;
 
-  if (numDecMembers()==0) return;
   //printf("%p: MemberList::writeDeclaration(title=`%s',subtitle=`%s')=%d\n",
-  //    this,title,subtitle,numDecMembers());
-  if (title) 
-  {
-    if (showInline)
-    {
-      ol.startInlineHeader();
-    }
-    else
-    {
-      ol.startMemberHeader(listTypeAsString());
-    }
-    ol.parseText(title);
-    if (showInline)
-    {
-      ol.endInlineHeader();
-    }
-    else
-    {
-      ol.endMemberHeader();
-    }
-  }
-  if (subtitle) 
-  {
-    QCString st=subtitle;
-    st = st.stripWhiteSpace();
-    if (!st.isEmpty())
-    {
-      ol.startMemberSubtitle();
-      ol.parseDoc("[generated]",-1,ctx,0,subtitle,FALSE,FALSE,0,FALSE,FALSE);
-      ol.endMemberSubtitle();
-    }
-  }
+  //       this,title,subtitle,numDecMembers());
 
-  // TODO: Two things need to be worked out for proper VHDL output:
-  // 1. Signals and types under the group need to be
-  //    formatted to associate them with the group somehow
-  //    indentation, or at the very least, extra space after
-  //    the group is done
-  // 2. This might need to be repeated below for memberGroupLists
-  if (optimizeVhdl) // use specific declarations function
+  int num = numDecMembers();
+  if (inheritedFrom)
   {
-    VhdlDocGen::writeVhdlDeclarations(this,ol,0,cd,0);
-  }
-  else
-  {
-    writePlainDeclarations(ol,cd,nd,fd,gd);
-  }
- 
-  //printf("memberGroupList=%p\n",memberGroupList);
-  if (memberGroupList)
-  {
-    MemberGroupListIterator mgli(*memberGroupList);
-    MemberGroup *mg;
-    while ((mg=mgli.current()))
+    //if ( cd && !optimizeVhdl && countInheritableMembers(inheritedFrom)>0 )
+    if ( cd && !optimizeVhdl && cd->countMembersIncludingGrouped(
+                                      m_listType,inheritedFrom,TRUE)>0 )
     {
-      bool hasHeader=!mg->header().isEmpty() && mg->header()!="[NOHEADER]";
-      //printf("mg->header=%s hasHeader=%d\n",mg->header().data(),hasHeader);
-      ol.startMemberGroupHeader(hasHeader);
-      if (hasHeader)
+      ol.pushGeneratorState();
+      ol.disableAllBut(OutputGenerator::Html);
+      inheritId = substitute(listTypeAsString(),"-","_")+"_"+
+                  stripPath(cd->getOutputFileBase());
+      if (title)
       {
-        ol.parseText(mg->header());
+        ol.writeInheritedSectionTitle(inheritId,cd->getReference(),
+                                      cd->getOutputFileBase(), 
+                                      cd->anchor(),title,cd->displayName());
       }
-      ol.endMemberGroupHeader();
-      if (!mg->documentation().isEmpty())
-      {
-        //printf("Member group has docs!\n");
-        ol.startMemberGroupDocs();
-        ol.parseDoc("[generated]",-1,ctx,0,mg->documentation()+"\n",FALSE,FALSE);
-        ol.endMemberGroupDocs();
-      }
-      ol.startMemberGroup();
-      //printf("--- mg->writePlainDeclarations ---\n");
-      mg->writePlainDeclarations(ol,cd,nd,fd,gd);
-      ol.endMemberGroup(hasHeader);
-      ++mgli;
+      ol.popGeneratorState();
     }
   }
+  else if (num>0)
+  {
+    if (title) 
+    {
+      if (showInline)
+      {
+        ol.startInlineHeader();
+      }
+      else
+      {
+        ol.startMemberHeader(listTypeAsString());
+      }
+      ol.parseText(title);
+      if (showInline)
+      {
+        ol.endInlineHeader();
+      }
+      else
+      {
+        ol.endMemberHeader();
+      }
+    }
+    if (subtitle) 
+    {
+      QCString st=subtitle;
+      st = st.stripWhiteSpace();
+      if (!st.isEmpty())
+      {
+        ol.startMemberSubtitle();
+        ol.parseDoc("[generated]",-1,ctx,0,subtitle,FALSE,FALSE,0,FALSE,FALSE);
+        ol.endMemberSubtitle();
+      }
+    }
+  }
+  if (num>0)
+  {
+    // TODO: Two things need to be worked out for proper VHDL output:
+    // 1. Signals and types under the group need to be
+    //    formatted to associate them with the group somehow
+    //    indentation, or at the very least, extra space after
+    //    the group is done
+    // 2. This might need to be repeated below for memberGroupLists
+    if (optimizeVhdl) // use specific declarations function
+    {
+      VhdlDocGen::writeVhdlDeclarations(this,ol,0,cd,0,0);
+    }
+    else
+    {
+      writePlainDeclarations(ol,cd,nd,fd,gd,inheritedFrom,inheritId);
+    }
+
+    //printf("memberGroupList=%p\n",memberGroupList);
+    if (memberGroupList)
+    {
+      MemberGroupListIterator mgli(*memberGroupList);
+      MemberGroup *mg;
+      while ((mg=mgli.current()))
+      {
+        bool hasHeader=!mg->header().isEmpty() && mg->header()!="[NOHEADER]";
+        if (inheritId.isEmpty())
+        {
+          //printf("mg->header=%s hasHeader=%d\n",mg->header().data(),hasHeader);
+          ol.startMemberGroupHeader(hasHeader);
+          if (hasHeader)
+          {
+            ol.parseText(mg->header());
+          }
+          ol.endMemberGroupHeader();
+          if (!mg->documentation().isEmpty())
+          {
+            //printf("Member group has docs!\n");
+            ol.startMemberGroupDocs();
+            ol.parseDoc("[generated]",-1,ctx,0,mg->documentation()+"\n",FALSE,FALSE);
+            ol.endMemberGroupDocs();
+          }
+          ol.startMemberGroup();
+        }
+        //printf("--- mg->writePlainDeclarations ---\n");
+        mg->writePlainDeclarations(ol,cd,nd,fd,gd,inheritedFrom,inheritId);
+        if (inheritId.isEmpty())
+        {
+          ol.endMemberGroup(hasHeader);
+        }
+        ++mgli;
+      }
+    }
+  }
+  if (inheritedFrom && cd) 
+  {
+    // also add members that of this list type, that are grouped together
+    // in a separate list in class 'inheritedFrom'
+    cd->addGroupedInheritedMembers(ol,m_listType,inheritedFrom,inheritId);
+  }
   //printf("----- end writeDeclaration() ----\n");
-
 }
 
 void MemberList::writeDocumentation(OutputList &ol,
@@ -515,7 +617,10 @@ void MemberList::writeDocumentation(OutputList &ol,
 
   if (title)
   {
-    ol.writeRuler();
+    ol.pushGeneratorState();
+      ol.disable(OutputGenerator::Html);
+      ol.writeRuler();
+    ol.popGeneratorState();
     ol.startGroupHeader(showInline ? 2 : 0);
     ol.parseText(title);
     ol.endGroupHeader(showInline ? 2 : 0);
@@ -542,6 +647,7 @@ void MemberList::writeDocumentation(OutputList &ol,
   ol.endMemberDocList();
 }
 
+// members in a table
 void MemberList::writeSimpleDocumentation(OutputList &ol,
                      Definition *container)
 {
@@ -559,6 +665,7 @@ void MemberList::writeSimpleDocumentation(OutputList &ol,
   ol.endMemberDocSimple();
 }
 
+// separate member pages
 void MemberList::writeDocumentationPage(OutputList &ol,
                      const char *scopeName, Definition *container)
 {
@@ -578,30 +685,30 @@ void MemberList::writeDocumentationPage(OutputList &ol,
     }
     ol.startContents();
 
-    ol.writeString("<table cellspacing=\"0\" cellpadding=\"0\" border=\"0\">\n"
-                   "  <tr>\n"
-                   "   <td valign=\"top\">\n");
-
-    container->writeQuickMemberLinks(ol,md);
-
-    ol.writeString("   </td>\n");
-    ol.writeString("   <td valign=\"top\" class=\"mempage\">\n");
-    
-    md->writeDocumentation(this,ol,scopeName,container,m_inGroup);
-
-    ol.writeString("    </td>\n");
-    ol.writeString("  </tr>\n");
-    ol.writeString("</table>\n");
-
 
     if (generateTreeView)
     {
+      md->writeDocumentation(this,ol,scopeName,container,m_inGroup);
       ol.endContents();
-      container->writeNavigationPath(ol);
-      endFile(ol,TRUE);
+      endFileWithNavPath(container,ol);
     }
     else
     {
+      ol.writeString("<table cellspacing=\"0\" cellpadding=\"0\" border=\"0\">\n"
+          "  <tr>\n"
+          "   <td valign=\"top\">\n");
+
+      container->writeQuickMemberLinks(ol,md);
+
+      ol.writeString("   </td>\n");
+      ol.writeString("   <td valign=\"top\" class=\"mempage\">\n");
+
+      md->writeDocumentation(this,ol,scopeName,container,m_inGroup);
+
+      ol.writeString("    </td>\n");
+      ol.writeString("  </tr>\n");
+      ol.writeString("</table>\n");
+
       endFile(ol);
     }
   }
@@ -808,10 +915,22 @@ void MemberList::setNeedsSorting(bool b)
 
 int MemberSDict::compareItems(GCI item1, GCI item2)
 {
+  // NOTE: this function can be triggered from unmarshalMemberSDict
+  // so it may not result in called to MemberDef::makeResident().
+  // As a result, the data returned by MemberDef::name() and 
+  // MemberDef::getDefLine() will always be kept in memory.
   MemberDef *c1=(MemberDef *)item1;
   MemberDef *c2=(MemberDef *)item2;
+  //printf("MemberSDict::compareItems(%s,%s)\n",c1->name().data(),c2->name().data());
   int cmp = stricmp(c1->name(),c2->name());
-  return cmp!=0 ? cmp : c1->getDefLine()-c2->getDefLine();
+  if (cmp)
+  {
+    return cmp;
+  }
+  else
+  {
+    return c1->getDefLine()-c2->getDefLine();
+  }
 }
 
 
