@@ -2,7 +2,7 @@
  *
  * $Id: outputgen.h,v 1.48 2001/03/19 19:27:41 root Exp $
  *
- * Copyright (C) 1997-2011 by Dimitri van Heesch.
+ * Copyright (C) 1997-2012 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -38,7 +38,7 @@ class MemberDef;
 class GroupDef;
 class Definition;
 
-/*! \brief Output interface for code parser. 
+/** Output interface for code parser. 
  */
 class CodeOutputInterface
 {
@@ -65,7 +65,7 @@ class CodeOutputInterface
 
     virtual void writeLineNumber(const char *ref,const char *file,
                                  const char *anchor,int lineNumber) = 0;
-    virtual void startCodeLine() = 0;
+    virtual void startCodeLine(bool hasLineNumbers) = 0;
     virtual void endCodeLine() = 0;
     virtual void startCodeAnchor(const char *label) = 0;
     virtual void endCodeAnchor() = 0;
@@ -76,7 +76,7 @@ class CodeOutputInterface
                  Definition *symDef,Definition *context) = 0;
 };
 
-/*! \brief Base Interface used for generating output outside of the
+/** Base Interface used for generating output outside of the
  *  comment blocks.
  *
  *  This abstract class is used by output generation functions
@@ -97,8 +97,6 @@ class BaseOutputDocInterface : public CodeOutputInterface
                         Examples 
                       };
 
-    virtual void parseDoc(const char *,int, const char *,MemberDef *,
-                          const QCString &,bool)  {} 
     virtual void parseText(const QCString &)  {}
     
     /*! Start of a bullet list: e.g. \c \<ul\> in html. startItemListItem() is
@@ -263,7 +261,7 @@ class BaseOutputDocInterface : public CodeOutputInterface
     virtual void endSubsubsection() = 0;
 };
 
-/*! \brief Abstract output generator.
+/** Abstract output generator.
  *
  *  Subclass this class to add support for a new output format
  */
@@ -294,14 +292,15 @@ class OutputGenerator : public BaseOutputDocInterface
     //void setEncoding(const QCString &enc) { encoding = enc; }
     //virtual void postProcess(QByteArray &) { }
 
-    virtual void printDoc(DocNode *,const char *langExt) = 0;
+    virtual void writeDoc(DocNode *,Definition *ctx,MemberDef *md) = 0;
 
     ///////////////////////////////////////////////////////////////
     // structural output interface
     ///////////////////////////////////////////////////////////////
     virtual void startFile(const char *name,const char *manName,
                            const char *title) = 0;
-    virtual void writeFooter() = 0;
+    virtual void writeSearchInfo() = 0;
+    virtual void writeFooter(const char *navPath) = 0;
     virtual void endFile() = 0;
     virtual void startIndexSection(IndexSections) = 0;
     virtual void endIndexSection(IndexSections) = 0;
@@ -339,10 +338,10 @@ class OutputGenerator : public BaseOutputDocInterface
     virtual void endInlineHeader() = 0;
     virtual void startAnonTypeScope(int) = 0;
     virtual void endAnonTypeScope(int) = 0;
-    virtual void startMemberItem(int) = 0;
+    virtual void startMemberItem(const char *,int,const char *) = 0;
     virtual void endMemberItem() = 0;
     virtual void startMemberTemplateParams() = 0;
-    virtual void endMemberTemplateParams() = 0;
+    virtual void endMemberTemplateParams(const char *,const char *) = 0;
     virtual void startMemberGroupHeader(bool) = 0;
     virtual void endMemberGroupHeader() = 0;
     virtual void startMemberGroupDocs() = 0;
@@ -361,8 +360,13 @@ class OutputGenerator : public BaseOutputDocInterface
     virtual void writeStartAnnoItem(const char *type,const char *file,
                                     const char *path,const char *name) = 0;
     virtual void writeEndAnnoItem(const char *name) = 0;
-    virtual void startMemberDescription() = 0;
+    virtual void startMemberDescription(const char *anchor,const char *inheritId) = 0;
     virtual void endMemberDescription() = 0;
+    virtual void startMemberDeclaration() = 0;
+    virtual void endMemberDeclaration(const char *anchor,const char *inheritId) = 0;
+    virtual void writeInheritedSectionTitle(const char *id,const char *ref,
+                                            const char *file,const char *anchor,
+                                            const char *title,const char *name) = 0;
     virtual void startIndent() = 0;
     virtual void endIndent() = 0;
     virtual void writeSynopsis() = 0;
@@ -382,8 +386,10 @@ class OutputGenerator : public BaseOutputDocInterface
     virtual void startQuickIndices() = 0;
     virtual void endQuickIndices() = 0;
     virtual void writeSplitBar(const char *) = 0;
+    virtual void writeNavigationPath(const char *) = 0;
     virtual void writeLogo() = 0;
-    virtual void writeQuickLinks(bool compact,HighlightedItem hli) = 0;
+    virtual void writeQuickLinks(bool compact,HighlightedItem hli,const char *file) = 0;
+    virtual void writeSummaryLink(const char *file,const char *anchor,const char *title,bool first) = 0;
     virtual void startContents() = 0;
     virtual void endContents() = 0;
     virtual void startTextBlock(bool) = 0;
@@ -393,7 +399,7 @@ class OutputGenerator : public BaseOutputDocInterface
     virtual void endMemberDocPrefixItem() = 0;
     virtual void startMemberDocName(bool) = 0;
     virtual void endMemberDocName() = 0;
-    virtual void startParameterType(bool,const char *) = 0;
+    virtual void startParameterType(bool,const char *key) = 0;
     virtual void endParameterType() = 0;
     virtual void startParameterName(bool) = 0;
     virtual void endParameterName(bool,bool,bool) = 0;
@@ -418,6 +424,11 @@ class OutputGenerator : public BaseOutputDocInterface
     virtual void startInlineMemberDoc() = 0;
     virtual void endInlineMemberDoc() = 0;
 
+
+    virtual void startLabels() = 0;
+    virtual void writeLabel(const char *,bool) = 0;
+    virtual void endLabels() = 0;
+
   protected:
     FTextStream t;
     QFile *file;
@@ -431,7 +442,7 @@ class OutputGenerator : public BaseOutputDocInterface
     OutputGenerator &operator=(const OutputGenerator &o);
 };
 
-/*! \brief Interface used for generating documentation.
+/** Interface used for generating documentation.
  *
  *  This abstract class is used by several functions
  *  to generate the output for a specific format. 
@@ -467,7 +478,7 @@ class OutputDocInterface : public BaseOutputDocInterface
     /*! Enables a specific output format (useful for OutputList only) */
     virtual void enable(OutputGenerator::OutputType o) = 0;
 
-    /*! Check whether a specific output format is currenly enabled 
+    /*! Check whether a specific output format is currently enabled 
      *  (useful for OutputList only) 
      */
     virtual bool isEnabled(OutputGenerator::OutputType o) = 0;
