@@ -2,7 +2,7 @@
  *
  * $Id:$
  *
- * Copyright (C) 1997-2005 by Dimitri van Heesch.
+ * Copyright (C) 1997-2012 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -15,46 +15,66 @@
  *
  */
 
+#ifndef PAGEDEF_H
+#define PAGEDEF_H
+
 #include "definition.h"
 #include "sortdict.h"
 
 class PageSDict;
 class OutputList;
 
+/** @brief A model of a page symbol. */
 class PageDef : public Definition
 {
   public:
     PageDef(const char *f,int l,const char *n,const char *d,const char *t);
    ~PageDef();
+
+    // setters
+    void setFileName(const char *name) { m_fileName = name; }
+    void setShowToc(bool b);
+
+    // getters
     DefType definitionType() const { return TypePage; }
     bool isLinkableInProject() const 
     { 
-      return hasDocumentation() && !isReference();
+      return /*hasDocumentation() &&*/ !isReference();
     }
     bool isLinkable() const 
     {
       return isLinkableInProject() || isReference();
     } 
-    void addSubPage(PageDef *def);
-    void writeDocumentation(OutputList &ol);
 
     // functions to get a uniform interface with Definitions
     QCString getOutputFileBase() const;
+    QCString anchor() const { return QCString(); }
     void findSectionsInDocumentation();
     QCString title() const { return m_title; }
     GroupDef *  getGroupDef() const;
-    PageSDict * getSubPages() const { return subPageDict; }
-    void setFileName(const char *name) { m_fileName = name; }
+    PageSDict * getSubPages() const { return m_subPageDict; }
     void addInnerCompound(Definition *d);
     bool visibleInIndex() const;
     bool documentedPage() const;
     bool hasSubPages() const;
+    bool hasParentPage() const;
+    bool showToc() const { return m_showToc; }
+    void setPageScope(Definition *d){ m_pageScope = d; }
+    Definition *getPageScope() const { return m_pageScope; }
+    QCString displayName(bool=TRUE) const { return !m_title.isEmpty() ? m_title : Definition::name(); }
+
+    void writeDocumentation(OutputList &ol);
 
   private:
+    void setNestingLevel(int l);
+    void writePageDocumentation(OutputList &ol);
     QCString m_fileName;
     QCString m_title;
     GroupDef *m_inGroup;
-    PageSDict *subPageDict;                 // list of pages in the group
+    PageSDict *m_subPageDict;                 // list of pages in the group
+    Definition *m_pageScope;
+    int m_nestingLevel;
+    bool m_showToc;
 };
 
 class PageSDict : public SDict<PageDef>
@@ -68,3 +88,5 @@ class PageSDict : public SDict<PageDef>
     }
 };
 
+#endif
+
