@@ -2,7 +2,7 @@
  *
  * $Id: parserintf.h,v 1.15 2001/03/19 19:27:41 root Exp $
  *
- * Copyright (C) 1997-2007 by Dimitri van Heesch.
+ * Copyright (C) 1997-2012 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -24,6 +24,7 @@ class Entry;
 class FileDef;
 class CodeOutputInterface;
 class MemberDef;
+class Definition;
 
 /** \brief Abstract interface for programming language parsers.
  *
@@ -68,6 +69,9 @@ class ParserInterface
      *  @param[in] memberDef Member definition to which the code
      *             is associated (non null in case of an inline fragment 
      *             for a member).
+     *  @param[in] showLineNumbers if set to TRUE and also fileDef is not 0,
+     *             line numbers will be added to the source fragement
+     *  @param[in] searchCtx context under which search data has to be stored.
      */
     virtual void parseCode(CodeOutputInterface &codeOutIntf,
                            const char *scopeName,
@@ -78,7 +82,9 @@ class ParserInterface
                            int startLine=-1,
                            int endLine=-1,
                            bool inlineFragment=FALSE,
-                           MemberDef *memberDef=0
+                           MemberDef *memberDef=0,
+                           bool showLineNumbers=TRUE,
+                           Definition *searchCtx=0
                           ) = 0;
 
     /** Resets the state of the code parser.
@@ -109,22 +115,38 @@ class ParserManager
 {
   public:
     /** Creates the parser manager object. 
-     *  @param defaultParser The default parser that is used when
-     *                       no explicit extension has been registered for
-     *                       a given input file.
      */
-    ParserManager(ParserInterface *defaultParser)
-      : m_defaultParser(defaultParser) { m_parsers.setAutoDelete(TRUE); }
+    ParserManager()
+      : m_defaultParser(0) { m_parsers.setAutoDelete(TRUE); }
 
     /** Registers an additional parser.
-     *  @param[in] extension The file extension that will trigger
-     *             the use of this parser (e.g. ".py", or ".bas").
+     *  @param[in] name      A symbolic name of the parser, i.e. "c",
+     *                       "python", "fortran", "vhdl", ...
      *  @param[in] parser    The parser that is to be used for the
-     *             given extension.
+     *                       given name.
+     *  @param[in] defParser Use this parser as the default parser, used
+     *                       for unregistered file extensions.
      */
-    void registerParser(const char *extension,ParserInterface *parser)
+    void registerParser(const char *name,ParserInterface *parser,bool defParser=FALSE)
     {
-      m_parsers.insert(extension,parser);
+      if (defParser && m_defaultParser==0) m_defaultParser=parser;
+      m_parsers.insert(name,parser);
+    }
+
+    /** Registers a file \a extension with a parser with name \a parserName. 
+     *  Returns TRUE if the extension was successfully registered.
+     */
+    bool registerExtension(const char *extension, const char *parserName)
+    {
+      if (parserName==0 || extension==0) return FALSE;
+      ParserInterface *intf = m_parsers.find(parserName);
+      if (intf==0) return FALSE;
+      if (m_extensions.find(extension)!=0) // extension already exists
+      {
+        m_extensions.remove(extension); // remove it
+      }
+      m_extensions.insert(extension,intf); // add new mapping
+      return TRUE;
     }
 
     /** Gets the interface to the parser associated with given \a extension.
@@ -134,12 +156,18 @@ class ParserManager
     ParserInterface *getParser(const char *extension)
     {
       if (extension==0) return m_defaultParser;
-      ParserInterface *intf = m_parsers.find(extension);
+      QCString ext = QCString(extension).lower();
+      ParserInterface *intf = m_extensions.find(ext);
+      if (intf==0 && ext.length()>4)
+      {
+        intf = m_extensions.find(ext.left(4));
+      }
       return intf ? intf : m_defaultParser;
     }
 
   private:
     QDict<ParserInterface> m_parsers;
+    QDict<ParserInterface> m_extensions;
     ParserInterface *m_defaultParser;
 };
 
