@@ -2,6 +2,7 @@
 #include <ctype.h>
 #if defined(_WIN32) && !defined(__CYGWIN__)
 #undef UNICODE
+#define _WIN32_DCOM
 #include <windows.h>
 #else
 #include <unistd.h>
@@ -26,12 +27,11 @@ extern char **environ;
 #endif
 //#include "doxygen.h"
 
-static double  sysElapsedTime;
+static double  g_sysElapsedTime;
+static QTime   g_time;
 
 int  portable_system(const char *command,const char *args,bool commandHasConsole)
 {
-  QTime time;
-  time.start();
 
   if (command==0) return 1;
 
@@ -49,7 +49,7 @@ int  portable_system(const char *command,const char *args,bool commandHasConsole
 #endif
 
 #if !defined(_WIN32) || defined(__CYGWIN__)
-  commandHasConsole=commandHasConsole;
+  (void)commandHasConsole;
   /*! taken from the system() manpage on my Linux box */
   int pid,status=0;
 
@@ -79,7 +79,6 @@ int  portable_system(const char *command,const char *args,bool commandHasConsole
       }
     }
   }
-  sysElapsedTime+=((double)time.elapsed())/1000.0;
   return status;
 
 #else  // Other Unices just use fork
@@ -104,7 +103,6 @@ int  portable_system(const char *command,const char *args,bool commandHasConsole
     }
     else
     {
-      sysElapsedTime+=((double)time.elapsed())/1000.0;
       if (WIFEXITED(status))
       {
         return WEXITSTATUS(status);
@@ -124,6 +122,14 @@ int  portable_system(const char *command,const char *args,bool commandHasConsole
   }
   else
   {
+    // Because ShellExecuteEx can delegate execution to Shell extensions 
+    // (data sources, context menu handlers, verb implementations) that 
+    // are activated using Component Object Model (COM), COM should be 
+    // initialized before ShellExecuteEx is called. Some Shell extensions 
+    // require the COM single-threaded apartment (STA) type. 
+    // For that case COM is initialized as follows
+    CoInitializeEx(NULL, COINIT_APARTMENTTHREADED | COINIT_DISABLE_OLE1DDE);
+
     // gswin32 is a GUI api which will pop up a window and run
     // asynchronously. To prevent both, we use ShellExecuteEx and
     // WaitForSingleObject (thanks to Robert Golias for the code)
@@ -158,7 +164,6 @@ int  portable_system(const char *command,const char *args,bool commandHasConsole
       CloseHandle(sInfo.hProcess);
     }
   }
-  sysElapsedTime+=((double)time.elapsed())/1000.0;
   return 0;
 #endif
 
@@ -179,6 +184,7 @@ static char **last_environ;
 
 void portable_setenv(const char *name,const char *value)
 {
+    if (value==0) value="";
 #if defined(_WIN32) && !defined(__CYGWIN__)
     SetEnvironmentVariable(name,value);
 #else
@@ -313,6 +319,17 @@ portable_off_t portable_ftell(FILE *f)
 #endif
 }
 
+FILE *portable_fopen(const char *fileName,const char *mode)
+{
+#if defined(_WIN32) && !defined(__CYGWIN__)
+  QString fn(fileName);
+  QString m(mode);
+  return _wfopen((wchar_t*)fn.ucs2(),(wchar_t*)m.ucs2());
+#else
+  return fopen(fileName,mode);
+#endif
+}
+
 char  portable_pathSeparator()
 {
 #if defined(_WIN32) && !defined(__CYGWIN__)
@@ -368,3 +385,26 @@ int portable_pclose(FILE *stream)
   return pclose(stream);
 }
 
+void portable_sysTimerStart()
+{
+  g_time.start();
+}
+
+void portable_sysTimerStop()
+{
+  g_sysElapsedTime+=((double)g_time.elapsed())/1000.0;
+}
+
+double portable_getSysElapsedTime()
+{
+  return g_sysElapsedTime;
+}
+
+void portable_sleep(int ms)
+{
+#if defined(_WIN32) && !defined(__CYGWIN__)
+  Sleep(ms);
+#else
+  usleep(1000*ms);
+#endif
+}
