@@ -2,7 +2,7 @@
  *
  * $Id: pycode.h,v 1.9 2001/03/19 19:27:39 root Exp $
  *
- * Copyright (C) 1997-2005 by Dimitri van Heesch.
+ * Copyright (C) 1997-2012 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -31,11 +31,12 @@
 class CodeOutputInterface;
 class FileDef;
 class MemberDef;
+class Definition;
 
 extern void parsePythonCode(CodeOutputInterface &,const char *,const QCString &, 
-             bool ,const char *,FileDef *fd=0,
-	     int startLine=-1,int endLine=-1,bool inlineFragment=FALSE,
-             MemberDef *memberDef=0);
+             bool ,const char *,FileDef *fd,
+	     int startLine,int endLine,bool inlineFragment,
+             MemberDef *memberDef,bool showLineNumbers,Definition *searchCtx);
 extern void resetPythonCodeParserState();
 
 #endif
