@@ -20,6 +20,7 @@
 #include "config.h"
 #include "memberdef.h"
 #include "groupdef.h"
+#include "doxygen.h"
 #include "filedef.h"
 
 #include <qstringlist.h>
@@ -27,8 +28,19 @@
 
 static QCString makeFileName(const char * withoutExtension)
 {
-  if (!withoutExtension) return QCString();
-  return QCString(withoutExtension)+".html";
+  QCString result=withoutExtension;
+  if (!result.isEmpty())
+  {
+    if (result.at(0)=='!') // relative URL -> strip marker
+    {
+      result=result.mid(1);
+    }
+    else // add specified HTML extension
+    {
+      result+=Doxygen::htmlFileExtension;
+    }
+  }
+  return result;
 }
 
 static QCString makeRef(const char * withoutExtension, const char * anchor)
@@ -89,7 +101,7 @@ void Qhp::initialize()
     QStringList customFilterAttributes = QStringList::split(QChar(' '), Config_getString("QHP_CUST_FILTER_ATTRS"));
     for (int i = 0; i < (int)customFilterAttributes.count(); i++)
     {
-      m_doc.openCloseContent("filterAttribute", customFilterAttributes[i]);
+      m_doc.openCloseContent("filterAttribute", customFilterAttributes[i].utf8());
     }
     m_doc.close("customFilter");
   }
@@ -105,7 +117,7 @@ void Qhp::initialize()
   }
   for (int i = 0; i < (int)sectionFilterAttributes.count(); i++)
   {
-    m_doc.openCloseContent("filterAttribute", sectionFilterAttributes[i]);
+    m_doc.openCloseContent("filterAttribute", sectionFilterAttributes[i].utf8());
   }
 
   m_toc.open("toc");
@@ -114,10 +126,11 @@ void Qhp::initialize()
   QCString fullProjectname = getFullProjectName();
   const char * const attributes[] =
   { "title", fullProjectname,
-    "ref",   "index.html",
+    "ref",   QCString("index")+Doxygen::htmlFileExtension,
     NULL
   };
   m_toc.open("section", attributes);
+  m_prevSectionTitle = getFullProjectName();
   m_prevSectionLevel = 1;
   m_sectionLevel = 1;
 
@@ -160,10 +173,12 @@ void Qhp::finalize()
 void Qhp::incContentsDepth()
 {
   m_sectionLevel++;
+  //printf("Qhp::incContentsDepth() %d->%d\n",m_sectionLevel-1,m_sectionLevel);
 }
 
 void Qhp::decContentsDepth()
 {
+  //printf("Qhp::decContentsDepth() %d->%d\n",m_sectionLevel,m_sectionLevel-1);
   if (m_sectionLevel <= 0)
   {
     return;
@@ -173,13 +188,20 @@ void Qhp::decContentsDepth()
 
 void Qhp::addContentsItem(bool /*isDir*/, const char * name, 
                           const char * /*ref*/, const char * file, 
-                          const char * /*anchor*/)
+                          const char * /*anchor*/,bool /* separateIndex */,
+                          bool /* addToNavIndex */,
+                          Definition * /*def*/)
 {
+  //printf("Qhp::addContentsItem(%s) %d\n",name,m_sectionLevel);
   // Backup difference before modification
+
+  QCString f = file;
+  if (!f.isEmpty() && f.at(0)=='^') return; // absolute URL not supported
+
   int diff = m_prevSectionLevel - m_sectionLevel;
 
   handlePrevSection();
-  setPrevSection(name, file, m_sectionLevel);
+  setPrevSection(name, f, m_sectionLevel);
 
   // Close sections as needed
   for (; diff > 0; diff--)
@@ -280,11 +302,11 @@ void Qhp::handlePrevSection()
 
   if (m_prevSectionTitle.isNull())
   {
-    return;
+    m_prevSectionTitle=" "; // should not happen...
   }
 
   // We skip "Main Page" as our extra root is pointing to that
-  if (!((m_prevSectionLevel==1) && (m_prevSectionTitle=="Main Page")))
+  if (!((m_prevSectionLevel==1) && (m_prevSectionTitle==getFullProjectName())))
   {
     QCString finalRef = makeFileName(m_prevSectionRef);
 
