@@ -2,7 +2,7 @@
  *
  * $Id: rtfgen.h,v 1.13 2001/03/19 19:27:41 root Exp $
  *
- * Copyright (C) 1997-2011 by Parker Waechter & Dimitri van Heesch.
+ * Copyright (C) 1997-2012 by Parker Waechter & Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -22,6 +22,7 @@
 
 class QFile;
 
+/** Generator for RTF output. */
 class RTFGenerator : public OutputGenerator
 {
   public:
@@ -40,10 +41,11 @@ class RTFGenerator : public OutputGenerator
     bool isEnabled(OutputType o) { return (o==RTF && active); } 
     OutputGenerator *get(OutputType o) { return (o==RTF) ? this : 0; }
 
-    void printDoc(DocNode *,const char *);
+    void writeDoc(DocNode *,Definition *,MemberDef *);
 
     void startFile(const char *name,const char *manName,const char *title);
-    void writeFooter() {}
+    void writeSearchInfo() {}
+    void writeFooter(const char *) {}
     void endFile();
     void clearBuffer();
     //void postProcess(QByteArray &);
@@ -110,10 +112,10 @@ class RTFGenerator : public OutputGenerator
     void endInlineHeader();
     void startAnonTypeScope(int) {}
     void endAnonTypeScope(int) {}
-    void startMemberItem(int);
+    void startMemberItem(const char *,int,const char *);
     void endMemberItem();
     void startMemberTemplateParams() {}
-    void endMemberTemplateParams() {}
+    void endMemberTemplateParams(const char *,const char *) {}
     void insertMemberAlign(bool) {}
 
     void writeRuler() { rtfwriteRuler_thin(); }
@@ -122,7 +124,7 @@ class RTFGenerator : public OutputGenerator
     void startCodeFragment();
     void endCodeFragment();
     void writeLineNumber(const char *,const char *,const char *,int l) { t << l << " "; }
-    void startCodeLine() { col=0; }
+    void startCodeLine(bool) { col=0; }
     void endCodeLine() { lineBreak(); }
     void startEmphasis() { t << "{\\i ";  }
     void endEmphasis()   { t << "}"; }
@@ -153,8 +155,12 @@ class RTFGenerator : public OutputGenerator
     void startSmall()       { t << "{\\sub "; }
     void endSmall()         { t << "}"; }
 
-    void startMemberDescription();
+    void startMemberDescription(const char *,const char *);
     void endMemberDescription();
+    void startMemberDeclaration() {} 
+    void endMemberDeclaration(const char *,const char *) {}
+    void writeInheritedSectionTitle(const char *,const char *,const char *,
+                      const char *,const char *,const char *) {}
     void startDescList(SectionTypes);
     void startSimpleSect(SectionTypes,const char *,const char *,const char *);
     void endSimpleSect();
@@ -176,8 +182,10 @@ class RTFGenerator : public OutputGenerator
     void startQuickIndices() {}
     void endQuickIndices() {}
     void writeSplitBar(const char *) {}
+    void writeNavigationPath(const char *) {}
     void writeLogo() {}
-    void writeQuickLinks(bool,HighlightedItem) {}
+    void writeQuickLinks(bool,HighlightedItem,const char *) {}
+    void writeSummaryLink(const char *,const char *,const char *,bool) {}
     void startContents() {}
     void endContents() {}
     void writeNonBreakableSpace(int);
@@ -241,6 +249,10 @@ class RTFGenerator : public OutputGenerator
     void startInlineMemberDoc();
     void endInlineMemberDoc();
 
+    void startLabels();
+    void writeLabel(const char *l,bool isLast);
+    void endLabels();
+
     void startFontClass(const char *) {}
     void endFontClass() {}
 
