@@ -2,7 +2,7 @@
  *
  * 
  *
- * Copyright (C) 1997-2005 by Dimitri van Heesch.
+ * Copyright (C) 1997-2012 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -23,14 +23,15 @@
 #include "util.h"
 #include "config.h"
 
+/** Abstract base class for all translatable text fragments. */
 class Translator
 {
   private:
     /* Tables for encoding conversions. */
     static const char Win1250ToISO88592Tab[];
     static const char ISO88592ToWin1250Tab[];
-    static Q_UINT16 Koi8RToWindows1251Tab[128];
-    static Q_UINT16 Windows1251ToKoi8RTab[128];
+    static const unsigned char Koi8RToWindows1251Tab[128];
+    static const unsigned char Windows1251ToKoi8RTab[128];
 
   protected:
     /*! Returns the string converted from windows-1250 to iso-8859-2. */
@@ -55,6 +56,7 @@ class Translator
      *  It is implemented by the adapter classes.
      */
     virtual QCString updateNeededMessage() { return ""; }
+    virtual ~Translator() {}
    
     // Please, have a look at comments inside the translator_en.h file
     // to learn the meaning of the following methods.  The translator_en.h
@@ -92,7 +94,7 @@ class Translator
     virtual QCString trClassHierarchy() = 0;
     virtual QCString trCompoundList() = 0;
     virtual QCString trFileList() = 0;
-    virtual QCString trHeaderFiles() = 0;
+    //virtual QCString trHeaderFiles() = 0;
     virtual QCString trCompoundMembers() = 0;
     virtual QCString trFileMembers() = 0;
     virtual QCString trRelatedPages() = 0;
@@ -103,11 +105,11 @@ class Translator
     virtual QCString trCompoundListDescription() = 0;
     virtual QCString trCompoundMembersDescription(bool extractAll) = 0;
     virtual QCString trFileMembersDescription(bool extractAll) = 0;
-    virtual QCString trHeaderFilesDescription() = 0;
+    //virtual QCString trHeaderFilesDescription() = 0;
     virtual QCString trExamplesDescription() = 0;
     virtual QCString trRelatedPagesDescription() = 0;
     virtual QCString trModulesDescription() = 0;
-    virtual QCString trNoDescriptionAvailable() = 0;
+    //virtual QCString trNoDescriptionAvailable() = 0;
     
     // index titles (the project name is prepended for these) 
 
@@ -133,7 +135,6 @@ class Translator
     virtual QCString trFunctionPrototypeDocumentation() = 0;
     virtual QCString trTypedefDocumentation() = 0;
     virtual QCString trEnumerationTypeDocumentation() = 0;
-    virtual QCString trEnumerationValueDocumentation() = 0;
     virtual QCString trFunctionDocumentation() = 0;
     virtual QCString trVariableDocumentation() = 0;
     virtual QCString trCompounds() = 0;
@@ -141,9 +142,9 @@ class Translator
     virtual QCString trWrittenBy() = 0;
     virtual QCString trClassDiagram(const char *clName) = 0;
     virtual QCString trForInternalUseOnly() = 0;
-    virtual QCString trReimplementedForInternalReasons() = 0;
+    //virtual QCString trReimplementedForInternalReasons() = 0;
     virtual QCString trWarning() = 0;
-    virtual QCString trBugsAndLimitations() = 0;
+    //virtual QCString trBugsAndLimitations() = 0;
     virtual QCString trVersion() = 0;
     virtual QCString trDate() = 0;
     virtual QCString trReturns() = 0;
@@ -223,7 +224,7 @@ class Translator
 // new since 0.49-991003
 //////////////////////////////////////////////////////////////////////////
 
-    virtual QCString trSources() = 0;
+    //virtual QCString trSources() = 0;
     virtual QCString trDefinedAtLineInSourceFile() = 0;
     virtual QCString trDefinedInSourceFile() = 0;
 
@@ -317,13 +318,13 @@ class Translator
 // new since 1.2.4
 //////////////////////////////////////////////////////////////////////////
 
-    virtual QCString trInterfaces() = 0;
+    //virtual QCString trInterfaces() = 0;
     virtual QCString trClasses() = 0;
     virtual QCString trPackage(const char *name) = 0;
     virtual QCString trPackageList() = 0;
     virtual QCString trPackageListDescription() = 0;
     virtual QCString trPackages() = 0;
-    virtual QCString trPackageDocumentation() = 0;
+    //virtual QCString trPackageDocumentation() = 0;
     virtual QCString trDefineValue() = 0;
     
 //////////////////////////////////////////////////////////////////////////
@@ -343,13 +344,13 @@ class Translator
    
     // Translation of the word
 
-   virtual QCString trClass(bool first_capital, bool singular) = 0;
+    virtual QCString trClass(bool first_capital, bool singular) = 0;
     virtual QCString trFile(bool first_capital, bool singular) = 0;
     virtual QCString trNamespace(bool first_capital, bool singular) = 0;
     virtual QCString trGroup(bool first_capital, bool singular) = 0;
     virtual QCString trPage(bool first_capital, bool singular) = 0;
     virtual QCString trMember(bool first_capital, bool singular) = 0;
-    virtual QCString trField(bool first_capital, bool singular) = 0;
+    //virtual QCString trField(bool first_capital, bool singular) = 0;
     virtual QCString trGlobal(bool first_capital, bool singular) = 0;
 
 //////////////////////////////////////////////////////////////////////////
@@ -433,6 +434,102 @@ class Translator
     virtual QCString trDirReference(const char *dirName) = 0;
     virtual QCString trDir(bool first_capital, bool singular) = 0;
 
+//////////////////////////////////////////////////////////////////////////
+// new since 1.4.1
+//////////////////////////////////////////////////////////////////////////
+
+    virtual QCString trOverloadText() = 0;
+
+//////////////////////////////////////////////////////////////////////////
+// new since 1.4.6
+//////////////////////////////////////////////////////////////////////////
+    
+    virtual QCString trCallerGraph() = 0;
+    virtual QCString trEnumerationValueDocumentation() = 0;
+
+//////////////////////////////////////////////////////////////////////////
+// new since 1.5.4
+//////////////////////////////////////////////////////////////////////////
+    
+    virtual QCString trMemberFunctionDocumentationFortran() = 0;
+    virtual QCString trCompoundListFortran() = 0;
+    virtual QCString trCompoundMembersFortran() = 0;
+    virtual QCString trCompoundListDescriptionFortran() = 0;
+    virtual QCString trCompoundMembersDescriptionFortran(bool extractAll) = 0;
+    virtual QCString trCompoundIndexFortran() = 0;
+    virtual QCString trTypeDocumentation() = 0;
+    virtual QCString trSubprograms() = 0;
+    virtual QCString trSubprogramDocumentation() = 0;
+    virtual QCString trDataTypes() = 0;
+    virtual QCString trModulesList() = 0;
+    virtual QCString trModulesListDescription(bool extractAll) = 0;
+    virtual QCString trCompoundReferenceFortran(const char *clName,
+                                 ClassDef::CompoundType compType,
+                                 bool isTemplate) = 0;
+    virtual QCString trModuleReference(const char *namespaceName) = 0;
+    virtual QCString trModulesMembers() = 0;
+    virtual QCString trModulesMemberDescription(bool extractAll) = 0;
+    virtual QCString trModulesIndex() = 0;
+    virtual QCString trModule(bool first_capital, bool singular) = 0;
+    virtual QCString trGeneratedFromFilesFortran(ClassDef::CompoundType compType,
+                     bool single) = 0;
+    virtual QCString trType(bool first_capital, bool singular) = 0;
+    virtual QCString trSubprogram(bool first_capital, bool singular) = 0;
+    virtual QCString trTypeConstraints() = 0;
+
+//////////////////////////////////////////////////////////////////////////
+// new since 1.6.0
+//////////////////////////////////////////////////////////////////////////
+
+    virtual QCString trDirRelation(const char *name) = 0;
+    virtual QCString trLoading() = 0;
+    virtual QCString trGlobalNamespace() = 0;
+    virtual QCString trSearching() = 0;
+    virtual QCString trNoMatches() = 0;
+
+//////////////////////////////////////////////////////////////////////////
+// new since 1.6.3
+//////////////////////////////////////////////////////////////////////////
+
+    virtual QCString trDirDependency(const char *name) = 0;
+    virtual QCString trFileIn(const char *name) = 0;
+    virtual QCString trIncludesFileIn(const char *name) = 0;
+    virtual QCString trDateTime(int year,int month,int day,int dayOfWeek,
+                                int hour,int minutes,int seconds,
+                                bool includeTime) = 0;
+                       
+//////////////////////////////////////////////////////////////////////////
+// new since 1.7.5
+//////////////////////////////////////////////////////////////////////////
+
+    virtual QCString trCiteReferences() = 0;
+    virtual QCString trCopyright() = 0;
+    virtual QCString trDirDepGraph(const char *name) = 0;
+
+//////////////////////////////////////////////////////////////////////////
+// new since 1.8.0
+//////////////////////////////////////////////////////////////////////////
+
+    virtual QCString trDetailLevel() = 0;
+    virtual QCString trTemplateParameters() = 0;
+    virtual QCString trAndMore(const QCString &number) = 0;
+    virtual QCString trEnumGeneratedFromFiles(bool single) = 0;
+    virtual QCString trEnumReference(const char *name) = 0;
+    virtual QCString trInheritedFrom(const char *members,const char *what) = 0;
+    virtual QCString trAdditionalInheritedMembers() = 0;
+
+//////////////////////////////////////////////////////////////////////////
+// new since 1.8.2
+//////////////////////////////////////////////////////////////////////////
+
+    virtual QCString trPanelSynchronisationTooltip(bool enable) = 0;
+    virtual QCString trProvidedByCategory() = 0;
+    virtual QCString trExtendsClass() = 0;
+    virtual QCString trClassMethods() = 0;
+    virtual QCString trInstanceMethods() = 0;
+    virtual QCString trMethodDocumentation() = 0;
+    virtual QCString trDesignOverview() = 0;
+
 };
 
 #endif
