@@ -2,7 +2,7 @@
  *
  * 
  *
- * Copyright (C) 1997-2008 by Dimitri van Heesch.
+ * Copyright (C) 1997-2012 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -27,7 +27,7 @@
 //   2001/03/23 Jens Seidel (jensseidel@users.sourceforge.net)
 //    - fixed typos
 //    - changed trPageDocumentation() "Seitenbeschreibung" to
-//      "Zustzliche Informationen"
+//      "Zusätzliche Informationen"
 //    - removed old trGeneratedFrom()
 //    - changed "/*!" to "/*" (documentation is inherited from translator_en.h
 //      (INHERIT_DOCS = YES), there's no need to make changes twice)
@@ -70,7 +70,7 @@
 //   2002/11/25 Jens Seidel (jensseidel@users.sourceforge.net)
 //    - sync with english version 1.3
 //    - TranslatorGerman doesn't inherit from TranslatorEnglish anymore,
-//      so I changed "/* " back to "/*! " as in the english file
+//      so I changed "/* " back to "/*! " as in the English file
 //    - use ngerman instead of german package in LaTeX
 //    - changed "Datenelemente" to "Methoden" in
 //      tr{,Static}{Public,Protected,Private}Members
@@ -117,13 +117,23 @@
 //   2009/01/09 Jens Seidel (jensseidel@users.sourceforge.net)
 //    - Resynced the trLegendDocs() method
 //
+//   2010/03/17 Jens Seidel (jensseidel@users.sourceforge.net)
+//    - Updated for "new since 1.6.0" version
+//    - Resynced the trLegendDocs() method (added closing HTML tags)
+//    - Changed file encoding to utf-8, adapted idLanguageCharset()
+//    - New German orthography (ngerman) is now default in LaTeX
+//
+//   2012/04/11 Peter Grotrian
+//    - Updated for "new since 1.8.0" version
+//    - Some small corrections
+//
 //   Todo:
 //    - see FIXME
 
 #ifndef TRANSLATOR_DE_H
 #define TRANSLATOR_DE_H
 
-class TranslatorGerman : public Translator
+class TranslatorGerman : public TranslatorAdapter_1_8_2
 {
   public:
 
@@ -143,17 +153,13 @@ class TranslatorGerman : public Translator
      */
     virtual QCString latexLanguageSupportCommand()
     {
-      //QCString result="\\usepackage{ngerman}\n";
-      //result+="\\usepackage[latin1]{inputenc}\n";
-      //result+="%\\usepackage[latin1]{inputenc}% Kodierung (cp850,latin1,ansinew)\n";
-      QCString result="\\usepackage[german]{babel}\n";
-      return result;
+      return "\\usepackage[ngerman]{babel}\n";
     }
 
     /*! return the language charset. This will be used for the HTML output */
     virtual QCString idLanguageCharset()
     {
-      return "iso-8859-1";
+      return "utf-8";
     }
 
     // --- Language translation methods -------------------
@@ -168,7 +174,7 @@ class TranslatorGerman : public Translator
 
     /*! header that is put before the detailed description of files, classes and namespaces. */
     virtual QCString trDetailedDescription()
-    { return "Ausfhrliche Beschreibung"; }
+    { return "Ausführliche Beschreibung"; }
 
     /*! header that is put before the list of typedefs. */
     virtual QCString trMemberTypedefDocumentation()
@@ -176,7 +182,7 @@ class TranslatorGerman : public Translator
     
     /*! header that is put before the list of enumerations. */
     virtual QCString trMemberEnumerationDocumentation()
-    { return "Dokumentation der Aufzhlungstypen"; }
+    { return "Dokumentation der Aufzählungstypen"; }
     
     /*! header that is put before the list of member functions. */
     virtual QCString trMemberFunctionDocumentation()
@@ -202,29 +208,29 @@ class TranslatorGerman : public Translator
 
     /*! this is the first part of a sentence that is followed by a class name */
     virtual QCString trThisIsTheListOfAllMembers()
-    { return "Vollstndige Aufstellung aller Elemente fr "; }
+    { return "Vollständige Aufstellung aller Elemente für "; }
 
     /*! this is the remainder of the sentence after the class name */
     virtual QCString trIncludingInheritedMembers()
-    { return " einschlielich aller geerbten Elemente."; }
+    { return " einschließlich aller geerbten Elemente."; }
     
     /*! this is put at the author sections at the bottom of man pages.
      *  parameter s is name of the project name.
      */
     virtual QCString trGeneratedAutomatically(const char *s)
     { QCString result="Automatisch erzeugt von Doxygen";
-      if (s) result+=(QCString)" fr "+s;
+      if (s) result+=(QCString)" für "+s;
       result+=" aus dem Quellcode.";
       return result;
     }
 
     /*! put after an enum name in the list of all members */
     virtual QCString trEnumName()
-    { return "enum Bezeichner"; }
+    { return "enum-Bezeichner"; }
     
     /*! put after an enum value in the list of all members */
     virtual QCString trEnumValue()
-    { return "enum Wert"; }
+    { return "enum-Wert"; }
     
     /*! put after an undocumented member in the list of all members */
     virtual QCString trDefinedIn()
@@ -279,7 +285,7 @@ class TranslatorGerman : public Translator
 
     /*! This is put above each page as a link to all related pages. */
     virtual QCString trRelatedPages()
-    { return "Zustzliche Informationen"; }
+    { return "Zusätzliche Informationen"; }
 
     /*! This is put above each page as a link to all examples. */
     virtual QCString trExamples()
@@ -291,14 +297,14 @@ class TranslatorGerman : public Translator
 
     /*! This is an introduction to the class hierarchy. */
     virtual QCString trClassHierarchyDescription()
-    { return "Die Liste der Ableitungen ist -mit Einschrnkungen- "
+    { return "Die Liste der Ableitungen ist -mit Einschränkungen- "
              "alphabetisch sortiert:";
     }
 
     /*! This is an introduction to the list with all files. */
     virtual QCString trFileListDescription(bool extractAll)
     {
-      QCString result="Hier folgt die Aufzhlung aller ";
+      QCString result="Hier folgt die Aufzählung aller ";
       if (!extractAll) result+="dokumentierten ";
       result+="Dateien mit einer Kurzbeschreibung:";
       return result;
@@ -309,12 +315,12 @@ class TranslatorGerman : public Translator
     { 
       if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
       {
-	return "Hier folgt die Aufzhlung aller Datenstrukturen "
+	return "Hier folgt die Aufzählung aller Datenstrukturen "
 	       "mit einer Kurzbeschreibung:";
       }
       else
       {
-	return "Hier folgt die Aufzhlung aller Klassen, Strukturen, "
+	return "Hier folgt die Aufzählung aller Klassen, Strukturen, "
 	       "Varianten und Schnittstellen mit einer Kurzbeschreibung:";
       }
     }
@@ -322,7 +328,7 @@ class TranslatorGerman : public Translator
     /*! This is an introduction to the page with all class members. */
     virtual QCString trCompoundMembersDescription(bool extractAll)
     {
-      QCString result="Hier folgt die Aufzhlung aller ";
+      QCString result="Hier folgt die Aufzählung aller ";
       if (!extractAll)
       {
         result+="dokumentierten ";
@@ -351,11 +357,11 @@ class TranslatorGerman : public Translator
       {
         if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
         {
-          result+="die zugehrigen Elemente:";
+          result+="die zugehörigen Elemente:";
         }
         else
         {
-          result+="die zugehrigen Klassen:";
+          result+="die zugehörigen Klassen:";
         }
       }
       return result;
@@ -364,11 +370,11 @@ class TranslatorGerman : public Translator
     /*! This is an introduction to the page with all file members. */
     virtual QCString trFileMembersDescription(bool extractAll)
     {
-      QCString result="Hier folgt die Aufzhlung aller ";
+      QCString result="Hier folgt die Aufzählung aller ";
       if (!extractAll) result+="dokumentierten ";
       if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
       {
-        result+="Funktionen, Variablen, Makros, Aufzhlungen und Typdefinitionen";
+        result+="Funktionen, Variablen, Makros, Aufzählungen und Typdefinitionen";
       }
       else
       {
@@ -378,7 +384,7 @@ class TranslatorGerman : public Translator
       if (extractAll) 
         result+="die Dokumentation zu jedem Element:";
       else
-        result+="die zugehrigen Dateien:";
+        result+="die zugehörigen Dateien:";
       return result;
     }
 
@@ -388,11 +394,11 @@ class TranslatorGerman : public Translator
 
     /*! This is an introduction to the page with the list of related pages */
     virtual QCString trRelatedPagesDescription()
-    { return "Hier folgt eine Liste mit zusammengehrigen Themengebieten:"; }
+    { return "Hier folgt eine Liste mit zusammengehörigen Themengebieten:"; }
 
     /*! This is an introduction to the page with the list of class/file groups */
     virtual QCString trModulesDescription()
-    { return "Hier folgt die Aufzhlung aller Module:"; }
+    { return "Hier folgt die Aufzählung aller Module:"; }
 
     // index titles (the project name is prepended for these) 
 
@@ -466,7 +472,7 @@ class TranslatorGerman : public Translator
      *  the documentation of all related pages.
      */
     virtual QCString trPageDocumentation()
-    { return "Zustzliche Informationen"; }
+    { return "Zusätzliche Informationen"; }
 
     /*! This is used in LaTeX as the title of the document */
     virtual QCString trReferenceManual()
@@ -494,7 +500,7 @@ class TranslatorGerman : public Translator
      *  list of enumerations
      */
     virtual QCString trEnumerations()
-    { return "Aufzhlungen"; }
+    { return "Aufzählungen"; }
 
     /*! This is used in the documentation of a file as a header before the 
      *  list of (global) functions
@@ -512,7 +518,7 @@ class TranslatorGerman : public Translator
      *  list of (global) variables
      */
     virtual QCString trEnumerationValues()
-    { return "Aufzhlungswerte"; }
+    { return "Aufzählungswerte"; }
     
     /*! This is used in the documentation of a file before the list of
      *  documentation blocks for defines
@@ -536,7 +542,7 @@ class TranslatorGerman : public Translator
      *  of documentation blocks for enumeration types
      */
     virtual QCString trEnumerationTypeDocumentation()
-    { return "Dokumentation der Aufzhlungstypen"; }
+    { return "Dokumentation der Aufzählungstypen"; }
 
     /*! This is used in the documentation of a file/namespace before the list 
      *  of documentation blocks for functions
@@ -571,7 +577,7 @@ class TranslatorGerman : public Translator
     virtual QCString trGeneratedAt(const char *date,const char *projName)
     {
       QCString result=(QCString)"Erzeugt am "+date;
-      if (projName) result+=(QCString)" fr "+projName;
+      if (projName) result+=(QCString)" für "+projName;
       result+=(QCString)" von";
       return result;
     }
@@ -586,12 +592,12 @@ class TranslatorGerman : public Translator
     /*! this text is put before a class diagram */
     virtual QCString trClassDiagram(const char *clName)
     {
-      return (QCString)"Klassendiagramm fr "+clName+":";
+      return (QCString)"Klassendiagramm für "+clName+":";
     }
     
     /*! this text is generated when the \\internal command is used. */
     virtual QCString trForInternalUseOnly()
-    { return "Nur fr den internen Gebrauch."; }
+    { return "Nur für den internen Gebrauch."; }
 
     /*! this text is generated when the \\warning command is used. */
     virtual QCString trWarning()
@@ -607,7 +613,7 @@ class TranslatorGerman : public Translator
 
     /*! this text is generated when the \\return command is used. */
     virtual QCString trReturns()
-    { return "Rckgabe"; }
+    { return "Rückgabe"; }
 
     /*! this text is generated when the \\sa command is used. */
     virtual QCString trSeeAlso()
@@ -700,25 +706,25 @@ class TranslatorGerman : public Translator
     }
     
     virtual QCString trPublicMembers()
-    { return "ffentliche Methoden"; }
+    { return "Öffentliche Methoden"; }
 
     virtual QCString trPublicSlots()
-    { return "ffentliche Slots"; }
+    { return "Öffentliche Slots"; }
 
     virtual QCString trSignals()
     { return "Signale"; }
 
     virtual QCString trStaticPublicMembers()
-    { return "ffentliche, statische Methoden"; }
+    { return "Öffentliche, statische Methoden"; }
 
     virtual QCString trProtectedMembers()
-    { return "Geschtzte Methoden"; }
+    { return "Geschützte Methoden"; }
 
     virtual QCString trProtectedSlots()
-    { return "Geschtzte Slots"; }
+    { return "Geschützte Slots"; }
 
     virtual QCString trStaticProtectedMembers()
-    { return "Geschtzte, statische Methoden"; }
+    { return "Geschützte, statische Methoden"; }
 
     virtual QCString trPrivateMembers()
     { return "Private Methoden"; }
@@ -767,7 +773,7 @@ class TranslatorGerman : public Translator
      */
     virtual QCString trInheritedByList(int numEntries)
     {
-      return "Basisklasse fr "+trWriteList(numEntries)+".";
+      return "Basisklasse für "+trWriteList(numEntries)+".";
     }
 
     /*! used in member documentation blocks to produce a list of 
@@ -793,13 +799,13 @@ class TranslatorGerman : public Translator
     /*! This is an introduction to the page with all namespace members */
     virtual QCString trNamespaceMemberDescription(bool extractAll)
     {
-      QCString result="Hier folgt die Aufzhlung aller ";
+      QCString result="Hier folgt die Aufzählung aller ";
       if (!extractAll) result+="dokumentierten ";
       result+="Namensbereichselemente mit Verweisen auf ";
       if (extractAll)
-        result+="die Namensbereichsdokumentation fr jedes Element:";
+        result+="die Namensbereichsdokumentation für jedes Element:";
       else
-        result+="die zugehrigen Dateien:";
+        result+="die zugehörigen Dateien:";
       return result;
     }
 
@@ -835,7 +841,7 @@ class TranslatorGerman : public Translator
     virtual QCString trGeneratedFromFiles(ClassDef::CompoundType compType,
         bool single)
     { // single is true implies a single file
-      QCString result=(QCString)"Die Dokumentation fr diese";
+      QCString result=(QCString)"Die Dokumentation für diese";
       switch(compType)
       {
         case ClassDef::Class:      result+=" Klasse"; break;
@@ -863,7 +869,7 @@ class TranslatorGerman : public Translator
 
     /*! This is used as the heading text for the retval command. */
     virtual QCString trReturnValues()
-    { return "Rckgabewerte"; }
+    { return "Rückgabewerte"; }
 
     /*! This is in the (quick) index as a link to the main page (index.html)
      */
@@ -906,13 +912,13 @@ class TranslatorGerman : public Translator
     /*! this text is put before a collaboration diagram */
     virtual QCString trCollaborationDiagram(const char *clName)
     {
-      return (QCString)"Zusammengehrigkeiten von "+clName+":";
+      return (QCString)"Zusammengehörigkeiten von "+clName+":";
     }
 
     /*! this text is put before an include dependency graph */
     virtual QCString trInclDepGraph(const char *fName)
     {
-      return (QCString)"Include-Abhngigkeitsdiagramm fr "+fName+":";
+      return (QCString)"Include-Abhängigkeitsdiagramm für "+fName+":";
     }
 
     /*! header that is put before the list of constructor/destructors. */
@@ -948,7 +954,7 @@ class TranslatorGerman : public Translator
     /*! Text for the \\invariant command */
     virtual QCString trInvariant()
     {
-      return "Invariant";
+      return "Invariante";
     }
 
     /*! Text shown before a multi-line variable/enum initialization */
@@ -995,7 +1001,7 @@ class TranslatorGerman : public Translator
 
     virtual QCString trPublicTypes()
     {
-      return "ffentliche Typen";
+      return "Öffentliche Typen";
     }
 
     virtual QCString trPublicAttribs()
@@ -1006,28 +1012,28 @@ class TranslatorGerman : public Translator
       }
       else
       {
-        return "ffentliche Attribute";
+        return "Öffentliche Attribute";
       }
     }
 
     virtual QCString trStaticPublicAttribs()
     {
-      return "Statische ffentliche Attribute";
+      return "Statische öffentliche Attribute";
     }
 
     virtual QCString trProtectedTypes()
     {
-      return "Geschtzte Typen";
+      return "Geschützte Typen";
     }
 
     virtual QCString trProtectedAttribs()
     {
-      return "Geschtzte Attribute";
+      return "Geschützte Attribute";
     }
 
     virtual QCString trStaticProtectedAttribs()
     {
-      return "Statische geschtzte Attribute";
+      return "Statische, geschützte Attribute";
     }
 
     virtual QCString trPrivateTypes()
@@ -1042,7 +1048,7 @@ class TranslatorGerman : public Translator
 
     virtual QCString trStaticPrivateAttribs()
     {
-      return "Statische private Attribute";
+      return "Statische, private Attribute";
     }
 
 //////////////////////////////////////////////////////////////////////////
@@ -1083,7 +1089,7 @@ class TranslatorGerman : public Translator
     virtual QCString trInclByDepGraph()
     {
       return "Dieser Graph zeigt, welche Datei direkt oder "
-             "indirekt diese Datei enthlt:";
+             "indirekt diese Datei enthält:";
     }
 
     virtual QCString trSince()
@@ -1098,7 +1104,7 @@ class TranslatorGerman : public Translator
     /*! title of the graph legend page */
     virtual QCString trLegendTitle()
     {
-      return "Erklrung des Graphen";
+      return "Erklärung des Graphen";
     }
 
     /*! page explaining how the dot graph's should be interpreted 
@@ -1107,21 +1113,21 @@ class TranslatorGerman : public Translator
     virtual QCString trLegendDocs()
     {
       return 
-        "Diese Seite erklrt die Interpretation der von doxygen "
+        "Diese Seite erklärt die Interpretation der von doxygen "
         "erzeugten Graphen.<p>\n"
         "Beispiel:\n"
         "\\code\n"
-        "/*! Wegen Verkrzung unsichtbare Klasse */\n"
+        "/*! Wegen Verkürzung unsichtbare Klasse */\n"
         "class Invisible { };\n\n"
-        "/*! Klasse verkrzt dargestellt, Vererbungsbeziehung ist versteckt */\n"
+        "/*! Klasse verkürzt dargestellt, Vererbungsbeziehung ist versteckt */\n"
         "class Truncated : public Invisible { };\n\n"
         "/* Nicht mit doxygen-Kommentaren dokumentierte Klasse */\n"
         "class Undocumented { };\n\n"
-        "/*! Mithilfe ffentlicher Vererbung vererbte Klasse */\n"
+        "/*! Mithilfe öffentlicher Vererbung vererbte Klasse */\n"
         "class PublicBase : public Truncated { };\n\n"
         "/*! Eine Template-Klasse */\n"
         "template<class T> class Templ { };\n\n"
-        "/*! Mithilfe geschtzter Vererbung vererbte Klasse */\n"
+        "/*! Mithilfe geschützter Vererbung vererbte Klasse */\n"
         "class ProtectedBase { };\n\n"
         "/*! Mithilfe privater Vererbung vererbte Klasse */\n"
         "class PrivateBase { };\n\n"
@@ -1139,34 +1145,37 @@ class TranslatorGerman : public Translator
         "};\n"
         "\\endcode\n"
         "Dies liefert den folgenden Graphen:"
-        "<p><center><img alt=\"\" src=\"graph_legend."+Config_getEnum("DOT_IMAGE_FORMAT")+"\"></center>\n"
+        "<p><center><img alt=\"\" src=\"graph_legend."+Config_getEnum("DOT_IMAGE_FORMAT")+"\"></center></p>\n"
         "<p>\n"
         "Die Rechtecke in obigem Graphen bedeuten:\n"
+        "</p>\n"
         "<ul>\n"
-        "<li>Ein grau geflltes Rechteck stellt die Struktur oder "
-        "Klasse dar, fr die der Graph erzeugt wurde.\n"
+        "<li>Ein grau gefülltes Rechteck stellt die Struktur oder "
+        "Klasse dar, für die der Graph erzeugt wurde.</li>\n"
         "<li>Ein Rechteck mit schwarzem Rahmen kennzeichnet eine dokumentierte "
-        "Struktur oder Klasse.\n"
+        "Struktur oder Klasse.</li>\n"
         "<li>Ein Rechteck mit grauem Rahmen kennzeichnet eine undokumentierte "
-        "Struktur oder Klasse.\n"
+        "Struktur oder Klasse.</li>\n"
         "<li>Ein Rechteck mit rotem Rahmen kennzeichnet eine dokumentierte "
-        "Struktur oder Klasse, fr die nicht alle Vererbungs-/"
-        "Enthaltenseinsbeziehungen dargestellt werden. Ein Graph wird gekrzt, "
-        "wenn er nicht in die angegebenen Schranken passt.\n"
+        "Struktur oder Klasse, für die nicht alle Vererbungs-/"
+        "Enthaltenseinsbeziehungen dargestellt werden. Ein Graph wird gekürzt, "
+        "wenn er nicht in die angegebenen Schranken passt.</li>\n"
         "</ul>\n"
+        "<p>\n"
         "Die Pfeile bedeuten:\n"
+        "</p>\n"
         "<ul>\n"
-        "<li>Ein dunkelblauer Pfeil stellt eine ffentliche Vererbungsbeziehung "
-        "zwischen zwei Klassen dar.\n"
-        "<li>Ein dunkelgrner Pfeil stellt geschtzte Vererbung dar.\n"
-        "<li>Ein dunkelroter Pfeil stellt private Vererbung dar.\n"
+        "<li>Ein dunkelblauer Pfeil stellt eine öffentliche Vererbungsbeziehung "
+        "zwischen zwei Klassen dar.</li>\n"
+        "<li>Ein dunkelgrüner Pfeil stellt geschützte Vererbung dar.</li>\n"
+        "<li>Ein dunkelroter Pfeil stellt private Vererbung dar.</li>\n"
         "<li>Ein gestrichelter violetter Pfeil bedeutet, dass eine Klasse in "
         "einer anderen enthalten ist oder von einer anderen benutzt wird. Am "
         "Pfeil stehen die Variable(n), mit deren Hilfe auf die Struktur oder "
-        "Klasse an der Pfeilspitze zugegriffen werden kann.\n"
-        "<li>Ein gestrichelter gelber Pfeil kennzeichnet eine Verknpfung "
-        "zwischen einer Template Instanz und der Template Klasse von welcher "
-        "es abstammt. Neben dem Pfeil sind die Template Parameter aufgefhrt.\n"
+        "Klasse an der Pfeilspitze zugegriffen werden kann.</li>\n"
+        "<li>Ein gestrichelter gelber Pfeil kennzeichnet eine Verknüpfung "
+        "zwischen einer Template-Instanz und der Template-Klasse von welcher "
+        "es abstammt. Neben dem Pfeil sind die Template-Parameter aufgeführt.</li>\n"
         "</ul>\n";
     }
 
@@ -1246,7 +1255,7 @@ class TranslatorGerman : public Translator
     /*! The description of the package index page */
     virtual QCString trPackageListDescription()
     {
-      return "Hier folgen die Pakete mit einer Kurzbeschreibung (wenn verfgbar):";
+      return "Hier folgen die Pakete mit einer Kurzbeschreibung (wenn verfügbar):";
     }
 
     /*! The link name in the Quick links header for each page */
@@ -1526,7 +1535,7 @@ class TranslatorGerman : public Translator
     /*! Put in front of the call graph for a function. */
     virtual QCString trCallGraph()
     {
-      return "Hier ist ein Graph der zeigt, was diese Funktion aufruft:";
+      return "Hier ist ein Graph, der zeigt, was diese Funktion aufruft:";
     }
 
 //////////////////////////////////////////////////////////////////////////
@@ -1617,7 +1626,7 @@ class TranslatorGerman : public Translator
      *  and the fact that it is sorted alphabetically per level
      */
     virtual QCString trDirDescription()
-    { return "Diese Verzeichnishierarchie ist -mit Einschrnkungen- "
+    { return "Diese Verzeichnishierarchie ist -mit Einschränkungen- "
 	     "alphabetisch sortiert:";
     }
 
@@ -1646,9 +1655,9 @@ class TranslatorGerman : public Translator
      */
     virtual QCString trOverloadText()
     {
-       return "Dies ist eine berladene Methode, die der Bequemlichkeit "
+       return "Dies ist eine überladene Methode, die der Bequemlichkeit "
               "wegen bereitgestellt wird. Sie unterscheidet sich von der "
-              "obigen Funktion nur in den Argumenten die sie untersttzt.";
+              "obigen Funktion nur in den Argumenten die sie unterstützt.";
     }
 
 //////////////////////////////////////////////////////////////////////////
@@ -1665,7 +1674,7 @@ class TranslatorGerman : public Translator
      *  of documentation blocks for enumeration values
      */
     virtual QCString trEnumerationValueDocumentation()
-    { return "Enumerator-Dokumentation"; }
+    { return "Dokumentation der Aufzählungswerte"; }
 
 //////////////////////////////////////////////////////////////////////////
 // new since 1.5.4 (mainly for Fortran)
@@ -1699,11 +1708,11 @@ class TranslatorGerman : public Translator
       result+=" mit Links ";
       if (!extractAll) 
       {
-         result+="zur Datenstruktur-Dokumentation fr jedes Element";
+         result+="zur Datenstruktur-Dokumentation für jedes Element";
       }
       else 
       {
-         result+="zu den Datentypen, zu denen sie gehren:";
+         result+="zu den Datentypen, zu denen sie gehören:";
       }
       return result;
     }
@@ -1792,11 +1801,11 @@ class TranslatorGerman : public Translator
       result+="Modulelemente mit Links ";
       if (extractAll) 
       {
-        result+="zur Moduldokumentation fr jedes Element:";
+        result+="zur Moduldokumentation für jedes Element:";
       }
       else 
       {
-        result+="zu den Modulen, zu denen sie gehren:";
+        result+="zu den Modulen, zu denen sie gehören:";
       }
       return result;
     }
@@ -1817,6 +1826,7 @@ class TranslatorGerman : public Translator
       if (!singular)  result+="e";
       return result; 
     }
+
     /*! This is put at the bottom of a module documentation page and is
      *  followed by a list of files that were used to generate the page.
      */
@@ -1824,7 +1834,7 @@ class TranslatorGerman : public Translator
         bool single)
     {
       // single is true implies a single file
-      QCString result=(QCString)"Die Dokumentation fr ";
+      QCString result=(QCString)"Die Dokumentation für ";
       switch(compType)
       {
         case ClassDef::Class:      result+="dieses Modul"; break;
@@ -1841,6 +1851,7 @@ class TranslatorGerman : public Translator
         result+=" wurde aus den folgenden Dateien erzeugt:";
       return result;
     }
+
     /*! This is used for translation of the word that will possibly
      *  be followed by a single name or by a list of names 
      *  of the category.
@@ -1851,6 +1862,7 @@ class TranslatorGerman : public Translator
       if (!singular)  result+="en";
       return result; 
     }
+
     /*! This is used for translation of the word that will possibly
      *  be followed by a single name or by a list of names 
      *  of the category.
@@ -1868,6 +1880,152 @@ class TranslatorGerman : public Translator
       return "Type Constraints";
     }
 
+//////////////////////////////////////////////////////////////////////////
+// new since 1.6.0 (mainly for the new search engine)
+//////////////////////////////////////////////////////////////////////////
+
+    /*! directory relation for \a name */
+    virtual QCString trDirRelation(const char *name)
+    {
+      return "Bezug " + QCString(name);
+    }
+
+    /*! Loading message shown when loading search results */
+    virtual QCString trLoading()
+    {
+      return "Lade ...";
+    }
+
+    /*! Label used for search results in the global namespace */
+    virtual QCString trGlobalNamespace()
+    {
+      return "Globaler Namensbereich";
+    }
+
+    /*! Message shown while searching */
+    virtual QCString trSearching()
+    {
+      return "Suche ...";
+    }
+
+    /*! Text shown when no search results are found */
+    virtual QCString trNoMatches()
+    {
+      return "Keine Treffer";
+    }
+
+//////////////////////////////////////////////////////////////////////////
+// new since 1.6.3 (missing items for the directory pages)
+//////////////////////////////////////////////////////////////////////////
+ 
+    /*! introduction text for the directory dependency graph */
+    virtual QCString trDirDependency(const char *name)
+    {
+      return (QCString)"Diagramm der Verzeichnisabhängigkeiten für "+name;
+    }
+ 
+    /*! when clicking a directory dependency label, a page with a
+     *  table is shown. The heading for the first column mentions the
+     *  source file that has a relation to another file.
+     */
+    virtual QCString trFileIn(const char *name)
+    {
+      return (QCString)"Datei in "+name;
+    }
+ 
+    /*! when clicking a directory dependency label, a page with a
+     *  table is shown. The heading for the second column mentions the
+     *  destination file that is included.
+     */
+    virtual QCString trIncludesFileIn(const char *name)
+    {
+      return (QCString)"Include-Dateien in "+name;
+    }
+ 
+    /** Compiles a date string.
+     *  @param year Year in 4 digits
+     *  @param month Month of the year: 1=January
+     *  @param day Day of the Month: 1..31
+     *  @param dayOfWeek Day of the week: 1=Monday..7=Sunday
+     *  @param hour Hour of the day: 0..23
+     *  @param minutes Minutes in the hour: 0..59
+     *  @param seconds Seconds within the minute: 0..59
+     *  @param includeTime Include time in the result string?
+     */
+    virtual QCString trDateTime(int year,int month,int day,int dayOfWeek,
+                                int hour,int minutes,int seconds,
+                                bool includeTime)
+    {
+      static const char *days[]   = { "Mon","Die","Mit","Don","Fre","Sam","Son" };
+      static const char *months[] = { "Jan","Feb","Mär","Apr","Mai","Jun","Jul","Aug","Sep","Okt","Nov","Dez" };
+      QCString sdate;
+      sdate.sprintf("%s %s %d %d",days[dayOfWeek-1],months[month-1],day,year);
+      if (includeTime)
+      {
+        QCString stime;
+        stime.sprintf(" %.2d:%.2d:%.2d",hour,minutes,seconds);
+        sdate+=stime;
+      }
+      return sdate;
+    }
+
+//////////////////////////////////////////////////////////////////////////
+// new since 1.7.5
+//////////////////////////////////////////////////////////////////////////
+
+    /*! Header for the page with bibliographic citations */
+    virtual QCString trCiteReferences()
+    { return "Literaturverzeichnis"; }
+
+    /*! Text for copyright paragraph */
+    virtual QCString trCopyright()
+    { return "Copyright"; }
+
+    /*! Header for the graph showing the directory dependencies */
+    virtual QCString trDirDepGraph(const char *name)
+    { return QCString("Diagramm der Verzeichnisabhängigkeiten für ")+name+":"; }
+
+//////////////////////////////////////////////////////////////////////////
+// new since 1.8.0
+//////////////////////////////////////////////////////////////////////////
+
+    /*! Detail level selector shown for hierarchical indices */
+    virtual QCString trDetailLevel()
+    { return "Detailebene"; }
+
+    /*! Section header for list of template parameters */
+    virtual QCString trTemplateParameters()
+    { return "Template-Parameter"; }
+
+    /*! Used in dot graph when UML_LOOK is enabled and there are many fields */
+    virtual QCString trAndMore(const QCString &number)
+    { return "und "+number+" mehr ..."; }
+
+    /*! Used file list for a Java enum */
+    virtual QCString trEnumGeneratedFromFiles(bool single)
+    { QCString result = "Die Dokumentation für diesen enum wurde aus ";
+      if (single)
+		result += "der folgenden Datei";
+	  else
+		result += "den folgenden Dateien";
+      result+=" generiert:";
+      return result;
+    }
+
+    /*! Header of a Java enum page (Java enums are represented as classes). */
+    virtual QCString trEnumReference(const char *name)
+    { return QCString(name)+" Enum-Referenz"; }
+
+    /*! Used for a section containing inherited members */
+    virtual QCString trInheritedFrom(const char *members,const char *what)
+    { return QCString(members)+" geerbt von "+what; }
+
+    /*! Header of the sections with inherited members specific for the 
+     *  base class(es) 
+     */
+    virtual QCString trAdditionalInheritedMembers()
+    { return "Weitere Geerbte Elemente"; }
+
 };
 
 #endif
