@@ -2,7 +2,7 @@
  *
  * $Id: translator_nl.h,v 1.22 2001/03/19 19:27:42 root Exp $
  *
- * Copyright (C) 1997-2010 by Dimitri van Heesch.
+ * Copyright (C) 1997-2012 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -33,9 +33,6 @@ class TranslatorDutch : public Translator
      *  "\\usepackage[latin2]{inputenc}\n"
      *  "\\usepackage[T1]{fontenc}\n"
      *  </pre>
-     * 
-     * The Dutch LaTeX does not use such commands.  Because of this
-     * the empty string is returned in this implementation.
      */
     QCString latexLanguageSupportCommand()
     {
@@ -64,7 +61,7 @@ class TranslatorDutch : public Translator
     QCString trMore()
     { return "Meer..."; }
     QCString trListOfAllMembers()
-    { return "Lijst van alle members."; }
+    { return "Lijst van alle members"; }
     QCString trMemberList()
     { return "Member Lijst"; }
     QCString trThisIsTheListOfAllMembers()
@@ -90,11 +87,11 @@ class TranslatorDutch : public Translator
     QCString trCompoundList()
     { return "Klasse Lijst"; }
     QCString trFileList()
-    { return "File Lijst"; }
+    { return "Bestandslijst"; }
     QCString trCompoundMembers()
     { return "Klasse Members"; }
     QCString trFileMembers()
-    { return "File members"; }
+    { return "Bestand members"; }
     QCString trRelatedPages()
     { return "Gerelateerde pagina's"; }
     QCString trExamples()
@@ -150,13 +147,13 @@ class TranslatorDutch : public Translator
     QCString trCompoundIndex()
     { return "Klasse Index"; }
     QCString trFileIndex() 
-    { return "File Index"; }
+    { return "Bestand Index"; }
     QCString trModuleDocumentation()
     { return "Module Documentatie"; }
     QCString trClassDocumentation()
     { return "Klassen Documentatie"; }
     QCString trFileDocumentation()
-    { return "File Documentatie"; }
+    { return "Bestand Documentatie"; }
     QCString trExampleDocumentation()
     { return "Documentatie van voorbeelden"; }
     QCString trPageDocumentation()
@@ -165,7 +162,7 @@ class TranslatorDutch : public Translator
     { return "Naslagwerk"; }
 
     QCString trDefines()
-    { return "Defines"; }
+    { return "Macros"; }
     QCString trFuncProtos()
     { return "Functie Prototypes"; }
     QCString trTypedefs()
@@ -179,7 +176,7 @@ class TranslatorDutch : public Translator
     QCString trEnumerationValues()
     { return "Enumeratie waarden"; }
     QCString trDefineDocumentation()
-    { return "Documentatie van defines"; }
+    { return "Documentatie van macro's"; }
     QCString trFunctionPrototypeDocumentation()
     { return "Documentatie van functie Prototypes"; }
     QCString trTypedefDocumentation()
@@ -420,8 +417,10 @@ class TranslatorDutch : public Translator
         case ClassDef::Category:   result+="deze categorie"; break;
         case ClassDef::Exception:  result+="deze exceptie"; break;
       }
-      result+=" is gegenereerd op grond van het volgende bestand";
-      if (single) result+=":"; else result+="s:";
+      result+=" is gegenereerd op grond van ";
+      if (single) result+="het"; else result+="de";
+      result+=" volgende bestand";
+      if (single) result+=":"; else result+="en:";
       return result;
     }
 
@@ -1127,25 +1126,25 @@ class TranslatorDutch : public Translator
      *  hierarchy.
      */
     virtual QCString trDirIndex()
-    { return "Directory Hi&euml;rarchie"; }
+    { return "Folder Hi&euml;rarchie"; }
 
     /*! This is used as the name of the chapter containing the documentation
      *  of the directories.
      */
     virtual QCString trDirDocumentation()
-    { return "Directory Documentatie"; }
+    { return "Folder Documentatie"; }
 
     /*! This is used as the title of the directory index and also in the
      *  Quick links of a HTML page, to link to the directory hierarchy.
      */
     virtual QCString trDirectories()
-    { return "Directories"; }
+    { return "Folders"; }
 
     /*! This returns a sentences that introduces the directory hierarchy. 
      *  and the fact that it is sorted alphabetically per level
      */
     virtual QCString trDirDescription()
-    { return "Deze directory hi&euml;rarchie is min of meer alfabetisch "
+    { return "Deze folder hi&euml;rarchie is min of meer alfabetisch "
              "gesorteerd:";
     }
 
@@ -1153,15 +1152,15 @@ class TranslatorDutch : public Translator
      *  directory is passed via \a dirName.
      */
     virtual QCString trDirReference(const char *dirName)
-    { QCString result=dirName; result+=" Directory Referentie"; return result; }
+    { QCString result=dirName; result+=" Folder Referentie"; return result; }
 
     /*! This returns the word directory with or without starting capital
      *  (\a first_capital) and in sigular or plural form (\a singular).
      */
     virtual QCString trDir(bool first_capital, bool singular)
     { 
-      QCString result((first_capital ? "Director" : "director"));
-      if (singular) result+="y"; else result+="ies";
+      QCString result((first_capital ? "Folder" : "folder"));
+      if (!singular) result+="s";
       return result; 
     }
 
@@ -1191,7 +1190,7 @@ class TranslatorDutch : public Translator
     
     /*! header that is put before the list of member subprograms (Fortran). */
     virtual QCString trMemberFunctionDocumentationFortran()
-    { return "Member Function/Subroutine Documentation"; }
+    { return "Member Functie/Subroutine Documentatie"; }
 
 //////////////////////////////////////////////////////////////////////////
 // new since 1.5.4 (mainly for Fortran)
@@ -1342,8 +1341,10 @@ class TranslatorDutch : public Translator
         case ClassDef::Category:   result+="deze category"; break;
         case ClassDef::Exception:  result+="deze exception"; break;
       }
-      result+=" is gegenereerd op grond van het volgende bestand";
-      if (single) result+=":"; else result+="s:";
+      result+=" is gegenereerd op grond van ";
+      if (single) result+="het"; else result+="de";
+      result+=" volgende bestand";
+      if (single) result+=":"; else result+="en:";
       return result;
     }
     /*! This is used for translation of the word that will possibly
@@ -1407,8 +1408,167 @@ class TranslatorDutch : public Translator
       return "Niets gevonden";
     }
 
+//////////////////////////////////////////////////////////////////////////
+// new since 1.6.3 (missing items for the directory pages)
+//////////////////////////////////////////////////////////////////////////
 
+    /*! introduction text for the directory dependency graph */
+    virtual QCString trDirDependency(const char *name)
+    {
+      return (QCString)"Folder afhankelijkheidsgraaf voor "+name;
+    }
 
+    /*! when clicking a directory dependency label, a page with a
+     *  table is shown. The heading for the first column mentions the
+     *  source file that has a relation to another file.
+     */
+    virtual QCString trFileIn(const char *name)
+    {
+      return (QCString)"Bestand in "+name;
+    }
+
+    /*! when clicking a directory dependency label, a page with a
+     *  table is shown. The heading for the second column mentions the
+     *  destination file that is included.
+     */
+    virtual QCString trIncludesFileIn(const char *name)
+    {
+      return (QCString)"Includeert bestand in "+name;
+    }
+    virtual QCString trDateTime(int year,int month,int day,int dayOfWeek,
+                                int hour,int minutes,int seconds,
+                                bool includeTime)
+    {
+      static const char *days[]   = { "Ma","Di","Wo","Do","Vr","Za","Zo" };
+      static const char *months[] = { "Jan","Feb","Maa","Apr","Mei","Jun","Jul","Aug","Sep","Okt","Nov","Dec" };
+      QCString sdate;
+      sdate.sprintf("%s %d %s %d",days[dayOfWeek-1],day,months[month-1],year);
+      if (includeTime)
+      {
+        QCString stime;
+        stime.sprintf(" %.2d:%.2d:%.2d",hour,minutes,seconds);
+        sdate+=stime;
+      }
+      return sdate;
+    }
+
+//////////////////////////////////////////////////////////////////////////
+// new since 1.7.5
+//////////////////////////////////////////////////////////////////////////
+
+    /*! Header for the page with bibliographic citations */
+    virtual QCString trCiteReferences()
+    { return "Bibliografie"; }
+
+    /*! Text for copyright paragraph */
+    virtual QCString trCopyright()
+    { return "Copyright"; }
+
+    /*! Header for the graph showing the directory dependencies */
+    virtual QCString trDirDepGraph(const char *name)
+    { return QCString("Folder afhankelijkheidsgraaf voor ")+name+":"; }
+
+//////////////////////////////////////////////////////////////////////////
+// new since 1.8.0
+//////////////////////////////////////////////////////////////////////////
+
+    /*! Detail level selector shown for hierarchical indices */
+    virtual QCString trDetailLevel()
+    { return "detail niveau"; }
+
+    /*! Section header for list of template parameters */
+    virtual QCString trTemplateParameters()
+    { return "Template Parameters"; }
+
+    /*! Used in dot graph when UML_LOOK is enabled and there are many fields */
+    virtual QCString trAndMore(const QCString &number)
+    { return "en "+number+ " anderen..."; }
+
+    /*! Used file list for a Java enum */
+    virtual QCString trEnumGeneratedFromFiles(bool single)
+    { QCString result = "De documentatie voor deze enum is gegenereerd op grond van ";
+      if (single) result+="het"; else result+="de";
+      result+=" volgende bestand";
+      if (single) result+=":"; else result+="en:";
+      return result;
+    }
+
+    /*! Header of a Java enum page (Java enums are represented as classes). */
+    virtual QCString trEnumReference(const char *name)
+    { return QCString(name)+" Enum Referentie"; }
+
+    /*! Used for a section containing inherited members */
+    virtual QCString trInheritedFrom(const char *members,const char *what)
+    { return QCString(members)+" overge&euml;rfd van "+what; }
+
+    /*! Header of the sections with inherited members specific for the 
+     *  base class(es) 
+     */
+    virtual QCString trAdditionalInheritedMembers()
+    { return "Additionele Overge&euml;rfde Members"; }
+
+//////////////////////////////////////////////////////////////////////////
+// new since 1.8.2
+//////////////////////////////////////////////////////////////////////////
+
+    /*! Used as a tooltip for the toggle button that appears in the
+     *  navigation tree in the HTML output when GENERATE_TREEVIEW is 
+     *  enabled. This tooltip explains the meaning of the button.
+     */
+    virtual QCString trPanelSynchronisationTooltip(bool enable)
+    {
+      QCString opt = enable ? "in" : "uit";
+      return "klik hier de paneel synchronisatie "+opt+" the schakelen";
+    }
+
+    /*! Used in a method of an Objective-C class that is declared in a
+     *  a category. Note that the @1 marker is required and is replaced
+     *  by a link.
+     */
+    virtual QCString trProvidedByCategory()
+    {
+      return "Wordt aangeboden door category @1.";
+    }
+
+    /*! Used in a method of an Objective-C category that extends a class.
+     *  Note that the @1 marker is required and is replaced by a link to
+     *  the class method.
+     */
+    virtual QCString trExtendsClass()
+    {
+      return "Uitbereiding van klasse @1.";
+    }
+
+    /*! Used as the header of a list of class methods in Objective-C.
+     *  These are similar to static public member functions in C++.
+     */
+    virtual QCString trClassMethods()
+    {
+      return "Klasse Methoden";
+    }
+
+    /*! Used as the header of a list of instance methods in Objective-C.
+     *  These are similar to public member functions in C++.
+     */
+    virtual QCString trInstanceMethods()
+    {
+      return "Instantie Methoden";
+    }
+
+    /*! Used as the header of the member functions of an Objective-C class.
+     */
+    virtual QCString trMethodDocumentation()
+    {
+      return "Methode Documentatie";
+    }
+
+    /*! Used as the title of the design overview picture created for the
+     *  VHDL output.
+     */
+    virtual QCString trDesignOverview()
+    {
+      return "Ontwerp Overzicht";
+    }
 };
 
 #endif
