@@ -1,6 +1,6 @@
 /******************************************************************************
  *
- * Copyright (C) 1997-2011 by Dimitri van Heesch.
+ * Copyright (C) 1997-2012 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -32,6 +32,7 @@
 #include "docparser.h"
 #include "language.h"
 #include "parserintf.h"
+#include "arguments.h"
 
 #include <qdir.h>
 #include <qfile.h>
@@ -59,6 +60,7 @@ static const char compound_xsd[] =
 
 //------------------
 
+/** Helper class mapping MemberList::ListType to a string representing */
 class XmlSectionMapper : public QIntDict<char>
 {
   public:
@@ -119,8 +121,8 @@ inline void writeXMLCodeString(FTextStream &t,const char *s, int &col)
     {
       case '\t': 
       { 
-	int spacesToNextTabStop = 
-          Config_getInt("TAB_SIZE") - (col%Config_getInt("TAB_SIZE")); 
+        static int tabSize = Config_getInt("TAB_SIZE");
+	int spacesToNextTabStop = tabSize - (col%tabSize); 
 	col+=spacesToNextTabStop;
 	while (spacesToNextTabStop--) t << "<sp/>";
 	break;
@@ -192,6 +194,7 @@ void writeXMLLink(FTextStream &t,const char *extRef,const char *compoundId,
   t << "</ref>";
 }
 
+/** Implements TextGeneratorIntf for an XML stream. */
 class TextGeneratorXMLImpl : public TextGeneratorIntf
 {
   public:
@@ -211,6 +214,7 @@ class TextGeneratorXMLImpl : public TextGeneratorIntf
     FTextStream &m_t;
 };
 
+/** Helper class representing a stack of objects stored by value */
 template<class T> class ValStack
 {
   public:
@@ -264,7 +268,7 @@ template<class T> class ValStack
     int m_size;
 };
 
-
+/** Generator for producing XML formatted source code. */
 class XMLCodeGenerator : public CodeOutputInterface
 {
   public:
@@ -297,7 +301,7 @@ class XMLCodeGenerator : public CodeOutputInterface
       writeXMLLink(m_t,ref,file,anchor,name,tooltip);
       col+=strlen(name);
     }
-    void startCodeLine() 
+    void startCodeLine(bool) 
     {
       XML_DB(("(startCodeLine)\n"));
       m_t << "<codeline";
@@ -495,12 +499,18 @@ void writeXMLCodeBlock(FTextStream &t,FileDef *fd)
   ParserInterface *pIntf=Doxygen::parserManager->getParser(fd->getDefFileExtension());
   pIntf->resetCodeParserState();
   XMLCodeGenerator *xmlGen = new XMLCodeGenerator(t);
-  pIntf->parseCode(*xmlGen,
-                0,
+  pIntf->parseCode(*xmlGen,  // codeOutIntf
+                0,           // scopeName
                 fileToString(fd->absFilePath(),Config_getBool("FILTER_SOURCE_FILES")),
-                FALSE,
-                0,
-                fd);
+                FALSE,       // isExampleBlock
+                0,           // exampleName
+                fd,          // fileDef
+                -1,          // startLine
+                -1,          // endLine
+                FALSE,       // inlineFragement
+                0,           // memberDef
+                TRUE         // showLineNumbers
+                );
   xmlGen->finish();
   delete xmlGen;
 }
@@ -543,20 +553,21 @@ static void stripQualifiers(QCString &typeStr)
 
 static QCString classOutputFileBase(ClassDef *cd)
 {
-  static bool inlineGroupedClasses = Config_getBool("INLINE_GROUPED_CLASSES");
-  if (inlineGroupedClasses && cd->partOfGroups()!=0) 
-    return cd->getXmlOutputFileBase();
-  else 
-    return cd->getOutputFileBase();
+  //static bool inlineGroupedClasses = Config_getBool("INLINE_GROUPED_CLASSES");
+  //if (inlineGroupedClasses && cd->partOfGroups()!=0) 
+  return cd->getOutputFileBase();
+  //else 
+  //  return cd->getOutputFileBase();
 }
 
 static QCString memberOutputFileBase(MemberDef *md)
 {
-  static bool inlineGroupedClasses = Config_getBool("INLINE_GROUPED_CLASSES");
-  if (inlineGroupedClasses && md->getClassDef() && md->getClassDef()->partOfGroups()!=0) 
-    return md->getClassDef()->getXmlOutputFileBase();
-  else 
-    return md->getOutputFileBase();
+  //static bool inlineGroupedClasses = Config_getBool("INLINE_GROUPED_CLASSES");
+  //if (inlineGroupedClasses && md->getClassDef() && md->getClassDef()->partOfGroups()!=0) 
+  //  return md->getClassDef()->getXmlOutputFileBase();
+  //else 
+  //  return md->getOutputFileBase();
+  return md->getOutputFileBase();
 }
 
 
@@ -725,12 +736,14 @@ static void generateXMLForMember(MemberDef *md,FTextStream &ti,FTextStream &t,De
     if (md->isSettable()) t << "yes"; else t << "no";
     t << "\"";
 
-    if (md->isAssign() || md->isCopy() || md->isRetain())
+    if (md->isAssign() || md->isCopy() || md->isRetain() || md->isStrong() || md->isWeak())
     {
       t << " accessor=\"";
-      if (md->isAssign()) t << "assign";
-      else if (md->isCopy()) t << "copy";
+      if (md->isAssign())      t << "assign";
+      else if (md->isCopy())   t << "copy";
       else if (md->isRetain()) t << "retain";
+      else if (md->isStrong()) t << "strong";
+      else if (md->isWeak())   t << "weak";
       t << "\"";
     }
   }
@@ -762,7 +775,7 @@ static void generateXMLForMember(MemberDef *md,FTextStream &ti,FTextStream &t,De
     QCString typeStr = md->typeString(); //replaceAnonymousScopes(md->typeString());
     stripQualifiers(typeStr);
     t << "        <type>";
-    linkifyText(TextGeneratorXMLImpl(t),def,md->getBodyDef(),md->name(),typeStr);
+    linkifyText(TextGeneratorXMLImpl(t),def,md->getBodyDef(),md,typeStr);
     t << "</type>" << endl;
     t << "        <definition>" << convertToXML(md->definition()) << "</definition>" << endl;
     t << "        <argsstring>" << convertToXML(md->argsString()) << "</argsstring>" << endl;
@@ -825,7 +838,7 @@ static void generateXMLForMember(MemberDef *md,FTextStream &ti,FTextStream &t,De
         if (!a->type.isEmpty())
         {
           t << "          <type>";
-          linkifyText(TextGeneratorXMLImpl(t),def,md->getBodyDef(),md->name(),a->type);
+          linkifyText(TextGeneratorXMLImpl(t),def,md->getBodyDef(),md,a->type);
           t << "</type>" << endl;
         }
         if (!a->name.isEmpty())
@@ -849,7 +862,7 @@ static void generateXMLForMember(MemberDef *md,FTextStream &ti,FTextStream &t,De
         if (!a->defval.isEmpty())
         {
           t << "          <defval>";
-          linkifyText(TextGeneratorXMLImpl(t),def,md->getBodyDef(),md->name(),a->defval);
+          linkifyText(TextGeneratorXMLImpl(t),def,md->getBodyDef(),md,a->defval);
           t << "</defval>" << endl;
         }
         if (defArg && defArg->hasDocumentation())
@@ -887,14 +900,14 @@ static void generateXMLForMember(MemberDef *md,FTextStream &ti,FTextStream &t,De
   if (!md->initializer().isEmpty() && md->initializer().length()<2000)
   {
     t << "        <initializer>";
-    linkifyText(TextGeneratorXMLImpl(t),def,md->getBodyDef(),md->name(),md->initializer());
+    linkifyText(TextGeneratorXMLImpl(t),def,md->getBodyDef(),md,md->initializer());
     t << "</initializer>" << endl;
   }
 
   if (md->excpString())
   {
     t << "        <exceptions>";
-    linkifyText(TextGeneratorXMLImpl(t),def,md->getBodyDef(),md->name(),md->excpString());
+    linkifyText(TextGeneratorXMLImpl(t),def,md->getBodyDef(),md,md->excpString());
     t << "</exceptions>" << endl;
   }
   
@@ -1092,7 +1105,7 @@ static void writeInnerClasses(const ClassSDict *cl,FTextStream &t)
     {
       if (!cd->isHidden() && cd->name().find('@')==-1) // skip anonymous scopes
       {
-        t << "    <innerclass refid=\"" << cd->getOutputFileBase()
+        t << "    <innerclass refid=\"" << classOutputFileBase(cd)
           << "\" prot=\"";
         switch(cd->protection())
         {
@@ -1207,6 +1220,7 @@ static void generateXMLForClass(ClassDef *cd,FTextStream &ti)
   if (cd->isHidden())           return; // skip hidden classes.
   if (cd->name().find('@')!=-1) return; // skip anonymous compounds.
   if (cd->templateMaster()!=0)  return; // skip generated template instances.
+  if (cd->isArtificial())       return; // skip artificially created classes
 
   msg("Generating XML output for class %s\n",cd->name().data());
 
@@ -1236,6 +1250,9 @@ static void generateXMLForClass(ClassDef *cd,FTextStream &ti)
     case Private:   t << "private";   break;
     case Package:   t << "package";   break;
   }
+  if (cd->isFinal()) t << "\" final=\"yes";
+  if (cd->isSealed()) t << "\" sealed=\"yes";
+  if (cd->isAbstract()) t << "\" abstract=\"yes";
   t << "\">" << endl;
   t << "    <compoundname>"; 
   writeXMLString(t,cd->name()); 
@@ -1327,7 +1344,7 @@ static void generateXMLForClass(ClassDef *cd,FTextStream &ti)
     }
   }
 
-  writeInnerClasses(cd->getInnerClasses(),t);
+  writeInnerClasses(cd->getClassSDict(),t);
 
   writeTemplateList(cd,t);
   if (cd->getMemberGroupSDict())
@@ -1850,7 +1867,7 @@ void generateXML()
   QCString outputDirectory = Config_getString("XML_OUTPUT");
   if (outputDirectory.isEmpty())
   {
-    outputDirectory=QDir::currentDirPath();
+    outputDirectory=QDir::currentDirPath().utf8();
   }
   else
   {
@@ -1871,7 +1888,7 @@ void generateXML()
       }
       dir.cd(outputDirectory);
     }
-    outputDirectory=dir.absPath();
+    outputDirectory=dir.absPath().utf8();
   }
 
   QDir dir(outputDirectory);
